@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qvaluelistiterator.cpp   3.0.5   edited Jun 25 11:41 $
+** $Id: qt/qvaluelistiterator.cpp   3.1.2   edited Nov 8 10:35 $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -27,9 +27,7 @@ public:
     void setSalary( int salary ) { s = salary; }
 
     // this is here to support very old compilers
-#if defined(Q_FULL_TEMPLATE_INSTANTIATION)
-    bool operator==( const Employee & ) const { return FALSE; }
-#endif
+    Q_DUMMY_COMPARISON_OPERATOR( Employee )
 
 private:
     QString n;
