@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/pbuilder_pbx.h   3.0.5   edited Jun 22 01:20 $
+** $Id: qt/pbuilder_pbx.h   3.1.2   edited Nov 8 10:35 $
 **
 ** Definition of ________ class.
 **
 ** Created : 970521
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the network module of the Qt GUI Toolkit.
 **
@@ -34,27 +34,29 @@
 ** not clear to you.
 **
 **********************************************************************/
-#ifndef __PBUILDERMAKE_H__
-#define __PBUILDERMAKE_H__
+#ifndef __PBUILDER_PBX_H__
+#define __PBUILDER_PBX_H__
 
 #include "unixmake.h"
 
 class ProjectBuilderMakefileGenerator : public UnixMakefileGenerator
 {
+    QString pbx_dir;
     int pbuilderVersion() const;
     bool writeMakeParts(QTextStream &);
     bool writeMakefile(QTextStream &);
 
     QMap<QString, QString> keys;
-    QString keyFor(QString file);
-    QString fixEnvs(QString file);
-    QString fixEnvsList(QString where);
+    QString keyFor(const QString &file);
+    QString fixEnvs(const QString &file);
+    QString fixEnvsList(const QString &where);
+    QString reftypeForFile(const QString &where);
 
 public:
     ProjectBuilderMakefileGenerator(QMakeProject *p);
     ~ProjectBuilderMakefileGenerator();
 
-    QString defaultMakefile() const;
+    virtual bool openOutput(QFile &) const;
 protected:
     virtual bool doDepends() const { return FALSE; } //never necesary
 };
@@ -63,4 +65,4 @@ inline ProjectBuilderMakefileGenerator::~ProjectBuilderMakefileGenerator()
 { }
 
 
-#endif /* __PBUILDERMAKE_H__ */
+#endif /* __PBUILDER_PBX_H__ */
