@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/makefile.h   3.1.2   edited Feb 5 02:00 $
 **
 ** Definition of ________ class.
 **
 ** Created : 970521
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the network module of the Qt GUI Toolkit.
 **
@@ -44,17 +44,17 @@
 class MakefileGenerator
 {
     QString spec;
-    bool init_opath_already, init_already, moc_aware;
+    bool init_opath_already, init_already, moc_aware, no_io;
     QStringList createObjectList(const QString &var);
     QString build_args();
-    QMap<QString, QString> depHeuristics, depKeyMap;
+    QMap<QString, QString> depHeuristics, depKeyMap, fileFixed;
     QMap<QString, QString> mocablesToMOC, mocablesFromMOC;
     QMap<QString, QStringList> depends;
 
 protected:
     void writeObj(QTextStream &, const QString &obj, const QString &src);
     void writeUicSrc(QTextStream &, const QString &ui);
-    void writeMocObj(QTextStream &, const QString &obj);
+    void writeMocObj(QTextStream &, const QString &obj, const QString &src);
     void writeMocSrc(QTextStream &, const QString &src);
     void writeLexSrc(QTextStream &, const QString &lex);
     void writeYaccSrc(QTextStream &, const QString &yac);
@@ -68,21 +68,24 @@ protected:
 
     class MakefileDependDir {
     public:
-	MakefileDependDir(QString r, QString l) : real_dir(r), local_dir(l) { }
+	MakefileDependDir(const QString &r, const QString &l) : real_dir(r), local_dir(l) { }
 	QString real_dir, local_dir;
     };
-    bool generateDependencies(QPtrList<MakefileDependDir> &dirs, QString x, bool recurse);
+    bool generateDependencies(QPtrList<MakefileDependDir> &dirs, const QString &x, bool recurse);
 
     QString buildArgs();
 
     QString specdir();
     QString cleanFilePath(const QString &file) const;
-    bool generateMocList(QString fn);
+    bool generateMocList(const QString &fn);
 
     QString findMocSource(const QString &moc_file) const;
     QString findMocDestination(const QString &src_file) const;
     QStringList &findDependencies(const QString &file);
 
+    void setNoIO(bool o);
+    bool noIO() const;
+
     void setMocAware(bool o);
     bool mocAware() const;
     void logicWarn(const QString &, const QString &);
@@ -114,22 +117,19 @@ protected:
     QString valList(const QStringList &varList);
 
 
-    bool fileFixify(QString &file, const QString &dir=QString::null) const;
-    bool fileFixify(QStringList &files, const QString &dir=QString::null) const;
+    QString fileFixify(const QString& file, const QString &out_dir=QString::null, 
+		    const QString &in_dir=QString::null, bool force_fix=FALSE) const;
+    QStringList fileFixify(const QStringList& files, const QString &out_dir=QString::null, 
+		    const QString &in_dir=QString::null, bool force_fix=FALSE) const;
 public:
     MakefileGenerator(QMakeProject *p);
     virtual ~MakefileGenerator();
 
     static MakefileGenerator *create(QMakeProject *);
     bool write();
-    virtual QString defaultMakefile() const;
+    virtual bool openOutput(QFile &) const;
 };
 
-inline QString MakefileGenerator::defaultMakefile() const
-{
-    return "Makefile";
-}
-
 inline QString MakefileGenerator::findMocSource(const QString &moc_file) const
 {
     QString tmp = cleanFilePath(moc_file);
@@ -154,6 +154,12 @@ inline void MakefileGenerator::setMocAware(bool o)
 inline bool MakefileGenerator::mocAware() const
 { return moc_aware; }
 
+inline void MakefileGenerator::setNoIO(bool o)
+{ no_io = o; }
+
+inline bool MakefileGenerator::noIO() const
+{ return no_io; }
+
 inline QString MakefileGenerator::defaultInstall(const QString &)
 { return QString(""); }
 
