@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/unixmake.cpp   3.1.2   edited Jan 23 23:17 $
 **
 ** Definition of ________ class.
 **
 ** Created : 970521
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the network module of the Qt GUI Toolkit.
 **
@@ -71,12 +71,31 @@ UnixMakefileGenerator::init()
 	return; /* subdirs is done */
     }
 
-    if( project->isEmpty("QMAKE_EXTENSION_SHLIB") )
-	project->variables()["QMAKE_EXTENSION_SHLIB"].append( "so" );
+    if( project->isEmpty("QMAKE_EXTENSION_SHLIB") ) {
+	if ( project->isEmpty("QMAKE_CYGWIN_SHLIB") ) {
+	    project->variables()["QMAKE_EXTENSION_SHLIB"].append( "so" );
+	} else {
+	    project->variables()["QMAKE_EXTENSION_SHLIB"].append( "dll" );
+	}
+    }
     if( project->isEmpty("QMAKE_COPY_FILE") )
 	project->variables()["QMAKE_COPY_FILE"].append( "$(COPY) -p" );
     if( project->isEmpty("QMAKE_COPY_DIR") )
 	project->variables()["QMAKE_COPY_DIR"].append( "$(COPY) -pR" );
+    //If the TARGET looks like a path split it into DESTDIR and the resulting TARGET
+    if(!project->isEmpty("TARGET")) {
+	QString targ = project->first("TARGET");
+	int slsh = QMAX(targ.findRev('/'), targ.findRev(Option::dir_sep));
+	if(slsh != -1) {
+	    if(project->isEmpty("DESTDIR"))
+		project->values("DESTDIR").append("");
+	    else if(project->first("DESTDIR").right(1) != Option::dir_sep)
+		project->variables()["DESTDIR"] = project->first("DESTDIR") + Option::dir_sep;
+	    project->variables()["DESTDIR"] = project->first("DESTDIR") + targ.left(slsh+1);
+	    project->variables()["TARGET"] = targ.mid(slsh+1);
+	}
+    }
+
     project->variables()["QMAKE_ORIG_TARGET"] = project->variables()["TARGET"];
 
     bool is_qt = (project->first("TARGET") == "qt" || project->first("TARGET") == "qte" ||
@@ -103,7 +122,7 @@ UnixMakefileGenerator::init()
 	project->variables()["QMAKE_CFLAGS"] += project->variables()["QMAKE_CFLAGS_DEBUG"];
 	project->variables()["QMAKE_CXXFLAGS"] += project->variables()["QMAKE_CXXFLAGS_DEBUG"];
 	project->variables()["QMAKE_LFLAGS"] += project->variables()["QMAKE_LFLAGS_DEBUG"];
-    } else if ( project->isActiveConfig("release") ) {
+    } else {
 	project->variables()["QMAKE_CFLAGS"] += project->variables()["QMAKE_CFLAGS_RELEASE"];
 	project->variables()["QMAKE_CXXFLAGS"] += project->variables()["QMAKE_CXXFLAGS_RELEASE"];
 	project->variables()["QMAKE_LFLAGS"] += project->variables()["QMAKE_LFLAGS_RELEASE"];
@@ -116,8 +135,12 @@ UnixMakefileGenerator::init()
 	project->variables()["QMAKE_LFLAGS"] += project->variables()["QMAKE_LFLAGS_PREBIND"];
     if(!project->isEmpty("QMAKE_INCDIR"))
 	project->variables()["INCLUDEPATH"] += project->variables()["QMAKE_INCDIR"];
-    if(!project->isEmpty("QMAKE_LIBDIR"))
-	project->variables()["QMAKE_LIBDIR_FLAGS"] += varGlue( "QMAKE_LIBDIR", " -L", " -L", "" );
+    if(!project->isEmpty("QMAKE_LIBDIR")) {
+	if ( !project->isEmpty("QMAKE_RPATH") )
+	    project->variables()["QMAKE_LIBDIR_FLAGS"] += varGlue("QMAKE_LIBDIR", " " + var("QMAKE_RPATH"),
+								  " " + var("QMAKE_RPATH"), "");
+	project->variables()["QMAKE_LIBDIR_FLAGS"] += varGlue( "QMAKE_LIBDIR", "-L", " -L", "" );
+    }
     if ( extern_libs && (project->isActiveConfig("qt") || project->isActiveConfig("opengl")) ) {
 	if(configs.findIndex("x11lib") == -1)
 	    configs.append("x11lib");
@@ -142,9 +165,9 @@ UnixMakefileGenerator::init()
 	if ( !is_qt ) {
 	    if ( !project->isEmpty("QMAKE_LIBDIR_QT") ) {
 		if ( !project->isEmpty("QMAKE_RPATH") )
-		    project->variables()["QMAKE_LIBDIR_FLAGS"] += varGlue("QMAKE_LIBDIR_QT", " " + var("QMAKE_RPATH"), 
+		    project->variables()["QMAKE_LIBDIR_FLAGS"] += varGlue("QMAKE_LIBDIR_QT", " " + var("QMAKE_RPATH"),
 									  " " + var("QMAKE_RPATH"), "");
-		project->variables()["QMAKE_LIBDIR_FLAGS"] += varGlue("QMAKE_LIBDIR_QT", " -L", " -L", "");
+		project->variables()["QMAKE_LIBDIR_FLAGS"] += varGlue("QMAKE_LIBDIR_QT", "-L", " -L", "");
 	    }
 	    if (project->isActiveConfig("thread") && !project->isEmpty("QMAKE_LIBS_QT_THREAD"))
 		project->variables()["QMAKE_LIBS"] += project->variables()["QMAKE_LIBS_QT_THREAD"];
@@ -152,6 +175,28 @@ UnixMakefileGenerator::init()
 		project->variables()["QMAKE_LIBS"] += project->variables()["QMAKE_LIBS_QT"];
 	}
     }
+    if ( project->isActiveConfig("opengl") ) {
+	project->variables()["INCLUDEPATH"] += project->variables()["QMAKE_INCDIR_OPENGL"];
+	if(!project->isEmpty("QMAKE_LIBDIR_OPENGL"))
+	    project->variables()["QMAKE_LIBDIR_FLAGS"] += varGlue("QMAKE_LIBDIR_OPENGL", "-L", " -L", "");
+	if ( is_qt )
+	    project->variables()["QMAKE_LIBS"] += project->variables()["QMAKE_LIBS_OPENGL_QT"];
+	else
+	    project->variables()["QMAKE_LIBS"] += project->variables()["QMAKE_LIBS_OPENGL"];
+    }
+    if(project->isActiveConfig("global_init_link_order"))
+	project->variables()["QMAKE_LIBS"] += project->variables()["LIBS"];
+    if ( project->isActiveConfig("x11inc") )
+	project->variables()["INCLUDEPATH"] += project->variables()["QMAKE_INCDIR_X11"];
+    if ( project->isActiveConfig("x11lib") ) {
+	if(!project->isEmpty("QMAKE_LIBDIR_X11"))
+	    project->variables()["QMAKE_LIBDIR_FLAGS"] += varGlue("QMAKE_LIBDIR_X11", "-L", " -L", "");
+	project->variables()["QMAKE_LIBS"] += project->variables()["QMAKE_LIBS_X11"];
+    }
+    if ( project->isActiveConfig("x11sm") )
+	project->variables()["QMAKE_LIBS"] += project->variables()["QMAKE_LIBS_X11SM"];
+    if ( project->isActiveConfig("dylib") )
+	project->variables()["QMAKE_LIBS"] += project->variables()["QMAKE_LIBS_DYNLOAD"];
     if ( project->isActiveConfig("thread") ) {
 	if(project->isActiveConfig("qt"))
 	    project->variables()[is_qt ? "PRL_EXPORT_DEFINES" : "DEFINES"].append("QT_THREAD_SUPPORT");
@@ -164,28 +209,6 @@ UnixMakefileGenerator::init()
 	if(!project->isEmpty("QMAKE_LFLAGS_THREAD"))
 	    project->variables()["QMAKE_LFLAGS"] += project->variables()["QMAKE_LFLAGS_THREAD"];
     }
-    if ( project->isActiveConfig("opengl") ) {
-	project->variables()["INCLUDEPATH"] += project->variables()["QMAKE_INCDIR_OPENGL"];
-	if(!project->isEmpty("QMAKE_LIBDIR_OPENGL")) 
-	    project->variables()["QMAKE_LIBDIR_FLAGS"] += varGlue("QMAKE_LIBDIR_OPENGL", " -L", " -L", "");
-	if ( is_qt )
-	    project->variables()["QMAKE_LIBS"] += project->variables()["QMAKE_LIBS_OPENGL_QT"];
-	else
-	    project->variables()["QMAKE_LIBS"] += project->variables()["QMAKE_LIBS_OPENGL"];
-    }
-    if(project->isActiveConfig("global_init_link_order"))
-	project->variables()["QMAKE_LIBS"] += project->variables()["LIBS"];
-    if ( project->isActiveConfig("x11sm") )
-	project->variables()["QMAKE_LIBS"] += project->variables()["QMAKE_LIBS_X11SM"];
-    if ( project->isActiveConfig("dylib") )
-	project->variables()["QMAKE_LIBS"] += project->variables()["QMAKE_LIBS_DYNLOAD"];
-    if ( project->isActiveConfig("x11inc") )
-	project->variables()["INCLUDEPATH"] += project->variables()["QMAKE_INCDIR_X11"];
-    if ( project->isActiveConfig("x11lib") ) {
-	if(!project->isEmpty("QMAKE_LIBDIR_X11"))
-	    project->variables()["QMAKE_LIBDIR_FLAGS"] += varGlue("QMAKE_LIBDIR_X11", " -L", " -L", "");
-	project->variables()["QMAKE_LIBS"] += project->variables()["QMAKE_LIBS_X11"];
-    }
     if ( project->isActiveConfig("moc") )
 	setMocAware(TRUE);
     if ( project->isEmpty("QMAKE_RUN_CC") )
@@ -199,7 +222,7 @@ UnixMakefileGenerator::init()
     project->variables()["QMAKE_FILETAGS"] += QStringList::split("HEADERS SOURCES TARGET DESTDIR", " ");
     if ( !project->isEmpty("PRECOMPH") ) {
 	initOutPaths(); 	// Need to fix MOC_DIR since we do this before init()
-	QString allmoc = project->first("MOC_DIR") + "/allmoc.cpp";
+	QString allmoc = fileFixify(project->first("MOC_DIR") + "/allmoc.cpp", QDir::currentDirPath(), Option::output_dir);
 	project->variables()["SOURCES"].prepend(allmoc);
 	project->variables()["HEADERS_ORIG"] = project->variables()["HEADERS"];
 	project->variables()["HEADERS"].clear();
@@ -243,12 +266,12 @@ UnixMakefileGenerator::init()
 	QStringList ar_sublibs, objs = project->variables()["OBJECTS"] + project->variables()["OBJMOC"];
 	if(ok && max_files > 5 && max_files < (int)objs.count()) {
 	    int obj_cnt = 0, lib_cnt = 0;
-	    QString destdir = project->first("DESTDIR"), lib;
+	    QString lib;
 	    for(QStringList::Iterator objit = objs.begin(); objit != objs.end(); ++objit) {
 		if((++obj_cnt) >= max_files) {
 		    if(lib_cnt) {
 			lib.sprintf("lib%s-tmp%d.a", project->first("QMAKE_ORIG_TARGET").latin1(), lib_cnt);
-			ar_sublibs << destdir + lib;
+			ar_sublibs << lib;
 			obj_cnt = 0;
 		    }
 		    lib_cnt++;
@@ -262,37 +285,159 @@ UnixMakefileGenerator::init()
     }
 }
 
-void
-UnixMakefileGenerator::processPrlVariable(const QString &var, const QStringList &l)
+QStringList
+UnixMakefileGenerator::uniqueSetLFlags(const QStringList &list1, QStringList &list2)
 {
-    if(var == "QMAKE_PRL_LIBS") {
-	QStringList &out = project->variables()["QMAKE_LIBS"];
-	for(QStringList::ConstIterator it = l.begin(); it != l.end(); ++it) {
-	    bool append = TRUE;
-	    if((*it).left(1) == "-") {
-		if((*it).left(2) == "-l" || (*it).left(2) == "-L") {
-		    append = out.findIndex((*it)) == -1;
-		} else if(project->isActiveConfig("macx") && (*it).left(2) == "-framework") {
-		    ++it;
-		    for(QStringList::ConstIterator outit = out.begin(); outit != out.end(); ++it) {
-			if((*outit) == "-framework") {
-			    ++outit;
-			    if((*outit) == (*it)) {
-				append = FALSE;
+    QStringList ret;
+    for(QStringList::ConstIterator it = list1.begin(); it != list1.end(); ++it) {
+	bool unique = TRUE;
+	if((*it).startsWith("-")) {
+	    if((*it).startsWith("-l") || (*it).startsWith("-L")) {
+		unique = list2.findIndex((*it)) == -1;
+	    } else if(project->isActiveConfig("macx") && (*it).startsWith("-framework")) {
+		int as_one = TRUE;
+		QString framework_in;
+		if((*it).length() > 11) {
+		    framework_in = (*it).mid(11);
+		} else {
+		    if(it != list1.end()) {
+			++it;
+			as_one = FALSE;
+			framework_in = (*it);
+		    }
+		}
+		if(!framework_in.isEmpty()) {
+		    for(QStringList::ConstIterator outit = list2.begin(); outit != list2.end(); ++outit) {
+			if((*outit).startsWith("-framework")) {
+			    QString framework_out;
+			    if((*outit).length() > 11) {
+				framework_out = (*outit).mid(11);
+			    } else {
+				if(it != list2.end()) {
+				    ++outit;
+				    framework_out = (*outit);
+				}
+			    }
+			    if(framework_out == framework_in) {
+				unique = FALSE;
 				break;
 			    }
 			}
 		    }
+		    if(unique) {
+			unique = FALSE; //because I'm about to just insert it myself
+			if(as_one) {
+			    ret.append("-framework " + framework_in);
+			} else {
+			    ret.append("-framework");
+			    ret.append(framework_in);
+			}
+		    }
 		}
-	    } else if(QFile::exists((*it))) {
-		append = out.findIndex((*it));
+	    } else {
+		unique = (list2.findIndex((*it)) == -1);
 	    }
-	    if(append)
-		out.append((*it));
+	} else if(QFile::exists((*it))) {
+	    unique = (list2.findIndex((*it)) == -1);
 	}
-    } else {
-	MakefileGenerator::processPrlVariable(var, l);
+	if(unique)
+	    ret.append((*it));
     }
+    return ret;
+}
+
+
+void
+UnixMakefileGenerator::processPrlVariable(const QString &var, const QStringList &l)
+{
+    if(var == "QMAKE_PRL_LIBS")
+	project->variables()["QMAKE_CURRENT_PRL_LIBS"] += uniqueSetLFlags(l, project->variables()["QMAKE_LIBS"]);
+    else
+	MakefileGenerator::processPrlVariable(var, l);
+}
+
+bool
+UnixMakefileGenerator::findLibraries()
+{
+    QPtrList<MakefileDependDir> libdirs;
+    libdirs.setAutoDelete(TRUE);
+    const QString lflags[] = { "QMAKE_LIBDIR_FLAGS", "QMAKE_LIBS", QString::null };
+    for(int i = 0; !lflags[i].isNull(); i++) {
+	QStringList &l = project->variables()[lflags[i]];
+	for(QStringList::Iterator it = l.begin(); it != l.end(); ++it) {
+	    QString stub, dir, extn, opt = (*it).stripWhiteSpace();
+	    if(opt.startsWith("-")) {
+		if(opt.startsWith("-L")) {
+		    QString r = opt.right(opt.length() - 2), l = r;
+		    fixEnvVariables(l);
+		    libdirs.append(new MakefileDependDir(r.replace("\"",""),
+							 l.replace("\"","")));
+		} else if(opt.startsWith("-l")) {
+		    stub = opt.mid(2);
+		} else if(project->isActiveConfig("macx") && opt.startsWith("-framework")) {
+		    if(opt.length() > 11) {
+			opt = opt.mid(11);
+		    } else {
+			++it;
+			opt = (*it);
+		    }
+		    extn = "";
+		    dir = "/System/Library/Frameworks/" + opt + ".framework/";
+		    stub = opt;
+		}
+	    } else {
+		extn = dir = "";
+		stub = opt;
+		int slsh = opt.findRev(Option::dir_sep);
+		if(slsh != -1) {
+		    dir = opt.left(slsh);
+		    stub = opt.mid(slsh+1);
+		}
+		QRegExp stub_reg("^.*lib(" + stub + "[^./=]*)\\.(.*)$");
+		if(stub_reg.exactMatch(stub)) {
+		    stub = stub_reg.cap(1);
+		    extn = stub_reg.cap(2);
+		}
+	    }
+	    if(!stub.isEmpty()) {
+		const QString modifs[] = { "-mt", QString::null };
+		for(int modif = 0; !modifs[modif].isNull(); modif++) {
+		    bool found = FALSE;
+		    QStringList extens;
+		    if(!extn.isNull())
+			extens << extn;
+		    else
+			extens << project->variables()["QMAKE_EXTENSION_SHLIB"].first() << "a";
+		    for(QStringList::Iterator extit = extens.begin(); extit != extens.end(); ++extit) {
+			if(dir.isNull()) {
+			    QString lib_stub;
+			    for(MakefileDependDir *mdd = libdirs.first(); mdd; mdd = libdirs.next() ) {
+				if(QFile::exists(mdd->local_dir + Option::dir_sep + "lib" + stub +
+						 modifs[modif] + "." + (*extit))) {
+				    lib_stub = stub + modifs[modif];
+				    break;
+				}
+			    }
+			    if(!lib_stub.isNull()) {
+				(*it) = "-l" + lib_stub;
+				found = TRUE;
+				break;
+			    }
+			} else {
+			    if(QFile::exists("lib" + stub + modifs[modif] + "." + (*extit))) {
+				(*it) = "lib" + stub + modifs[modif] + "." + (*extit);
+				found = TRUE;
+				break;
+			    }
+			}
+		    }
+		    if(found)
+			break;
+		}
+	    }
+	}
+    }
+    return FALSE;
 }
 
 void
@@ -307,19 +452,20 @@ UnixMakefileGenerator::processPrlFiles()
 	    QStringList l_out;
 	    QStringList &l = project->variables()[lflags[i]];
 	    for(QStringList::Iterator it = l.begin(); it != l.end(); ++it) {
-		QString opt = (*it);
-		if(opt.left(1) == "-") {
-		    if(opt.left(2) == "-L") {
+		project->variables()["QMAKE_CURRENT_PRL_LIBS"].clear();
+		QString opt = (*it).stripWhiteSpace();;
+		if(opt.startsWith("-")) {
+		    if(opt.startsWith("-L")) {
 			QString r = opt.right(opt.length() - 2), l = r;
 			fixEnvVariables(l);
-			libdirs.append(new MakefileDependDir(r.replace(QRegExp("\""),""),
-							     l.replace(QRegExp("\""),"")));
-		    } else if(opt.left(2) == "-l" && !processed[opt]) {
+			libdirs.append(new MakefileDependDir(r.replace("\"",""),
+							     l.replace("\"","")));
+		    } else if(opt.startsWith("-l") && !processed[opt]) {
 			QString lib = opt.right(opt.length() - 2), prl;
 			for(MakefileDependDir *mdd = libdirs.first(); mdd; mdd = libdirs.next() ) {
 			    prl = mdd->local_dir + Option::dir_sep + "lib" + lib + Option::prl_ext;
 			    if(processPrlFile(prl)) {
-				if(prl.left(mdd->local_dir.length()) == mdd->local_dir)
+				if(prl.startsWith(mdd->local_dir))
 				    prl.replace(0, mdd->local_dir.length(), mdd->real_dir);
 				QRegExp reg("^.*lib(" + lib + "[^./=]*)\\..*$");
 				if(reg.exactMatch(prl))
@@ -330,17 +476,22 @@ UnixMakefileGenerator::processPrlFiles()
 				break;
 			    }
 			}
-		    } else if(project->isActiveConfig("macx") && opt == "-framework") {
-			l_out.append(opt);
-			++it;
-			opt = (*it);
+		    } else if(project->isActiveConfig("macx") && opt.startsWith("-framework")) {
+			if(opt.length() > 11) {
+			    opt = opt.mid(11);
+			} else {
+			    ++it;
+			    opt = (*it);
+			}
 			QString prl = "/System/Library/Frameworks/" + opt +
 				      ".framework/" + opt + Option::prl_ext;
 			if(processPrlFile(prl))
 			    ret = TRUE;
+			l_out.append("-framework");
 		    }
 		    if(!opt.isEmpty())
 			l_out.append(opt);
+		    l_out += uniqueSetLFlags(project->variables()["QMAKE_CURRENT_PRL_LIBS"], l_out);
 		} else {
 		    if(!processed[opt] && processPrlFile(opt)) {
 			processed.insert(opt, (void*)1);
@@ -348,6 +499,7 @@ UnixMakefileGenerator::processPrlFiles()
 		    }
 		    if(!opt.isEmpty())
 			l_out.append(opt);
+		    l_out += uniqueSetLFlags(project->variables()["QMAKE_CURRENT_PRL_LIBS"], l_out);
 		}
 	    }
 	    if(ret && l != l_out)
@@ -365,16 +517,13 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
 	return QString();
 
     bool resource = FALSE;
+    const QString root = "$(INSTALL_ROOT)";
     QStringList &uninst = project->variables()[t + ".uninstall"];
     QString ret, destdir=project->first("DESTDIR");
     QString targetdir = Option::fixPathToTargetOS(project->first("target.path"), FALSE);
-
-    fileFixify( destdir );
-    fileFixify( targetdir );
-
     if(!destdir.isEmpty() && destdir.right(1) != Option::dir_sep)
 	destdir += Option::dir_sep;
-    targetdir = "$(INSTALL_ROOT)" + Option::fixPathToTargetOS(targetdir, FALSE);
+    targetdir = fileFixify(targetdir);
     if(targetdir.right(1) != Option::dir_sep)
 	targetdir += Option::dir_sep;
 
@@ -393,32 +542,36 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
 	    int slsh = dst_prl.findRev('/');
 	    if(slsh != -1)
 		dst_prl = dst_prl.right(dst_prl.length() - slsh - 1);
-	    dst_prl = targetdir + dst_prl;
-	    ret += "-$(COPY) " + project->first("QMAKE_INTERNAL_PRL_FILE") + " " + dst_prl;
+	    dst_prl = root + targetdir + dst_prl;
+	    ret += "-$(COPY) \"" + project->first("QMAKE_INTERNAL_PRL_FILE") + "\" \"" + dst_prl + "\"";
 	    if(!uninst.isEmpty())
 		uninst.append("\n\t");
 	    uninst.append("-$(DEL_FILE) \"" + dst_prl + "\"");
 	}
-	if(!project->isActiveConfig("staticlib")) {
-	    if(project->isActiveConfig("plugin")) {
-	    } else if ( !project->isEmpty("QMAKE_HPUX_SHLIB") ) {
-		links << "$(TARGET0)";
-	    }else
-		links << "$(TARGET0)" << "$(TARGET1)" << "$(TARGET2)";
+	if ( project->isEmpty("QMAKE_CYGWIN_SHLIB") ) {
+	    if ( !project->isActiveConfig("staticlib") && !project->isActiveConfig("plugin") ) {
+		if ( project->isEmpty("QMAKE_HPUX_SHLIB") ) {
+		    links << "$(TARGET0)" << "$(TARGET1)" << "$(TARGET2)";
+		} else {
+		    links << "$(TARGET0)";
+	        }
+	    }
 	}
     }
     QString src_targ = target;
     if(!destdir.isEmpty())
 	src_targ = Option::fixPathToTargetOS(destdir + target, FALSE);
-    QString dst_targ = targetdir + target;
+    QString dst_targ = root + fileFixify(targetdir + target);
     if(!ret.isEmpty())
 	ret += "\n\t";
     ret += QString(resource ? "-$(COPY_DIR)" : "-$(COPY)") + " \"" +
 	   src_targ + "\" \"" + dst_targ + "\"";
-    if(!project->isEmpty("QMAKE_STRIP")) {
+    if(!project->isActiveConfig("debug") && !project->isEmpty("QMAKE_STRIP")) {
 	ret += "\n\t-" + var("QMAKE_STRIP");
+	if(!project->isEmpty("QMAKE_STRIPFLAGS_LIB") && project->first("TEMPLATE") == "lib")
+	    ret += " " + var("QMAKE_STRIPFLAGS_LIB");
 	if(resource)
-	    ret = " \"" + dst_targ + "/Contents/MacOS/$(QMAKE_TARGET)";
+	    ret = " \"" + dst_targ + "/Contents/MacOS/$(QMAKE_TARGET)\"";
 	else
 	    ret += " \"" + dst_targ + "\"";
     }
@@ -438,9 +591,7 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
 		int lslash = link.findRev(Option::dir_sep);
 		if(lslash != -1)
 		    link = link.right(link.length() - (lslash + 1));
-		QString dst_link = targetdir + link;
-		fileFixify(dst_link);
-		fileFixify(dst_targ);
+		QString dst_link = root + fileFixify(targetdir + link);
 		ret += "\n\t-$(SYMLINK) \"$(TARGET)\" \"" + dst_link + "\"";
 		if(!uninst.isEmpty())
 		    uninst.append("\n\t");
@@ -450,3 +601,4 @@ UnixMakefileGenerator::defaultInstall(const QString &t)
     }
     return ret;
 }
+
