@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/unixmake2.cpp   3.0.5   edited Jul 1 06:24 $
+** $Id: qt/unixmake2.cpp   3.1.2   edited Jan 16 01:03 $
 **
 ** Definition of ________ class.
 **
 ** Created : 970521
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the network module of the Qt GUI Toolkit.
 **
@@ -75,8 +75,7 @@ UnixMakefileGenerator::writeMakefile(QTextStream &t)
 void
 UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 {
-    QString deps = Option::output.name(), prl;
-    fileFixify(deps, QDir::currentDirPath());
+    QString deps = fileFixify(Option::output.name()), prl;
     bool do_incremental = (project->isActiveConfig("incremental") &&
 			   !project->variables()["QMAKE_INCREMENTAL"].isEmpty() &&
 			   (!project->variables()["QMAKE_APP_FLAG"].isEmpty() ||
@@ -108,7 +107,14 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
       << varGlue("DEFINES","-D"," -D","") << endl;
     t << "LEXFLAGS = " << var("QMAKE_LEXFLAGS") << endl;
     t << "YACCFLAGS= " << var("QMAKE_YACCFLAGS") << endl;
-    t << "INCPATH  = " << varGlue("INCLUDEPATH","-I", " -I", "") << " -I" << specdir() << endl;
+    t << "INCPATH  = " << "-I" << specdir();
+    if(!project->isActiveConfig("no_include_pwd")) {
+	QString pwd = fileFixify(QDir::currentDirPath());
+	if(pwd.isEmpty())
+	    pwd = ".";
+	t << " -I" << pwd;
+    }
+    t << varGlue("INCLUDEPATH"," -I", " -I", "") << endl;
 
     if(!project->isActiveConfig("staticlib")) {
 	t << "LINK     = ";
@@ -136,6 +142,8 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
     t << "SYMLINK  = " << var("QMAKE_SYMBOLIC_LINK") << endl;
     t << "DEL_DIR  = " << var("QMAKE_DEL_DIR") << endl;
     t << "MOVE     = " << var("QMAKE_MOVE") << endl;
+    t << "CHK_DIR_EXISTS= " << var("QMAKE_CHK_DIR_EXISTS") << endl;
+    t << "MKDIR    = " << var("QMAKE_MKDIR") << endl;
     t << endl;
 
     t << "####### Output directory" << endl << endl;
@@ -211,7 +219,7 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
     }
     if(do_incremental && !moc_incremental && !src_incremental)
 	do_incremental = FALSE;
-    t << "DIST	   = " << varList("DISTFILES") << endl;
+    t << "DIST	   = " << valList(fileFixify(project->variables()["DISTFILES"])) << endl;
     t << "QMAKE_TARGET = " << var("QMAKE_ORIG_TARGET") << endl;
     t << "DESTDIR  = " << var("DESTDIR") << endl;
     t << "TARGET   = " << var("TARGET") << endl;
@@ -219,13 +227,12 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 	t << "TARGETD   = " << var("TARGET") << endl;
     } else if (!project->isActiveConfig("staticlib") && project->variables()["QMAKE_APP_FLAG"].isEmpty()) {
 	t << "TARGETA	= " << var("TARGETA") << endl;
-	if(project->variables()["QMAKE_HPUX_SHLIBS"].isEmpty()) {
+	if (project->isEmpty("QMAKE_HPUX_SHLIB")) {
 	    t << "TARGETD	= " << var("TARGET_x.y.z") << endl;
 	    t << "TARGET0	= " << var("TARGET_") << endl;
 	    t << "TARGET1	= " << var("TARGET_x") << endl;
 	    t << "TARGET2	= " << var("TARGET_x.y") << endl;
-	}
-	else {
+	} else {
 	    t << "TARGETD	= " << var("TARGET_x") << endl;
 	    t << "TARGET0	= " << var("TARGET_") << endl;
 	}
@@ -259,7 +266,7 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 	QString odir;
 	if(!project->variables()["OBJECTS_DIR"].isEmpty())
 	    odir = project->first("OBJECTS_DIR");
-	t << "###### Dependancies" << endl << endl;
+	t << "###### Dependencies" << endl << endl;
 	t << odir << ".deps/%.d: %.cpp\n\t"
 	  << "@echo Creating depend for $<" << "\n\t"
 	  << "@test -d $(@D) || mkdir -p $(@D)" << "\n\t"
@@ -277,12 +284,12 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 	    for(QStringList::Iterator it = l.begin(); it != l.end(); ++it) {
 		if(!(*it).isEmpty()) {
 		    QString d_file;
-		    if((*it).right(2) == ".c") {
+		    if((*it).endsWith(".c")) {
 			d_file = (*it).left((*it).length() - 2);
 		    } else {
 			for(QStringList::Iterator cppit = Option::cpp_ext.begin();
 			    cppit != Option::cpp_ext.end(); ++cppit) {
-			    if((*it).right((*cppit).length()) == (*cppit)) {
+			    if((*it).endsWith((*cppit))) {
 				d_file = (*it).left((*it).length() - (*cppit).length());
 				break;
 			    }
@@ -384,7 +391,8 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 	      << endl << endl;
 
 	    //real target
-	    t << var("TARGET") << ": " << " " << incr_deps << " " << var("TARGETDEPS") << "\n\t";
+	    t << var("TARGET") << ": " << var("PRE_TARGETDEPS") << " " << incr_deps 
+	      << " " << var("POST_TARGETDEPS") << "\n\t";
 	    if(!destdir.isEmpty())
 		t << "\n\t" << "test -d " << destdir << " || mkdir -p " << destdir << "\n\t";
 	    if(!project->isEmpty("QMAKE_PRE_LINK"))
@@ -397,7 +405,8 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 	    t << "all: " << deps <<  " " << varGlue("ALL_DEPS",""," "," ") <<  "$(TARGET)"
 	      << endl << endl;
 
-	    t << "$(TARGET): $(UICDECLS) $(OBJECTS) $(OBJMOC) " << var("TARGETDEPS") << "\n\t";
+	    t << "$(TARGET): " << var("PRE_TARGETDEPS") << " $(UICDECLS) $(OBJECTS) $(OBJMOC) " 
+	      << var("POST_TARGETDEPS") << "\n\t";
 	    if(!destdir.isEmpty())
 		t << "test -d " << destdir << " || mkdir -p " << destdir << "\n\t";
 	    if(!project->isEmpty("QMAKE_PRE_LINK"))
@@ -425,7 +434,7 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 		  << "ld -r  -o " << incr_target_dir << " " << link_deps << endl;
 		//communicated below
 		QStringList &cmd = project->variables()["QMAKE_LINK_SHLIB_CMD"];
-		cmd.first().replace(QRegExp("\\$\\(OBJECTS\\) \\$\\(OBJMOC\\)"),
+		cmd.first().replace("$(OBJECTS) $(OBJMOC)",
 				    "$(INCREMENTAL_OBJECTS) $(INCREMENTAL_OBJMOC)"); //ick
 		cmd.append(incr_target_dir);
 		deps.prepend(incr_target_dir + " ");
@@ -458,13 +467,13 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 	      << " " <<  var("DESTDIR_TARGET") << endl << endl;
 
 	    //real target
-	    t << var("DESTDIR_TARGET") << ": " << incr_deps << " $(SUBLIBS) " <<
-		var("TARGETDEPS");
+	    t << var("DESTDIR_TARGET") << ": " << var("PRE_TARGETDEPS") << " " 
+	      << incr_deps << " $(SUBLIBS) " << var("POST_TARGETDEPS");
 	} else {
 	    t << "all: " << deps << " " << varGlue("ALL_DEPS",""," ","") << " " <<
 		var("DESTDIR_TARGET") << endl << endl;
-	    t << var("DESTDIR_TARGET") << ":  $(UICDECLS) $(OBJECTS) $(OBJMOC) $(SUBLIBS) " <<
-		var("TARGETDEPS");
+	    t << var("DESTDIR_TARGET") << ": " << var("PRE_TARGETDEPS") 
+	      << " $(UICDECLS) $(OBJECTS) $(OBJMOC) $(SUBLIBS) " << var("POST_TARGETDEPS");
 	}
 	if(!destdir.isEmpty())
 	    t << "\n\t" << "test -d " << destdir << " || mkdir -p " << destdir;
@@ -481,13 +490,13 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 	    if(!project->isEmpty("QMAKE_POST_LINK"))
 		t << "\n\t" << var("QMAKE_POST_LINK") << "\n\t";
 	    t << endl << endl;
-	} else if(project->variables()["QMAKE_HPUX_SHLIB"].isEmpty()) {
+	} else if(project->isEmpty("QMAKE_HPUX_SHLIB")) {
 	    t << "\n\t"
 	      << "-$(DEL_FILE) $(TARGET) $(TARGET0) $(TARGET1) $(TARGET2)" << "\n\t"
 	      << var("QMAKE_LINK_SHLIB_CMD") << "\n\t";
-	    t << varGlue("QMAKE_LN_SHLIB",""," "," $(TARGET) $(TARGET0)")  << "\n\t"
-	      << varGlue("QMAKE_LN_SHLIB",""," "," $(TARGET) $(TARGET1)") << "\n\t"
-	      << varGlue("QMAKE_LN_SHLIB",""," "," $(TARGET) $(TARGET2)");
+	    t << varGlue("QMAKE_LN_SHLIB","-"," "," $(TARGET) $(TARGET0)")  << "\n\t"
+	      << varGlue("QMAKE_LN_SHLIB","-"," "," $(TARGET) $(TARGET1)") << "\n\t"
+	      << varGlue("QMAKE_LN_SHLIB","-"," "," $(TARGET) $(TARGET2)");
 	    if(!destdir.isEmpty())
 		t << "\n\t"
 		  << "-$(DEL_FILE) " << var("DESTDIR") << "$(TARGET)\n\t"
@@ -516,10 +525,10 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 
 	if (! project->isActiveConfig("plugin")) {
 	    t << "staticlib: $(TARGETA)" << endl << endl;
-	    t << "$(TARGETA): $(UICDECLS) $(OBJECTS) $(OBJMOC)";
+	    t << "$(TARGETA): " << var("PRE_TARGETDEPS") << " $(UICDECLS) $(OBJECTS) $(OBJMOC)";
 	    if(do_incremental)
 		t << " $(INCREMENTAL_OBJECTS) $(INCREMENTAL_OBJMOC)";
-	    t << var("TARGETDEPS") << "\n\t"
+	    t << var("POST_TARGETDEPS") << "\n\t"
 	      << "-$(DEL_FILE) $(TARGETA) " << "\n\t"
 	      << var("QMAKE_AR_CMD");
 	    if(do_incremental)
@@ -529,11 +538,12 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 	    t << endl << endl;
 	}
     } else {
-	t << "all: " << deps << " " << varGlue("ALL_DEPS",""," "," ") << "$(TARGET) " 
-	  << varList("QMAKE_AR_SUBLIBS") << "\n\n"
-	  << "staticlib: $(TARGET)" << "\n\n";
+	t << "all: " << deps << " " << varGlue("ALL_DEPS",""," "," ") << var("DESTDIR") << "$(TARGET) " 
+	  << varGlue("QMAKE_AR_SUBLIBS", var("DESTDIR"), " " + var("DESTDIR"), "") << "\n\n"
+	  << "staticlib: " << var("DESTDIR") << "$(TARGET)" << "\n\n";
 	if(project->isEmpty("QMAKE_AR_SUBLIBS")) {
-	    t << "$(TARGET): $(UICDECLS) $(OBJECTS) $(OBJMOC) $(TARGETDEPS) " << "\n\t";
+	    t << var("DESTDIR") << "$(TARGET): " << var("PRE_TARGETDEPS") 
+	      << " $(UICDECLS) $(OBJECTS) $(OBJMOC) " << var("POST_TARGETDEPS") << "\n\t";
 	    if(!project->isEmpty("DESTDIR")) {
 		QString destdir = project->first("DESTDIR");
 		t << "test -d " << destdir << " || mkdir -p " << destdir << "\n\t";
@@ -544,8 +554,11 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 		t << "\t" << var("QMAKE_POST_LINK") << "\n";
 	    if(!project->isEmpty("QMAKE_RANLIB"))
 		t << "\t" << "$(RANLIB) $(TARGET)" << "\n";
+	    if(!project->isEmpty("DESTDIR")) 
+		t << "\t" << "-$(DEL_FILE) " << var("DESTDIR") << "$(TARGET)" << "\n"
+		  << "\t" << "-$(MOVE) $(TARGET) " << var("DESTDIR") << "\n";
 	} else {
-	    int cnt = 0, max_files = project->first("QMAKE_MAX_FILES_PER_AR").toInt();
+	    int max_files = project->first("QMAKE_MAX_FILES_PER_AR").toInt();
 	    QStringList objs = project->variables()["OBJECTS"] + project->variables()["OBJMOC"],
 			libs = project->variables()["QMAKE_AR_SUBLIBS"];
 	    libs.prepend("$(TARGET)");
@@ -556,11 +569,11 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 		    build << (*objit);
 		QString ar;
 		if((*libit) == "$(TARGET)") {
-		    t << "$(TARGET): $(UICDECLS) " << " $(TARGETDEPS) " 
-		      << valList(build) << "\n\t";
+		    t << var("DESTDIR") << "$(TARGET): " << var("PRE_TARGETDEPS") 
+		      << " $(UICDECLS) " << var("POST_TARGETDEPS") << valList(build) << "\n\t";
 		    ar = project->variables()["QMAKE_AR_CMD"].first();
-		    ar = ar.replace(QRegExp("\\$\\(OBJMOC\\)"), "").replace(QRegExp("\\$\\(OBJECTS\\)"), 
-									    build.join(" "));
+		    ar = ar.replace("$(OBJMOC)", "").replace("$(OBJECTS)", 
+							     build.join(" "));
 		} else {
 		    t << (*libit) << ": " << valList(build) << "\n\t";
 		    ar = "$(AR) " + (*libit) + " " + build.join(" ");
@@ -575,6 +588,9 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 		    t << "\t" << var("QMAKE_POST_LINK") << "\n";
 		if(!project->isEmpty("QMAKE_RANLIB"))
 		    t << "\t" << "$(RANLIB) " << (*libit) << "\n";
+		if(!project->isEmpty("DESTDIR")) 
+		    t << "\t" << "-$(DEL_FILE) " << var("DESTDIR") << (*libit) << "\n"
+		      << "\t" << "-$(MOVE) " << (*libit) << " " << var("DESTDIR") << "\n";
 	    }
 	}
 	t << endl << endl;
@@ -584,7 +600,7 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 
     if(!project->isActiveConfig("no_mocdepend")) {
 	//this is an implicity depend on moc, so it will be built if necesary, however
-	//moc itself shouldn't have this dependancy - this is a little kludgy but it is
+	//moc itself shouldn't have this dependency - this is a little kludgy but it is
 	//better than the alternative for now.
 	QString moc = project->first("QMAKE_MOC"), target = project->first("TARGET");
 	fixEnvVariables(target);
@@ -624,9 +640,8 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 
     QString ddir = project->isEmpty("QMAKE_DISTDIR") ? project->first("QMAKE_ORIG_TARGET") :
 		   project->first("QMAKE_DISTDIR");
-    QString ddir_c = (project->isEmpty("OBJECTS_DIR") ? QString(".tmp/") :
-		    project->first("OBJECTS_DIR")) + ddir;
-    fileFixify(ddir_c);
+    QString ddir_c = fileFixify((project->isEmpty("OBJECTS_DIR") ? QString(".tmp/") :
+				 project->first("OBJECTS_DIR")) + ddir);
     t << "dist: " << "\n\t"
       << "@mkdir -p " << ddir_c << " && "
       << "$(COPY_FILE) --parents $(SOURCES) $(HEADERS) $(FORMS) $(DIST) " << ddir_c << Option::dir_sep << " && ";
@@ -635,8 +650,7 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
     if(!project->isEmpty("FORMS")) {
 	QStringList &forms = project->variables()["FORMS"], ui_headers;
 	for(QStringList::Iterator formit = forms.begin(); formit != forms.end(); ++formit) {
-	    QString ui_h = (*formit) + Option::h_ext.first();
-	    fileFixify(ui_h);
+	    QString ui_h = fileFixify((*formit) + Option::h_ext.first());
 	    if(QFile::exists(ui_h) ) 
 	       ui_headers << ui_h;
 	}
@@ -647,7 +661,7 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
       << "$(TAR) " << var("QMAKE_ORIG_TARGET") << ".tar " << ddir << " && "
       << "$(GZIP) " << var("QMAKE_ORIG_TARGET") << ".tar ) && "
       << "$(MOVE) `dirname " << ddir_c << "`" << Option::dir_sep << var("QMAKE_ORIG_TARGET") << ".tar.gz . && "
-      << "$(DEL_DIR) " << ddir_c
+      << "$(DEL_FILE) -r " << ddir_c
       << endl << endl;
 
     QString clean_targets;
@@ -671,6 +685,45 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
     }
     t << endl;
 
+    t << "yaccclean:" << "\n";
+    if(!var("YACCSOURCES").isEmpty()) {
+	QStringList clean, &l = project->variables()["YACCSOURCES"];
+	for(QStringList::Iterator it = l.begin(); it != l.end(); ++it) {
+	    QFileInfo fi((*it));
+	    QString dir;
+	    if(fi.dirPath() != ".")
+		dir = fi.dirPath() + Option::dir_sep;
+	    dir = fileFixify(dir, QDir::currentDirPath(), Option::output_dir);
+	    if(!dir.isEmpty() && dir.right(Option::dir_sep.length()) != Option::dir_sep)
+		dir += Option::dir_sep;
+	    clean <<  dir + fi.baseName(TRUE) + Option::yacc_mod + Option::cpp_ext.first();
+	    clean << dir + fi.baseName(TRUE) + Option::yacc_mod + Option::h_ext.first();
+	}
+	if(!clean.isEmpty()) {
+	    t << "\t-$(DEL_FILE) " << clean.join(" ") << "\n";
+	    clean_targets += " yaccclean";
+	}
+    }
+
+    t << "lexclean:" << "\n";
+    if(!var("LEXSOURCES").isEmpty()) {
+	QStringList clean, &l = project->variables()["LEXSOURCES"];
+	for(QStringList::Iterator it = l.begin(); it != l.end(); ++it) {
+	    QFileInfo fi((*it));
+	    QString dir;
+	    if(fi.dirPath() != ".")
+		dir = fi.dirPath() + Option::dir_sep;
+	    dir = fileFixify(dir, QDir::currentDirPath(), Option::output_dir);
+	    if(!dir.isEmpty() && dir.right(Option::dir_sep.length()) != Option::dir_sep)
+		dir += Option::dir_sep;
+	    clean <<  dir + fi.baseName(TRUE) + Option::lex_mod + Option::cpp_ext.first();
+	}
+	if(!clean.isEmpty()) {
+	    t << "\t-$(DEL_FILE) " << clean.join(" ") << "\n";
+	    clean_targets += " lexclean";
+	}
+    }
+
     if(do_incremental) {
 	t << "incrclean:" << "\n";
 	if(src_incremental)
@@ -680,8 +733,9 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 	t << endl;
     }
 
-    t << "clean:" << clean_targets << "\n\t"
-      << "-$(DEL_FILE) $(OBJECTS) " << "\n\t";
+    t << "clean:" << clean_targets << "\n\t";
+    if(!project->isEmpty("OBJECTS"))
+	t << "-$(DEL_FILE) $(OBJECTS) " << "\n\t";
     if(!project->isEmpty("IMAGES"))
 	t << varGlue("QMAKE_IMAGE_COLLECTION", "\t-$(DEL_FILE) ", " ", "") << "\n\t";
     if(src_incremental)
@@ -703,8 +757,12 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
     QString destdir = project->first("DESTDIR");
     if(!destdir.isEmpty() && destdir.right(1) != Option::dir_sep)
 	destdir += Option::dir_sep;
-    t << "distclean: " << "clean\n\t"
-      << "-$(DEL_FILE) " << destdir << "$(TARGET)" << " " << "$(TARGET)" << "\n";
+    t << "distclean: " << "clean\n";
+    if(project->first("TEMPLATE") == "app" &&
+       project->isActiveConfig("resource_fork") && !project->isActiveConfig("console"))
+	t << "\t-$(DEL_FILE) -r " << destdir.section(Option::dir_sep, 0, -4) << "\n";
+    else
+	t << "\t-$(DEL_FILE) " << destdir << "$(TARGET)" << " " << "$(TARGET)" << "\n";
     if(!project->isActiveConfig("staticlib") && project->variables()["QMAKE_APP_FLAG"].isEmpty() &&
        !project->isActiveConfig("plugin"))
 	t << "\t-$(DEL_FILE) " << destdir << "$(TARGET0) " << destdir << "$(TARGET1) "
@@ -734,8 +792,8 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 		 cmd = var((*it) + ".commands"), deps;
 	if(targ.isEmpty())
 	    targ = (*it);
-	QStringList &depends = project->variables()[(*it) + ".depends"];
-	for(QStringList::Iterator dep_it = depends.begin(); dep_it != depends.end(); ++dep_it) {
+	QStringList &deplist = project->variables()[(*it) + ".depends"];
+	for(QStringList::Iterator dep_it = deplist.begin(); dep_it != deplist.end(); ++dep_it) {
 	    QString dep = var((*dep_it) + ".target");
 	    if(dep.isEmpty())
 		dep = (*dep_it);
@@ -747,57 +805,89 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
     t <<"FORCE:" << endl << endl;
 }
 
+struct SubDir
+{
+    QString directory, profile, target, makefile;
+};
+
 void
 UnixMakefileGenerator::writeSubdirs(QTextStream &t, bool direct)
 {
+    QPtrList<SubDir> subdirs;
+    {
+	QStringList subdirs_in = project->variables()["SUBDIRS"];
+	for(QStringList::Iterator it = subdirs_in.begin(); it != subdirs_in.end(); ++it) {
+	    QString file = (*it);
+	    fileFixify(file);
+	    SubDir *sd = new SubDir;
+	    subdirs.append(sd);
+	    sd->makefile = "$(MAKEFILE)";
+	    if((*it).right(4) == ".pro") {
+		int slsh = file.findRev(Option::dir_sep);
+		if(slsh != -1) {
+		    sd->directory = file.left(slsh+1);
+		    sd->profile = file.mid(slsh+1);
+		} else {
+		    sd->profile = file;
+		}
+	    } else {
+		sd->directory = file;
+	    }
+	    while(sd->directory.right(1) == Option::dir_sep)
+		sd->directory = sd->directory.left(sd->directory.length() - 1);
+	    if(!sd->profile.isEmpty()) {
+		QString basename = sd->directory;
+		int new_slsh = basename.findRev(Option::dir_sep);
+		if(new_slsh != -1)
+		    basename = basename.mid(new_slsh+1);
+		if(sd->profile != basename + ".pro")
+		    sd->makefile += "." + sd->profile.left(sd->profile.length() - 4); //no need for the .pro
+	    }
+	    sd->target = "sub-" + (*it);
+	    sd->target.replace('/', '-');
+	    sd->target.replace('.', '_');
+	}
+    }
+    QPtrListIterator<SubDir> it(subdirs);
+
     QString ofile = Option::output.name();
     if(ofile.findRev(Option::dir_sep) != -1)
 	ofile = ofile.right(ofile.length() - ofile.findRev(Option::dir_sep) -1);
-
     t << "MAKEFILE =	" << var("MAKEFILE") << endl;
     t << "QMAKE    =	" << var("QMAKE") << endl;
-    t << "SUBDIRS  =	" << varList("SUBDIRS") << endl;
     t << "DEL_FILE =    " << var("QMAKE_DEL_FILE") << endl;
-
-    // subdirectory targets are sub-directory
-    QStringList::Iterator it;
-    QStringList subdirs = project->variables()["SUBDIRS"];
-    t << "SUBTARGETS =	";
-    for(it = subdirs.begin(); it != subdirs.end(); ++it) {
-	QString sr = (*it);
-	sr.replace(QRegExp("/"), "-");
-	t << " \\\n\t\tsub-" << sr;
-    }
+    t << "SUBTARGETS =	";     // subdirectory targets are sub-directory
+    for( it.toFirst(); it.current(); ++it) 
+	t << " \\\n\t\t" << it.current()->target;
     t << endl << endl;
-
     t << "first: all\n\nall: " << ofile << " $(SUBTARGETS)" << endl << endl;
 
     // generate target rules
-    for(it = subdirs.begin(); it != subdirs.end(); ++it) {
-	QString sr = (*it), mkfile = (*it) + Option::dir_sep + "$(MAKEFILE)", out;
-	if(direct)
-	    out = " -o $(MAKEFILE)";
-	sr.replace(QRegExp("/"), "-");
+    for( it.toFirst(); it.current(); ++it) {
+	bool have_dir = !(*it)->directory.isEmpty();
+	QString mkfile = (*it)->makefile, out;
+	if(have_dir)
+	    mkfile.prepend((*it)->directory + Option::dir_sep);
+	if(direct || (*it)->makefile != "$(MAKEFILE)")
+	    out = " -o " + (*it)->makefile;
 	//qmake it
-	t << mkfile << ": " << "\n\t"
-	  << "cd " << (*it) << " && $(QMAKE)" << buildArgs() << out << endl;
+	t << mkfile << ": " << "\n\t";
+	if(have_dir)
+	    t << "cd " << (*it)->directory << " && ";
+	t << "$(QMAKE) " << (*it)->profile << buildArgs() << out << endl;
 	//actually compile
-	t << "sub-" << sr << ": " << mkfile << " FORCE" << "\n\t"
-	  << "cd " << *(it) << " && $(MAKE) -f $(MAKEFILE)" << endl << endl;
+	t << (*it)->target << ": " << mkfile << " FORCE" << "\n\t";
+	if(have_dir)
+	    t << "cd " << (*it)->directory << " && ";
+	t << "$(MAKE) -f " << (*it)->makefile << endl << endl;
     }
 
-    if (project->isActiveConfig("ordered")) {
-	// generate dependencies
-	QString tar, dep;
-	it = subdirs.begin();
-	while (it != subdirs.end()) {
-	    tar = *it++;
-            tar.replace(QRegExp("/"), "-");
-	    if (it != subdirs.end()) {
-		dep = *it;
-                dep.replace(QRegExp("/"), "-");
-		t << "sub-" << dep << ": sub-" << tar << endl;
-	    }
+    if (project->isActiveConfig("ordered")) { 	// generate dependencies
+	for( it.toFirst(); it.current(); ) {
+	    QString tar = it.current()->target;
+	    ++it;
+	    if (it.current()) 
+		t << it.current()->target << ": " << tar << endl;
 	}
 	t << endl;
     }
@@ -805,25 +895,40 @@ UnixMakefileGenerator::writeSubdirs(QTextStream &t, bool direct)
     writeMakeQmake(t);
 
     if(project->isEmpty("SUBDIRS")) {
-	t << "all qmake_all distclean install uiclean mocclean clean: FORCE" << endl;
+	t << "all qmake_all distclean install uiclean mocclean lexclean yaccclean clean: FORCE" << endl;
     } else {
-	t << "qmake_all: " << varGlue("SUBDIRS", "", Option::dir_sep + "$(MAKEFILE) ",
-				      Option::dir_sep + "$(MAKEFILE)") << "\n\t"
-	  << "for i in $(SUBDIRS); do ( if [ -d $$i ]; then cd $$i ; "
-	  << "grep \"^qmake_all:\" $(MAKEFILE) 2>/dev/null >/dev/null && "
-	  << "$(MAKE) -f $(MAKEFILE) qmake_all || true; fi; ) ; done" << endl << endl;
-
-	t << "clean: qmake_all FORCE" << "\n\t"
-	  << varGlue("QMAKE_CLEAN","-$(DEL_FILE) "," ","\n\t")
-	  << "for i in $(SUBDIRS); do ( if [ -d $$i ]; then cd $$i ; $(MAKE) "
-	    "-f $(MAKEFILE) clean; fi; ) ; done" << endl;
-	t <<"uninstall install uiclean mocclean: qmake_all FORCE" << "\n\t"
-	  << "for i in $(SUBDIRS); do ( if [ -d $$i ]; then cd $$i ; $(MAKE) "
-	    "-f $(MAKEFILE) $@; fi; ) ; done" << endl;
-	t <<"distclean: qmake_all" << " FORCE\n\t"
-	  << "for i in $(SUBDIRS); do ( if [ -d $$i ]; then cd $$i ; $(MAKE) "
-	    "-f $(MAKEFILE) $@ ; $(DEL_FILE) $(MAKEFILE) ; fi; ) ; done"
-	  << endl << endl;
+	t << "all: $(SUBTARGETS)" << endl;
+	t << "qmake_all:";
+	for( it.toFirst(); it.current(); ++it) {
+	    t << " ";
+	    if(!(*it)->directory.isEmpty())
+		t << (*it)->directory << Option::dir_sep;
+	    t << (*it)->makefile;
+	}
+	for( it.toFirst(); it.current(); ++it) {
+	    t << "\n\t ( ";
+	    if(!(*it)->directory.isEmpty())
+		t << "[ -d " << (*it)->directory << " ] && cd " << (*it)->directory << " ; ";
+	    t << "grep \"^qmake_all:\" " << (*it)->makefile 
+	      << " && $(MAKE) -f " << (*it)->makefile << " qmake_all" << "; ) || true";
+	}
+	t << endl;
+	t << "clean uninstall install uiclean mocclean lexclean yaccclean: qmake_all FORCE";
+	for( it.toFirst(); it.current(); ++it) {
+	    t << "\n\t ( ";
+	    if(!(*it)->directory.isEmpty())
+		t << "[ -d " << (*it)->directory << " ] && cd " << (*it)->directory << " ; ";
+	    t << "$(MAKE) -f " << (*it)->makefile << " $@" << "; ) || true";
+	}
+	t << endl;
+	t << "distclean: qmake_all FORCE";
+	for( it.toFirst(); it.current(); ++it) {
+	    t << "\n\t ( ";
+	    if(!(*it)->directory.isEmpty())
+		t << "[ -d " << (*it)->directory << " ] && cd " << (*it)->directory << " ; ";
+	    t << "$(MAKE) -f " << (*it)->makefile << " $@; $(DEL_FILE) " << (*it)->makefile << "; ) || true";
+	}
+	t << endl << endl;
     }
     t <<"FORCE:" << endl << endl;
 }
@@ -835,7 +940,8 @@ void UnixMakefileGenerator::init2()
 	project->variables()["VERSION"].append("1.0." +
 					       (project->isEmpty("VER_PAT") ? QString("0") :
 						project->first("VER_PAT")) );
-    QStringList l = QStringList::split('.', project->first("VERSION")) << "0" << "0"; //make sure there are three
+    QStringList l = QStringList::split('.', project->first("VERSION"));
+    l << "0" << "0"; //make sure there are three
     project->variables()["VER_MAJ"].append(l[0]);
     project->variables()["VER_MIN"].append(l[1]);
     project->variables()["VER_PAT"].append(l[2]);
@@ -851,15 +957,17 @@ void UnixMakefileGenerator::init2()
 	}
 #endif
 	project->variables()["TARGET"].first().prepend(project->first("DESTDIR"));
+       if ( !project->variables()["QMAKE_CYGWIN_EXE"].isEmpty() )
+	    project->variables()["TARGET_EXT"].append(".exe");
     } else if ( project->isActiveConfig("staticlib") ) {
-	project->variables()["TARGET"].first().prepend(project->first("DESTDIR") + "lib");
+	project->variables()["TARGET"].first().prepend("lib");
 	project->variables()["TARGET"].first() += ".a";
 	if(project->variables()["QMAKE_AR_CMD"].isEmpty())
 	    project->variables()["QMAKE_AR_CMD"].append("$(AR) $(TARGET) $(OBJECTS) $(OBJMOC)");
     } else {
 	project->variables()["TARGETA"].append(project->first("DESTDIR") + "lib" + project->first("TARGET") + ".a");
 	if ( !project->variables()["QMAKE_AR_CMD"].isEmpty() )
-	    project->variables()["QMAKE_AR_CMD"].first().replace(QRegExp("\\(TARGET\\)"),"(TARGETA)");
+	    project->variables()["QMAKE_AR_CMD"].first().replace("(TARGET)","(TARGETA)");
 	else
 	    project->variables()["QMAKE_AR_CMD"].append("$(AR) $(TARGETA) $(OBJECTS) $(OBJMOC)");
 	if( project->isActiveConfig("plugin") ) {
@@ -877,7 +985,7 @@ void UnixMakefileGenerator::init2()
 	    project->variables()["TARGET"] = project->variables()["TARGET_x.y.z"];
 	    if(project->isActiveConfig("qt"))
 		project->variables()["DEFINES"].append("QT_PLUGIN");
-	} else if ( !project->variables()["QMAKE_HPUX_SHLIB"].isEmpty() ) {
+	} else if ( !project->isEmpty("QMAKE_HPUX_SHLIB") ) {
 	    project->variables()["TARGET_"].append("lib" + project->first("TARGET") + ".sl");
 	    if(project->isActiveConfig("lib_version_first"))
 		project->variables()["TARGET_x"].append("lib" + project->first("VER_MAJ") + "." + 
@@ -886,7 +994,7 @@ void UnixMakefileGenerator::init2()
 		project->variables()["TARGET_x"].append("lib" + project->first("TARGET") + "." + 
 							project->first("VER_MAJ"));
 	    project->variables()["TARGET"] = project->variables()["TARGET_x"];
-	} else if ( !project->variables()["QMAKE_AIX_SHLIB"].isEmpty() ) {
+	} else if ( !project->isEmpty("QMAKE_AIX_SHLIB") ) {
 	    project->variables()["TARGET_"].append("lib" + project->first("TARGET") + ".a");
 	    if(project->isActiveConfig("lib_version_first")) {
 		project->variables()["TARGET_x"].append("lib" + project->first("TARGET") + "." +
@@ -950,12 +1058,20 @@ void UnixMakefileGenerator::init2()
 	    }
 	    project->variables()["TARGET"] = project->variables()["TARGET_x.y.z"];
 	}
-	project->variables()["QMAKE_LN_SHLIB"].append("-ln -s");
+	if(project->isEmpty("QMAKE_LN_SHLIB"))
+	    project->variables()["QMAKE_LN_SHLIB"].append("ln -s");
 	project->variables()["DESTDIR_TARGET"].append("$(TARGET)");
 	if ( !project->variables()["DESTDIR"].isEmpty() )
 	    project->variables()["DESTDIR_TARGET"].first().prepend(project->first("DESTDIR"));
-	if ( !project->variables()["QMAKE_LFLAGS_SONAME"].isEmpty() && !project->variables()["TARGET_x"].isEmpty() )
-	    project->variables()["QMAKE_LFLAGS_SONAME"].first() += project->first("TARGET_x");
+	if ( !project->variables()["QMAKE_LFLAGS_SONAME"].isEmpty()) {
+	    if(project->isActiveConfig("plugin")) {
+		if(!project->variables()["TARGET"].isEmpty() )
+		    project->variables()["QMAKE_LFLAGS_SONAME"].first() += project->first("TARGET");
+	    } else {
+		if(!project->variables()["TARGET_x"].isEmpty() )
+		    project->variables()["QMAKE_LFLAGS_SONAME"].first() += project->first("TARGET_x");
+	    }
+	}
 	if ( project->variables()["QMAKE_LINK_SHLIB_CMD"].isEmpty() )
 	    project->variables()["QMAKE_LINK_SHLIB_CMD"].append(
 		"$(LINK) $(LFLAGS) -o $(TARGET) $(OBJECTS) $(OBJMOC) $(LIBS)");
@@ -969,13 +1085,25 @@ void UnixMakefileGenerator::init2()
 	project->variables()["QMAKE_CXXFLAGS"] += project->variables()["QMAKE_CXXFLAGS_SHLIB"];
 	if ( project->isActiveConfig("plugin") ) {
 	    project->variables()["QMAKE_LFLAGS"] += project->variables()["QMAKE_LFLAGS_PLUGIN"];
-	    project->variables()["QMAKE_LFLAGS"] += project->variables()["QMAKE_LFLAGS_SONAME"];
+	    if( !project->isActiveConfig("plugin_no_soname") )
+		project->variables()["QMAKE_LFLAGS"] += project->variables()["QMAKE_LFLAGS_SONAME"];
 	} else {
 	    project->variables()["QMAKE_LFLAGS"] += project->variables()["QMAKE_LFLAGS_SHLIB"];
 	    project->variables()["QMAKE_LFLAGS"] += project->variables()["QMAKE_LFLAGS_SONAME"];
 	}
 	QString destdir = project->first("DESTDIR");
-	if ( !destdir.isEmpty() && !project->variables()["QMAKE_RPATH"].isEmpty() )
-	    project->variables()["QMAKE_LFLAGS"] += project->first("QMAKE_RPATH") + destdir;
+	if ( !destdir.isEmpty() && !project->variables()["QMAKE_RPATH"].isEmpty() ) {
+	    QString rpath_destdir = destdir;
+	    if(QDir::isRelativePath(rpath_destdir)) {
+		QFileInfo fi(Option::fixPathToLocalOS(rpath_destdir));
+		if(fi.convertToAbs())  //strange, shouldn't really happen
+		    rpath_destdir = Option::fixPathToTargetOS(rpath_destdir, FALSE);
+		else
+		    rpath_destdir = fi.filePath();
+	    } else {
+		rpath_destdir = Option::fixPathToTargetOS(rpath_destdir, FALSE);
+	    }
+	    project->variables()["QMAKE_LFLAGS"] += project->first("QMAKE_RPATH") + rpath_destdir;
+	}
     }
 }
