@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qeucjpcodec.cpp   3.0.5   edited Apr 2 21:10 $
+** $Id: qt/qeucjpcodec.cpp   3.1.2   edited Nov 7 16:55 $
 **
 ** Implementation of QEucJpCodec class
 **
 ** Created : 990225
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the tools module of the Qt GUI Toolkit.
 **
@@ -36,10 +36,11 @@
 **********************************************************************/
 
 // Most of the code here was originally written by Serika Kurusugawa
-// a.k.a. Junji Takagi, and is include in Qt with the author's permission,
+// a.k.a. Junji Takagi, and is included in Qt with the author's permission,
 // and the grateful thanks of the Trolltech team.
 
 /*! \class QEucJpCodec qeucjpcodec.h
+    \reentrant
     \ingroup i18n
 
   \brief The QEucJpCodec class provides conversion to and from EUC-JP character sets.
@@ -59,7 +60,7 @@
 
   \legalese
 
-  Copyright (c) 1999 Serika Kurusugawa. All rights reserved.
+  Copyright (C) 1999 Serika Kurusugawa. All rights reserved.
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
@@ -85,10 +86,8 @@
   SUCH DAMAGE.
 */
 
-
-
 /*
- * Copyright (c) 1999 Serika Kurusugawa, All rights reserved.
+ * Copyright (C) 1999 Serika Kurusugawa, All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -168,9 +167,9 @@ int QEucJpCodec::mibEnum() const
 /*!
   \reimp
 */
-QCString QEucJpCodec::fromUnicode(const QString& uc, int& len_in_out) const
+QCString QEucJpCodec::fromUnicode(const QString& uc, int& lenInOut) const
 {
-    int l = QMIN((int)uc.length(),len_in_out);
+    int l = QMIN((int)uc.length(),lenInOut);
     int rlen = l*3+1;
     QCString rstr(rlen);
     uchar* cursor = (uchar*)rstr.data();
@@ -203,8 +202,8 @@ QCString QEucJpCodec::fromUnicode(const QString& uc, int& len_in_out) const
 	    *cursor++ = '?';	// unknown char
 	}
     }
-    len_in_out = cursor - (uchar*)rstr.data();
-    rstr.truncate(len_in_out);
+    lenInOut = cursor - (uchar*)rstr.data();
+    rstr.truncate(lenInOut);
     return rstr;
 }
 
