@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qmessagebox.cpp   3.1.2   edited Dec 10 10:39 $
 **
 ** Implementation of QMessageBox class
 **
 ** Created : 950503
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the dialogs module of the Qt GUI Toolkit.
 **
@@ -45,7 +45,6 @@
 #include "qimage.h"
 #include "qapplication.h"
 #include "qstyle.h"
-#include "qguardedptr.h"
 #include "qobjectlist.h"
 #if defined(QT_ACCESSIBILITY_SUPPORT)
 #include "qaccessible.h"
@@ -55,6 +54,21 @@
 #endif
 
 
+// Internal class - don't touch
+
+class QMessageBoxLabel : public QLabel
+{
+    Q_OBJECT
+public:
+    QMessageBoxLabel( QWidget* parent ) : QLabel( parent, "messageBoxText")
+    {
+	setAlignment( AlignAuto );
+    }
+};
+#include "qmessagebox.moc"
+
+
+
 // the Qt logo, for aboutQt
 /* XPM */
 static const char * const qtlogo_xpm[] = {
@@ -436,12 +450,6 @@ static const char * const mb_texts[] = {
 QMessageBox::QMessageBox( QWidget *parent, const char *name )
     : QDialog( parent, name, TRUE, WStyle_Customize | WStyle_DialogBorder | WStyle_Title | WStyle_SysMenu )
 {
-#ifndef QT_NO_WIDGET_TOPEXTRA
-    if ( parent && parent->icon() && !parent->icon()->isNull() )
-        QDialog::setIcon( *parent->icon() );
-    else if ( qApp->mainWidget() && qApp->mainWidget()->icon() && !qApp->mainWidget()->icon()->isNull() )
-        QDialog::setIcon( *qApp->mainWidget()->icon() );
-#endif
     init( Ok, 0, 0 );
 }
 
@@ -472,7 +480,8 @@ QMessageBox::QMessageBox( QWidget *parent, const char *name )
   \endlist
 
   Use QMessageBox::NoButton for the later parameters to have fewer than
-  three buttons in your message box.
+  three buttons in your message box. There has to be at least one button,
+  otherwise QMessageBox adds an Ok button.
 
   One of the buttons can be OR-ed with the \c QMessageBox::Default flag
   to make it the default button (clicked when Enter is pressed).
@@ -512,13 +521,6 @@ QMessageBox::QMessageBox( const QString& caption,
 			  bool modal, WFlags f )
     : QDialog( parent, name, modal, f | WStyle_Customize | WStyle_DialogBorder | WStyle_Title | WStyle_SysMenu )
 {
-#ifndef QT_NO_WIDGET_TOPEXTRA
-    if ( parent && parent->icon() && !parent->icon()->isNull() )
-        QDialog::setIcon( *parent->icon() );
-    else if ( qApp->mainWidget() && qApp->mainWidget()->icon() && !qApp->mainWidget()->icon()->isNull() )
-        QDialog::setIcon( *qApp->mainWidget()->icon() );
-#endif
-
     init( button0, button1, button2 );
 #ifndef QT_NO_WIDGET_TOPEXTRA
     setCaption( caption );
@@ -563,9 +565,8 @@ void QMessageBox::init( int button0, int button1, int button2 )
 #endif
 
     }
-    label = new QLabel( this, "text" );
+    label = new QMessageBoxLabel( this );
     Q_CHECK_PTR( label );
-    label->setAlignment( AlignAuto );
 
     if ( (button2 && !button1) || (button1 && !button0) ) {
 #if defined(QT_CHECK_RANGE)
@@ -893,7 +894,7 @@ void QMessageBox::adjustSize()
             h = mbd->iconLabel.height() + 3*border + bh;
     }
     int w = QMAX( buttons, labelSize.width() + lmargin ) + 2*border;
-    QRect screen = QApplication::desktop()->screenGeometry( QApplication::desktop()->screenNumber( pos() ) );
+    QRect screen = QApplication::desktop()->screenGeometry( pos() );
     if ( w > screen.width() )
         w = screen.width();
     resize( w, h );
@@ -1037,8 +1038,7 @@ int QMessageBox::information( QWidget *parent,
                                        parent, "qt_msgbox_information", TRUE,
 				       WDestructiveClose);
     Q_CHECK_PTR( mb );
-    int reply = mb->exec();
-    return reply;
+    return mb->exec();
 }
 
 
@@ -1081,8 +1081,7 @@ int QMessageBox::warning( QWidget *parent,
                                        parent, "qt_msgbox_warning", TRUE,
 				       WDestructiveClose);
     Q_CHECK_PTR( mb );
-    int reply = mb->exec();
-    return reply;
+    return mb->exec();
 }
 
 
@@ -1125,8 +1124,7 @@ int QMessageBox::critical( QWidget *parent,
                                        parent, "qt_msgbox_critical", TRUE,
 				       WDestructiveClose);
     Q_CHECK_PTR( mb );
-    int reply = mb->exec();
-    return reply;
+    return mb->exec();
 }
 
 
@@ -1159,19 +1157,6 @@ void QMessageBox::about( QWidget *parent, const QString &caption,
                                        parent, "qt_msgbox_simple_about_box", TRUE,
 				       WDestructiveClose);
     Q_CHECK_PTR( mb );
-    QPixmap i;
-#ifndef QT_NO_WIDGET_TOPEXTRA
-    if ( parent && parent->icon())
-        i = *(parent->icon());
-    if ( i.isNull() && parent &&
-         parent->topLevelWidget()->icon() )
-        i = *(parent->topLevelWidget()->icon());
-    if ( i.isNull() && qApp && qApp->mainWidget() &&
-         qApp->mainWidget()->icon() )
-        i = *(qApp->mainWidget()->icon());
-#endif
-    if ( !i.isNull() )
-        mb->setIconPixmap( i );
     mb->exec();
 }
 
@@ -1226,8 +1211,7 @@ static int textBox( QWidget *parent, QMessageBox::Icon severity,
 #ifndef QT_NO_CURSOR
     mb->setCursor( Qt::arrowCursor );
 #endif
-    int reply = mb->exec();
-    return reply-1;
+    return mb->exec() - 1;
 }
 
 
@@ -1364,7 +1348,8 @@ void QMessageBox::aboutQt( QWidget *parent, const QString &caption )
     if ( c.isNull() )
         c = "About Qt";
     QMessageBox *mb = new QMessageBox( parent, "qt_msgbox_about_qt" );
-    QGuardedPtr<QMessageBox> ptr = mb;
+    Q_CHECK_PTR( mb );
+    mb->setWFlags( WDestructiveClose );
 
 #ifndef QT_NO_WIDGET_TOPEXTRA
     mb->setCaption( caption.isNull()?QString::fromLatin1("About Qt"):caption );
@@ -1396,9 +1381,6 @@ void QMessageBox::aboutQt( QWidget *parent, const QString &caption )
         mb->mbd->pb[0]->setFocus();
     }
     mb->exec();
-    // ptr will be zero if mb is deleted
-    mb = ptr;
-    delete mb;
 }
 
 /*!
