@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qasyncimageio.cpp   3.1.2   edited Jan 29 13:20 $
 **
 ** Implementation of asynchronous image/movie loading classes
 **
@@ -46,7 +46,7 @@
 extern void qt_init_image_handlers();
 extern void qt_init_image_plugins();
 
-#define TRANSPARENT 0x00000000
+#define Q_TRANSPARENT 0x00ffffff
 
 /*!
     \class QImageConsumer qasyncimageio.h
@@ -752,7 +752,7 @@ void QGIFFormat::disposePrevious( QImage& img, QImageConsumer* consumer )
       case RestoreBackground:
 	if (trans_index>=0) {
 	    // Easy:  we use the transparent color
-	    fillRect(img, l, t, r-l+1, b-t+1, TRANSPARENT);
+	    fillRect(img, l, t, r-l+1, b-t+1, Q_TRANSPARENT);
 	} else if (bgcol>=0) {
 	    // Easy:  we use the bgcol given
 	    fillRect(img, l, t, r-l+1, b-t+1, color(bgcol));
@@ -831,7 +831,7 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 		    ccount=0;
 		    state=GlobalColorMap;
 		    globalcmap = new QRgb[gncols+1]; // +1 for trans_index
-		    globalcmap[gncols] = TRANSPARENT;
+		    globalcmap[gncols] = Q_TRANSPARENT;
 		} else {
 		    state=Introducer;
 		}
@@ -939,7 +939,7 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 		    if ( localcmap )
 			delete [] localcmap;
 		    localcmap = new QRgb[lncols+1];
-		    localcmap[lncols] = TRANSPARENT;
+		    localcmap[lncols] = Q_TRANSPARENT;
 		    ncols = lncols;
 		} else {
 		    ncols = gncols;
@@ -1061,11 +1061,8 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 		} else {
 		    if (needfirst) {
 			firstcode=oldcode=code;
-			if (!out_of_bounds) {
-			    if ( firstcode!=trans_index && line ) {
-				line[y][x] = color(firstcode);
-			    }
-			}
+			if (!out_of_bounds && line && firstcode!=trans_index)
+			    line[y][x] = color(firstcode);
 			x++;
 			if (x>=swidth) out_of_bounds = TRUE;
 			needfirst=FALSE;
@@ -1109,11 +1106,8 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 			oldcode=incode;
 			while (sp>stack) {
 			    --sp;
-			    if (!out_of_bounds) {
-				if ( *sp!=trans_index ) {
-				    line[y][x] = color(*sp);
-				}
-			    }
+			    if (!out_of_bounds && *sp!=trans_index)
+				line[y][x] = color(*sp);
 			    x++;
 			    if (x>=swidth) out_of_bounds = TRUE;
 			    if (x>right) {
@@ -1133,20 +1127,20 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 	    break;
 	  case ExtensionLabel:
 	    switch (ch) {
-	     case 0xf9:
+	    case 0xf9:
 		state=GraphicControlExtension;
 		break;
-	     case 0xff:
+	    case 0xff:
 		state=ApplicationExtension;
 		break;
-/////////////////////////////////////////// Ignored at this time //////
-//           case 0xfe:
-//                state=CommentExtension;
-//                break;
-//           case 0x01:
-//                break;
-///////////////////////////////////////////////////////////////////////
-	     default:
+#if 0
+	    case 0xfe:
+		state=CommentExtension;
+		break;
+	    case 0x01:
+		break;
+#endif
+	    default:
 		state=SkipBlockSize;
 	    }
 	    count=0;
@@ -1193,8 +1187,10 @@ int QGIFFormat::decode(QImage& img, QImageConsumer* consumer,
 		disposal=Disposal((hold[1]>>2)&0x7);
 		//UNUSED: waitforuser=!!((hold[1]>>1)&0x1);
 		int delay=count>3 ? LM(hold[2], hold[3]) : 1;
-		if ( delay < 1 )
-		    delay = 1;
+		// IE and mozilla use a minimum delay of 10. With the minumum delay of 10
+		// we are compatible to them and avoid huge loads on the app and xserver.
+		if ( delay < 10 )
+		    delay = 10;
 
 		bool havetrans=hold[1]&0x1;
  		trans_index = havetrans ? hold[4] : -1;
@@ -1317,14 +1313,10 @@ void QGIFFormat::nextY(QImage& img, QImageConsumer* consumer)
 
 QRgb QGIFFormat::color( uchar index ) const
 {
-    QRgb *map;
-    if ( lcmap )
-	map = localcmap;
-    else
-	map =globalcmap;
-    if ( index > ncols )
-	return TRANSPARENT;
-    return map[index];
+    if ( index == trans_index || index > ncols )
+	return Q_TRANSPARENT;
+    QRgb *map = lcmap ? localcmap : globalcmap;
+    return map ? map[index] : 0;
 }
 
 
