@@ -1,9 +1,9 @@
 /****************************************************************************
-** $Id:  qt/qdragobject.cpp   3.0.5   edited Apr 29 19:35 $
+** $Id: qt/qdragobject.cpp   3.1.2   edited Feb 18 17:41 $
 **
 ** Implementation of Drag and Drop support
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -33,11 +33,17 @@
 **
 **********************************************************************/
 
-#include "qtextcodec.h"
+#include "qplatformdefs.h"
+
+// POSIX Large File Support redefines open -> open64
+#if defined(open)
+# undef open
+#endif
 
 #ifndef QT_NO_MIME
 
 #include "qdragobject.h"
+#include "qtextcodec.h"
 #include "qapplication.h"
 #include "qpoint.h"
 #include "qwidget.h"
@@ -46,7 +52,6 @@
 #include "qregexp.h"
 #include <ctype.h>
 
-
 // both a struct for storing stuff in and a wrapper to avoid polluting
 // the name space
 
@@ -249,7 +254,7 @@ static const char * const link_xpm[] = {
 #ifndef QT_NO_DRAGANDDROP
 
 // the universe's only drag manager
-static QDragManager * manager = 0;
+QDragManager * qt_dnd_manager = 0;
 
 
 QDragManager::QDragManager()
@@ -263,8 +268,8 @@ QDragManager::QDragManager()
     object = 0;
     dragSource = 0;
     dropWidget = 0;
-    if ( !manager )
-	manager = this;
+    if ( !qt_dnd_manager )
+	qt_dnd_manager = this;
     beingCancelled = FALSE;
     restoreCursor = FALSE;
     willDrop = FALSE;
@@ -277,7 +282,7 @@ QDragManager::~QDragManager()
     if ( restoreCursor )
 	QApplication::restoreOverrideCursor();
 #endif
-    manager = 0;
+    qt_dnd_manager = 0;
     delete [] pm_cursor;
 }
 
@@ -297,7 +302,7 @@ QDragObject::QDragObject( QWidget * dragSource, const char * name )
 {
     d = new QDragObjectData();
 #ifndef QT_NO_DRAGANDDROP
-    if ( !manager && qApp )
+    if ( !qt_dnd_manager && qApp )
 	(void)new QDragManager();
 #endif
 }
@@ -311,8 +316,8 @@ QDragObject::QDragObject( QWidget * dragSource, const char * name )
 QDragObject::~QDragObject()
 {
 #ifndef QT_NO_DRAGANDDROP
-    if ( manager && manager->object == this )
-	manager->cancel( FALSE );
+    if ( qt_dnd_manager && qt_dnd_manager->object == this )
+	qt_dnd_manager->cancel( FALSE );
 #endif
     delete d;
 }
@@ -328,13 +333,18 @@ QDragObject::~QDragObject()
     The \a hotspot is the point on (or off) the pixmap that should be
     under the cursor as it is dragged. It is relative to the top-left
     pixel of the pixmap.
+
+    \warning We have seen problems with drag cursors on different
+    graphics hardware and driver software on Windows. Setting the
+    graphics acceleration in the display settings down one tick solved
+    the problems in all cases.
 */
 void QDragObject::setPixmap(QPixmap pm, const QPoint& hotspot)
 {
     d->pixmap = pm;
     d->hot = hotspot;
-    if ( manager && manager->object == this )
-	manager->updatePixmap();
+    if ( qt_dnd_manager && qt_dnd_manager->object == this )
+	qt_dnd_manager->updatePixmap();
 }
 
 /*!
@@ -380,6 +390,9 @@ QPoint QDragObject::pixmapHotSpot() const
     always be correct anyway, but on Windows this is not necessarily
     the case (e.g. the file manager starts a background process to
     move files, so the source \e{must not} delete the files!)
+
+    Note that on Windows the drag operation will spin a blocking modal
+    event loop that will not dispatch any QTimers.
 */
 bool QDragObject::drag()
 {
@@ -462,8 +475,8 @@ void QDragObject::dragLink()
 */
 bool QDragObject::drag( DragMode mode )
 { // ### In Qt 1.x?  huh?
-    if ( manager )
-	return manager->drag( this, mode );
+    if ( qt_dnd_manager )
+	return qt_dnd_manager->drag( this, mode );
     else
 	return FALSE;
 }
@@ -598,7 +611,9 @@ void QTextDrag::setSubtype( const QCString & st)
     \ingroup draganddrop
 
     Plain text is passed in a QString which may contain multiple lines
-    (i.e. may contain newline characters).
+    (i.e. may contain newline characters). The drag target will receive
+    the newlines according to the runtime environment, e.g. LF on Unix,
+    and CRLF on Windows.
 
     Qt provides no built-in mechanism for delivering only a single-line.
 
@@ -663,7 +678,6 @@ const char * QTextDrag::format(int i) const
     return d->fmt[i];
 }
 
-static
 QTextCodec* findcharset(const QCString& mimetype)
 {
     int i=mimetype.find("charset=");
@@ -1362,6 +1376,16 @@ QCString QUriDrag::localFileToUri(const QString& filename)
 	}
 	r.insert(0,'/');
     }
+#endif
+#if defined ( Q_WS_X11 ) && 0
+    // URL without the hostname is considered to be errorneous by XDnD.
+    // See: http://www.newplanetsoftware.com/xdnd/dragging_files.html
+    // This feature is not active because this would break dnd between old and new qt apps.
+    char hostname[257];
+    if ( gethostname( hostname, 255 ) == 0 ) {
+	hostname[256] = '\0';
+	r.prepend( QString::fromLatin1( hostname ) );
+    }
 #endif
     return unicodeUriToUri(QString("file://" + r));
 }
@@ -1408,8 +1432,25 @@ QString QUriDrag::uriToLocalFile(const char* uri)
 
     if ( uri && 0==qstrnicmp(uri,"file:/",6) ) {
 	uri += 6;
-	if ( uri[0] != '/' || uri[1] == '/' ) {
-	    // It is local.
+	bool local = uri[0] != '/' || ( uri[0] != '\0' && uri[1] == '/' );
+#ifdef Q_WS_X11
+	// do we have a hostname?
+        if ( !local && uri[0] == '/' && uri[2] != '/' ) {
+	    // then move the pointer to after the 'hostname/' part of the uri
+            const char* hostname_end = strchr( uri+1, '/' );
+            if ( hostname_end != NULL ) {
+                char hostname[ 257 ];
+                if ( gethostname( hostname, 255 ) == 0 ) {
+                    hostname[ 256 ] = '\0';
+                    if ( qstrncmp( uri+1, hostname, hostname_end - ( uri+1 )) == 0 ) {
+                        uri = hostname_end + 1; // point after the slash
+                        local = TRUE;
+                    }
+                }
+            }
+        }
+#endif
+	if ( local ) {
 	    file = uriToUnicodeUri(uri);
 	    if ( uri[1] == '/' ) {
 		file.remove(0,1);
@@ -1488,7 +1529,7 @@ bool QUriDrag::decodeToUnicodeUris( const QMimeSource* e, QStringList& l )
 */
 QWidget* QDropEvent::source() const
 {
-    return manager ? manager->dragSource : 0;
+    return qt_dnd_manager ? qt_dnd_manager->dragSource : 0;
 }
 #endif
 
