@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qfocusdata.cpp   3.0.5   edited Apr 10 19:11 $
+** $Id: qt/qfocusdata.cpp   3.1.2   edited Nov 7 16:55 $
 **
 ** Implementation of QFocusData class
 **
@@ -118,3 +118,21 @@ QWidget* QFocusData::prev()
 	r = focusWidgets.last();
     return r;
 }
+
+/*!
+    Returns the last widget in the focus chain.
+    The cursor is not modified.
+*/
+QWidget *QFocusData::last() const
+{
+    return focusWidgets.getLast();
+}
+
+/*!
+    Returns the first widget in the focus chain.
+    The cursor is not modified.
+*/
+QWidget *QFocusData::first() const
+{
+    return focusWidgets.getFirst();
+}
