@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qfontdatabase.cpp   3.0.5   edited May 2 13:46 $
+** $Id: qt/qfontdatabase.cpp   3.1.2   edited Dec 11 23:05 $
 **
 ** Implementation of font database class.
 **
@@ -49,13 +49,10 @@
 #include "private/qapplication_p.h"
 #include "private/qfontdata_p.h"
 
-#include <stdlib.h>
-#include <ctype.h>
-
 class QtFontFamily;
 class QtFontFoundry;
 
-#if defined( Q_WS_MAC )
+#if defined(Q_WS_MAC)
 static void add_style( QtFontFamily *family, const QString& styleName,
 		       bool italic, bool lesserItalic, int weight );
 #endif
@@ -132,7 +129,7 @@ private:
 
 #endif // Q_WS_WIN
 
-#if defined( Q_WS_MAC )
+#if defined(Q_WS_MAC)
     friend void add_style( QtFontFamily *family, const QString& styleName,
                 bool italic, bool lesserItalic, int weight );
 #endif
@@ -212,6 +209,33 @@ private:
 
 };
 
+class QtFamilyNamesList : public QStringList
+{
+public:
+    void sort();
+};
+
+void QtFamilyNamesList::sort()
+{
+    // temporarily replace '[' with '#', so that 'Arial [Xft]' isn't placed
+    // after 'Arial Black [Xft]' ( '[' is after 'B', but '#' is before 'B' )
+    Iterator it;
+    for(  it = begin();
+         it != end();
+         ++it ) {
+        int pos = (*it).find( '[' );
+        if( pos >= 0 )
+            (*it)[ pos ] = '#';
+    }
+    qHeapSort( *this );
+    for( it = begin();
+         it != end();
+         ++it ) {
+        int pos = (*it).find( '#' );
+        if( pos >= 0 )
+            (*it)[ pos ] = '[';
+    }
+}
 
 class QtFontFoundry
 {
@@ -230,7 +254,7 @@ public:
 private:
     QString nm;
 
-    QStringList familyNames;
+    QtFamilyNamesList familyNames;
     QDict<QtFontFamily> familyDict;
 
     bool namesDirty;
@@ -267,7 +291,7 @@ private:
     QStringList foundryNames;
     QDict<QtFontFoundry> foundryDict;
 
-    QStringList familyNames;
+    QtFamilyNamesList familyNames;
     QDict<QtFontFamily> bestFamilyDict;
 
     bool namesDirty;
@@ -285,6 +309,7 @@ private:
     friend void QFontDatabase::qwsAddDiskFont( QDiskFont *qdf );
 #endif
 };
+
 static QFontDatabasePrivate *db=0;
 
 #if defined( Q_WS_X11 )
@@ -708,9 +733,6 @@ QString QFontDatabase::styleString( const QFont &f )
 }
 
 
-static QStringList emptyList;
-
-
 /*!
     \class QFontDatabase qfontdatabase.h
     \brief The QFontDatabase class provides information about the fonts available in the underlying window system.
@@ -872,8 +894,7 @@ QFontDatabase::QFontDatabase()
 }
 
 
-/*!
-    Returns a list of the names of the available font families.
+/*! Returns a sorted list of the names of the available font families.
 
     If a family exists in several foundries, the returned name for
     that font is in the form "family [foundry]". Examples: "Times
@@ -894,7 +915,7 @@ QStringList QFontDatabase::styles( const QString &family) const
 {
     const QtFontFamily *fam = d->family( family );
     if ( !fam )
-        return emptyList;
+        return QStringList();
 
     return fam->styles();
 }
@@ -996,9 +1017,6 @@ static const QtFontStyle * getStyle( QFontDatabasePrivate *d,
 }
 
 
-static QValueList<int> emptySizeList;
-
-
 /*!
     Returns a list of the point sizes available for the font that has
     family \a family and style \a style. The list may be empty.
@@ -1016,7 +1034,7 @@ QValueList<int> QFontDatabase::pointSizes( const QString &family,
 
     const QtFontStyle *sty = getStyle( d, family, s);
 
-    return sty ? sty->pointSizes() : emptySizeList;
+    return sty ? sty->pointSizes() : QValueList<int>();
 }
 
 
@@ -1062,7 +1080,7 @@ QValueList<int> QFontDatabase::smoothSizes( const QString &family,
 					    const QString &style)
 {
     const QtFontStyle *sty = getStyle( d, family, style );
-    return sty ? sty->smoothSizes() : emptySizeList;
+    return sty ? sty->smoothSizes() : QValueList<int>();
 }
 
 
