@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qlayout.cpp   3.0.5   edited May 8 02:51 $
+** $Id: qt/qlayout.cpp   3.1.2   edited Dec 20 14:27 $
 **
 ** Implementation of layout classes
 **
@@ -84,7 +84,6 @@ public:
     int vStretch() { return (item_->widget()) ?
 			 item_->widget()->sizePolicy().verStretch() : 0; }
 
-
 private:
     friend class QGridLayoutData;
     friend class QGridLayoutDataIterator;
@@ -144,6 +143,7 @@ public:
     bool isDirty() const { return needRecalc; }
     bool hasHeightForWidth( int space );
     int heightForWidth( int, int defB );
+    int minimumHeightForWidth( int, int defB );
 
     bool findWidget( QWidget* w, int *row, int *col );
 
@@ -175,6 +175,7 @@ private:
 
     int hfw_width;
     int hfw_height;
+    int hfw_minheight;
     int nextR;
     int nextC;
 
@@ -251,18 +252,22 @@ void QGridLayoutData::recalcHFW( int w, int spacing )
     QMemArray<QLayoutStruct> &rData = *hfwData;
 
     int h = 0;
+    int mh = 0;
     int n = 0;
     for ( int r = 0; r < rr; r++ ) {
-	h = h + rData[r].sizeHint;
+	h += rData[r].sizeHint;
+	mh += rData[r].minimumSize;
 	if ( !rData[r].empty )
 	    n++;
     }
-    if ( n )
+    if ( n ) {
 	h += ( n - 1 ) * spacing;
-    h = QMIN( QLAYOUTSIZE_MAX, h );
+	mh += ( n - 1 ) * spacing;
+    }
 
-    hfw_height = h;
     hfw_width = w;
+    hfw_height = QMIN( QLAYOUTSIZE_MAX, h );
+    hfw_minheight = QMIN( QLAYOUTSIZE_MAX, h );
 }
 
 int QGridLayoutData::heightForWidth( int w, int spacing )
@@ -275,6 +280,12 @@ int QGridLayoutData::heightForWidth( int w, int spacing )
     return hfw_height;
 }
 
+int QGridLayoutData::minimumHeightForWidth( int w, int spacing )
+{
+    (void) heightForWidth( w, spacing );
+    return hfw_minheight;
+}
+
 bool QGridLayoutData::findWidget( QWidget* w, int *row, int *col )
 {
     QPtrListIterator<QGridBox> it( things );
@@ -310,7 +321,7 @@ bool QGridLayoutData::findWidget( QWidget* w, int *row, int *col )
 
 QSize QGridLayoutData::findSize( QCOORD QLayoutStruct::*size, int spacer ) const
 {
-    QGridLayoutData *that = (QGridLayoutData*)this;
+    QGridLayoutData *that = (QGridLayoutData *)this;
     that->setupLayoutData( spacer );
 
     int w = 0;
@@ -435,14 +446,14 @@ void QGridLayoutData::add( QGridBox *box, int row, int col )
     setNextPosAfter( row, col );
 }
 
-void QGridLayoutData::add( QGridBox *box,  int row1, int row2,
-			   int col1, int col2  )
+void QGridLayoutData::add( QGridBox *box, int row1, int row2, int col1,
+			   int col2  )
 {
 #ifdef QT_CHECK_RANGE
     if ( row2 >= 0 && row2 < row1 )
-	qWarning( "QGridLayout: multicell fromRow greater than toRow" );
+	qWarning( "QGridLayout: Multi-cell fromRow greater than toRow" );
     if ( col2 >= 0 && col2 < col1 )
-	qWarning( "QGridLayout: multicell fromCol greater than toCol" );
+	qWarning( "QGridLayout: Multi-cell fromCol greater than toCol" );
 #endif
     if ( row1 == row2 && col1 == col2 ) {
 	add( box, row1, col1 );
@@ -454,7 +465,7 @@ void QGridLayoutData::add( QGridBox *box,  int row1, int row2,
     QGridMultiBox *mbox = new QGridMultiBox( box, row2, col2 );
     if ( !multi ) {
 	multi = new QPtrList<QGridMultiBox>;
-	multi->setAutoDelete(TRUE);
+	multi->setAutoDelete( TRUE );
     }
     multi->append( mbox );
     setDirty();
@@ -473,8 +484,8 @@ void QGridLayoutData::addData( QGridBox *box, bool r, bool c )
 
     if ( c ) {
 	if ( !cStretch[box->col] )
-	    colData[box->col].stretch = QMAX(colData[box->col].stretch,
-					     box->hStretch() );
+	    colData[box->col].stretch = QMAX( colData[box->col].stretch,
+					      box->hStretch() );
 	colData[box->col].sizeHint = QMAX( hint.width(),
 					   colData[box->col].sizeHint );
 	colData[box->col].minimumSize = QMAX( minS.width(),
@@ -486,8 +497,8 @@ void QGridLayoutData::addData( QGridBox *box, bool r, bool c )
     }
     if ( r ) {
 	if ( !rStretch[box->row] )
-	    rowData[box->row].stretch = QMAX(rowData[box->row].stretch,
-	    box->vStretch() );
+	    rowData[box->row].stretch = QMAX( rowData[box->row].stretch,
+					      box->vStretch() );
 	rowData[box->row].sizeHint = QMAX( hint.height(),
 					   rowData[box->row].sizeHint );
 	rowData[box->row].minimumSize = QMAX( minS.height(),
@@ -527,8 +538,6 @@ static void distributeMultiBox( QMemArray<QLayoutStruct> &chain, int spacing,
 				int minSize, int sizeHint,
 				QMemArray<int> &stretchArray, int stretch )
 {
-    //distribute the sizes and stretch somehow.
-
     int i;
     int w = 0;
     int wh = 0;
@@ -657,7 +666,7 @@ void QGridLayoutData::setupLayoutData( int spacing )
     needRecalc = FALSE;
 }
 
-void QGridLayoutData::addHfwData ( QGridBox *box, int width )
+void QGridLayoutData::addHfwData( QGridBox *box, int width )
 {
     QMemArray<QLayoutStruct> &rData = *hfwData;
     if ( box->hasHeightForWidth() ) {
@@ -892,10 +901,9 @@ inline QGridLayoutDataIterator::QGridLayoutDataIterator( QGridLayoutData *d )
     to distribute the size over the columns/rows (based on the stretch
     factors).
 
-    To remove a widget from a layout, either delete it or reparent it
-    with QWidget::reparent(). Hiding a widget with QWidget::hide()
-    also effectively removes the widget from the layout, until
-    QWidget::show() is called.
+    To remove a widget from a layout, call remove(). Calling
+    QWidget::hide() on a widget also effectively removes the widget
+    from the layout until QWidget::show() is called.
 
     This illustration shows a fragment of a dialog with a five-column,
     three-row grid (the grid is shown overlaid in magenta):
@@ -1000,7 +1008,7 @@ QGridLayout::QGridLayout( int nRows, int nCols,
     Destroys the grid layout. Geometry management is terminated if
     this is a top-level grid.
 
-    The layout's widgets are \e not destroyed.
+    The layout's widgets aren't destroyed.
 */
 QGridLayout::~QGridLayout()
 {
@@ -1068,9 +1076,17 @@ bool QGridLayout::hasHeightForWidth() const
 */
 int QGridLayout::heightForWidth( int w ) const
 {
-    QGridLayout *that = ((QGridLayout*)this);
-    return that->data->heightForWidth( w - 2*margin(), spacing() )
-	+ 2 * margin();
+    QGridLayout *that = (QGridLayout*)this;
+    return that->data->heightForWidth( w - 2 *margin(), spacing() )
+	   + 2 * margin();
+}
+
+/*! \internal */
+int QGridLayout::minimumHeightForWidth( int w ) const
+{
+    QGridLayout *that = (QGridLayout*)this;
+    return that->data->minimumHeightForWidth( w - 2 * margin(), spacing() )
+	   + 2 * margin();
 }
 
 /*!
@@ -1093,9 +1109,9 @@ void QGridLayout::setGeometry( const QRect &r )
 {
     if ( data->isDirty() || r != geometry() ) {
 	QLayout::setGeometry( r );
-	QRect cr = alignment() ? alignmentRect(r) : r;
-	QRect s( cr.x()+margin(), cr.y()+margin(),
-		 cr.width()-2*margin(), cr.height()-2*margin() );
+	QRect cr = alignment() ? alignmentRect( r ) : r;
+	QRect s( cr.x() + margin(), cr.y() + margin(),
+		 cr.width() - 2 * margin(), cr.height() - 2 * margin() );
 	data->distribute( s, spacing() );
     }
 }
@@ -1191,7 +1207,7 @@ static bool checkWidget( QLayout *l, QWidget *w )
 {
     if ( !w ) {
 #if defined(QT_CHECK_STATE)
-	qWarning( "cannot add null widget to %s/%s", l->className(),
+	qWarning( "QLayout: Cannot add null widget to %s/%s", l->className(),
 		  l->name() );
 #endif
 	return FALSE;
@@ -1199,12 +1215,12 @@ static bool checkWidget( QLayout *l, QWidget *w )
     if ( w->parentWidget() != l->mainWidget() && l->mainWidget() ) {
 #if defined(QT_CHECK_STATE)
 	if ( w->parentWidget() )
-	    qWarning( "Warning: adding %s/%s (child of %s/%s) to layout for "
+	    qWarning( "QLayout: Adding %s/%s (child of %s/%s) to layout for "
 		      "%s/%s", w->className(), w->name(),
 		      w->parentWidget()->className(), w->parentWidget()->name(),
 		      l->mainWidget()->className(), l->mainWidget()->name() );
 	else
-	    qWarning( "Warning: adding %s/%s (top-level widget) to layout for"
+	    qWarning( "QLayout: Adding %s/%s (top-level widget) to layout for"
 		      " %s/%s", w->className(), w->name(),
 		      l->mainWidget()->className(), l->mainWidget()->name() );
 #endif
@@ -1237,8 +1253,8 @@ void QGridLayout::addWidget( QWidget *w, int row, int col, int alignment )
 	return;
     if ( row < 0 || col < 0 ) {
 #if defined(QT_CHECK_STATE)
-	qWarning( "cannot add %s/%s to %s/%s at row %d col %d",
-		 w->className(), w->name(), className(), name(), row, col );
+	qWarning( "QGridLayout: Cannot add %s/%s to %s/%s at row %d col %d",
+		  w->className(), w->name(), className(), name(), row, col );
 #endif
 	return;
     }
@@ -1272,7 +1288,7 @@ void QGridLayout::addMultiCellWidget( QWidget *w, int fromRow, int toRow,
     Places the \a layout at position (\a row, \a col) in the grid. The
     top-left position is (0, 0).
 */
-void QGridLayout::addLayout( QLayout *layout, int row, int col)
+void QGridLayout::addLayout( QLayout *layout, int row, int col )
 {
     addChildLayout( layout );
     add( layout, row, col );
@@ -1294,6 +1310,7 @@ void QGridLayout::addLayout( QLayout *layout, int row, int col)
 void QGridLayout::addMultiCellLayout( QLayout *layout, int fromRow, int toRow,
 				      int fromCol, int toCol, int alignment )
 {
+    addChildLayout( layout );
     QGridBox *b = new QGridBox( layout );
     b->setAlignment( alignment );
     data->add( b, fromRow, toRow, fromCol, toCol );
@@ -1362,7 +1379,7 @@ void QGridLayout::setColStretch( int col, int stretch )
 void QGridLayout::addRowSpacing( int row, int minsize )
 {
     QLayoutItem *b = new QSpacerItem( 0, minsize );
-    //b.setAlignment( align );
+    // b.setAlignment( align );
     add( b, row, 0 );
 }
 
@@ -1372,12 +1389,15 @@ void QGridLayout::addRowSpacing( int row, int minsize )
 void QGridLayout::addColSpacing( int col, int minsize )
 {
     QLayoutItem *b = new QSpacerItem( minsize, 0 );
-    //b.setAlignment( align );
+    // b.setAlignment( align );
     add( b, 0, col );
 }
 
 /*!
-    Returns the expansiveness of this layout.
+    Returns whether this layout can make use of more space than
+    sizeHint(). A value of \c Vertical or \c Horizontal means that it wants
+    to grow in only one dimension, whereas \c BothDirections means that
+    it wants to grow in both dimensions.
 */
 QSizePolicy::ExpandData QGridLayout::expanding() const
 {
@@ -1411,8 +1431,10 @@ QGridLayout::Corner QGridLayout::origin() const
 */
 void QGridLayout::invalidate()
 {
+    data->setReversed( FALSE, QApplication::reverseLayout() );
+
     QLayout::invalidate();
-    QLayout::setGeometry( QRect() ); //###binary compatibility
+    QLayout::setGeometry( QRect() ); // for binary compatibility (?)
     data->setDirty();
 }
 
@@ -1424,20 +1446,40 @@ QLayoutIterator QGridLayout::iterator()
 
 struct QBoxLayoutItem
 {
-    QBoxLayoutItem( QLayoutItem *it, int stretch_ = 0)
-	: item(it), stretch(stretch_), magic(FALSE) {}
+    QBoxLayoutItem( QLayoutItem *it, int stretch_ = 0 )
+	: item( it ), stretch( stretch_ ), magic( FALSE ) { }
     ~QBoxLayoutItem() { delete item; }
+
     int hfw( int w ) {
-	if ( item->hasHeightForWidth() )
+	if ( item->hasHeightForWidth() ) {
 	    return item->heightForWidth( w );
-	else
+	} else {
 	    return item->sizeHint().height();
+	}
     }
+    int mhfw( int w ) {
+	if ( item->hasHeightForWidth() ) {
+	    return item->heightForWidth( w );
+	} else {
+	    return item->minimumSize().height();
+	}
+    }
+    int hStretch() {
+	if ( stretch == 0 && item->widget() ) {
+	    return item->widget()->sizePolicy().horStretch();
+	} else {
+	    return stretch;
+	}
+    }
+    int vStretch() {
+	if ( stretch == 0 && item->widget() ) {
+	    return item->widget()->sizePolicy().verStretch();
+	} else {
+	    return stretch;
+	}
+    }
+
     QLayoutItem *item;
-    int hStretch() { if (stretch == 0 && item->widget())
-	return item->widget()->sizePolicy().horStretch(); else return stretch;}
-    int vStretch() { if (stretch == 0 && item->widget())
-	return item->widget()->sizePolicy().verStretch(); else return stretch;}
     int stretch;
     bool magic;
 };
@@ -1451,8 +1493,9 @@ public:
     ~QBoxLayoutData() { delete geomArray; }
     void setDirty() {
 	delete geomArray;
-	geomArray=0;
-	hfwWidth=hfwHeight=-1;
+	geomArray = 0;
+	hfwWidth = -1;
+	hfwHeight = -1;
 	dirty = TRUE;
     }
 
@@ -1460,12 +1503,13 @@ public:
     QMemArray<QLayoutStruct> *geomArray;
     int hfwWidth;
     int hfwHeight;
+    int hfwMinHeight;
     QSize sizeHint;
     QSize minSize;
     QSize maxSize;
     QSizePolicy::ExpandData expanding;
-    uint hasHfw	    : 1;
-    uint dirty	    : 1;
+    uint hasHfw : 1;
+    uint dirty : 1;
 };
 
 class QBoxLayoutIterator : public QGLayoutIterator
@@ -1572,10 +1616,9 @@ private:
     margin width for a top-level layout, or to the same as the parent
     layout. Both are parameters to the constructor.
 
-    To remove a widget from a layout, either delete it or reparent it
-    with QWidget::reparent(). Hiding a widget with QWidget::hide()
-    also effectively removes the widget from the layout, until
-    QWidget::show() is called.
+    To remove a widget from a layout, call remove(). Calling
+    QWidget::hide() on a widget also effectively removes the widget
+    from the layout until QWidget::show() is called.
 
     You will almost always want to use QVBoxLayout and QHBoxLayout
     rather than QBoxLayout because of their convenient constructors.
@@ -1659,7 +1702,7 @@ QBoxLayout::QBoxLayout( Direction d, int spacing, const char *name )
 /*!
     Destroys this box layout.
 
-    The layout's widgets are \e not destroyed.
+    The layout's widgets aren't destroyed.
 */
 QBoxLayout::~QBoxLayout()
 {
@@ -1699,13 +1742,12 @@ QSize QBoxLayout::maximumSize() const
 	QBoxLayout *that = (QBoxLayout*)this;
 	that->setupGeom();
     }
-    QSize s = (data->maxSize + QSize(2 * margin(), 2 * margin()))
+    QSize s = ( data->maxSize + QSize(2 * margin(), 2 * margin()) )
 	      .boundedTo(QSize(QLAYOUTSIZE_MAX, QLAYOUTSIZE_MAX));
     if ( alignment() & Qt::AlignHorizontal_Mask )
 	s.setWidth( QLAYOUTSIZE_MAX );
     if ( alignment() & Qt::AlignVertical_Mask )
 	s.setHeight( QLAYOUTSIZE_MAX );
-
     return s;
 }
 
@@ -1727,14 +1769,21 @@ bool QBoxLayout::hasHeightForWidth() const
 */
 int QBoxLayout::heightForWidth( int w ) const
 {
-    w -= 2*margin();
+    w -= 2 * margin();
     if ( data->dirty || w != data->hfwWidth ) {
 	QBoxLayout *that = (QBoxLayout*)this;
 	if ( data->dirty )
 	    that->setupGeom();
-	return that->calcHfw( w ) + 2*margin();
+	that->calcHfw( w );
     }
-    return data->hfwHeight + 2*margin();
+    return data->hfwHeight + 2 * margin();
+}
+
+/*! \internal */
+int QBoxLayout::minimumHeightForWidth( int w ) const
+{
+    (void) heightForWidth( w );
+    return data->hfwMinHeight + 2 * margin();
 }
 
 /*!
@@ -1751,11 +1800,14 @@ void QBoxLayout::invalidate()
 */
 QLayoutIterator QBoxLayout::iterator()
 {
-    return QLayoutIterator( new QBoxLayoutIterator( data ) );
+    return QLayoutIterator( new QBoxLayoutIterator(data) );
 }
 
 /*!
-    Returns the expansiveness of this layout.
+    Returns whether this layout can make use of more space than
+    sizeHint(). A value of \c Vertical or \c Horizontal means that it wants
+    to grow in only one dimension, whereas \c BothDirections means that
+    it wants to grow in both dimensions.
 */
 QSizePolicy::ExpandData QBoxLayout::expanding() const
 {
@@ -1950,7 +2002,7 @@ void QBoxLayout::insertLayout( int index, QLayout *layout, int stretch )
 void QBoxLayout::insertWidget( int index, QWidget *widget, int stretch,
 			       int alignment )
 {
-    if ( !checkWidget( this, widget ) )
+    if ( !checkWidget(this, widget) )
 	 return;
 
     if ( index < 0 )				// append
@@ -2130,8 +2182,8 @@ void QBoxLayout::setDirection( Direction direction )
 			//spacing or strut
 			QSize s = sp->sizeHint();
 			sp->changeSize( s.height(), s.width(),
-		horz(direction) ? QSizePolicy::Fixed:QSizePolicy::Minimum,
-		horz(direction) ? QSizePolicy::Minimum:QSizePolicy::Fixed );
+			    horz(direction) ? QSizePolicy::Fixed:QSizePolicy::Minimum,
+			    horz(direction) ? QSizePolicy::Minimum:QSizePolicy::Fixed );
 
 		    } else {
 			//stretch
@@ -2143,7 +2195,6 @@ void QBoxLayout::setDirection( Direction direction )
 					    QSizePolicy::Expanding );
 		    }
 		}
-
 	    }
 	}
     }
@@ -2212,7 +2263,7 @@ void QBoxLayout::setupGeom()
 	    a[i].expansive = expand;
 	    a[i].stretch = box->stretch ? box->stretch : box->hStretch();
 	} else {
-	    bool expand = exp & QSizePolicy::Vertically || box->stretch > 0;
+	    bool expand = ( exp & QSizePolicy::Vertically || box->stretch > 0 );
 	    verexp = verexp || expand;
 	    maxh += max.height() + space;
 	    minh += min.height() + space;
@@ -2240,11 +2291,9 @@ void QBoxLayout::setupGeom()
 			 | (verexp ? QSizePolicy::Vertically : 0) );
 
     data->minSize = QSize( minw, minh );
-    data->maxSize = QSize( maxw, maxh );
-    data->sizeHint = QSize( hintw, hinth );
-
-    data->maxSize = data->maxSize.expandedTo( data->minSize );
-    data->sizeHint = data->sizeHint.expandedTo( data->minSize )
+    data->maxSize = QSize( maxw, maxh ).expandedTo( data->minSize );
+    data->sizeHint = QSize( hintw, hinth )
+		     .expandedTo( data->minSize )
 		     .boundedTo( data->maxSize );
 
     data->dirty = FALSE;
@@ -2253,33 +2302,39 @@ void QBoxLayout::setupGeom()
 /*
   Calculates and stores the preferred height given the width \a w.
 */
-int QBoxLayout::calcHfw( int w )
+void QBoxLayout::calcHfw( int w )
 {
     int h = 0;
+    int mh = 0;
+
     if ( horz(dir) ) {
 	QMemArray<QLayoutStruct> &a = *data->geomArray;
 	int n = a.count();
 	qGeomCalc( a, 0, n, 0, w, spacing() );
 	for ( int i = 0; i < n; i++ ) {
 	    QBoxLayoutItem *box = data->list.at(i);
-	    h = QMAX( h, box->hfw( a[i].size ) );
+	    h = QMAX( h, box->hfw(a[i].size) );
+	    mh = QMAX( mh, box->mhfw(a[i].size) );
 	}
     } else {
 	QPtrListIterator<QBoxLayoutItem> it( data->list );
 	QBoxLayoutItem *box;
 	bool first = TRUE;
-	while ( (box=it.current()) != 0 ) {
+	while ( (box = it.current()) != 0 ) {
 	    ++it;
 	    bool empty = box->item->isEmpty();
 	    h += box->hfw( w );
-	    if ( !first && !empty )
+	    mh += box->mhfw( w );
+	    if ( !first && !empty ) {
 		h += spacing();
+		mh += spacing();
+	    }
 	    first = first && empty;
 	}
     }
-    data->hfwHeight = h;
     data->hfwWidth = w;
-    return h;
+    data->hfwHeight = h;
+    data->hfwMinHeight = mh;
 }
 
 /*!
@@ -2351,9 +2406,9 @@ QHBoxLayout::QHBoxLayout( QWidget *parent, int margin,
 */
 QHBoxLayout::QHBoxLayout( QLayout *parentLayout, int spacing,
 			  const char *name )
-    :QBoxLayout( parentLayout,
-		 QApplication::reverseLayout() ? RightToLeft : LeftToRight,
-		 spacing, name )
+    : QBoxLayout( parentLayout,
+		  QApplication::reverseLayout() ? RightToLeft : LeftToRight,
+		  spacing, name )
 {
 }
 
@@ -2374,7 +2429,7 @@ QHBoxLayout::QHBoxLayout( int spacing, const char *name )
 /*!
     Destroys this box layout.
 
-    The layout's widgets are \e not destroyed.
+    The layout's widgets aren't destroyed.
 */
 QHBoxLayout::~QHBoxLayout()
 {
@@ -2413,8 +2468,8 @@ QHBoxLayout::~QHBoxLayout()
     number of pixels between neighboring children. If \a spacing is -1
     the value of \a margin is used for \a spacing.
 */
-QVBoxLayout::QVBoxLayout( QWidget *parent, int margin,
-			  int spacing, const char *name )
+QVBoxLayout::QVBoxLayout( QWidget *parent, int margin, int spacing,
+			  const char *name )
     : QBoxLayout( parent, TopToBottom, margin, spacing, name )
 {
 
@@ -2430,7 +2485,7 @@ QVBoxLayout::QVBoxLayout( QWidget *parent, int margin,
 */
 QVBoxLayout::QVBoxLayout( QLayout *parentLayout, int spacing,
 			  const char *name )
-    :QBoxLayout( parentLayout, TopToBottom, spacing, name )
+    : QBoxLayout( parentLayout, TopToBottom, spacing, name )
 {
 
 }
@@ -2444,14 +2499,14 @@ QVBoxLayout::QVBoxLayout( QLayout *parentLayout, int spacing,
     parent's spacing().
 */
 QVBoxLayout::QVBoxLayout( int spacing, const char *name )
-    :QBoxLayout( TopToBottom, spacing, name )
+    : QBoxLayout( TopToBottom, spacing, name )
 {
 }
 
 /*!
     Destroys this box layout.
 
-    The layout's widgets are \e not destroyed.
+    The layout's widgets aren't destroyed.
 */
 QVBoxLayout::~QVBoxLayout()
 {
