@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qobject.cpp   3.1.2   edited Feb 19 21:25 $
 **
 ** Implementation of QObject class
 **
 ** Created : 930418
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -42,22 +42,34 @@
 #include "qsignalslotimp.h"
 #include "qregexp.h"
 #include "qmetaobject.h"
-#include "qpixmap.h"
-#include "qiconset.h"
-#include "qimage.h"
-#include "qregion.h"
-#include "qbitmap.h"
-#include "qpointarray.h"
-#include "qcursor.h"
-#include "qdatetime.h"
 #include <private/qucom_p.h>
+#include "qucomextra_p.h"
+#include "qptrvector.h"
+
+#ifdef QT_THREAD_SUPPORT
+#include <qmutex.h>
+#endif
 
 #include <ctype.h>
 
-class QObjectPrivate
-{
-};
 
+#ifndef QT_NO_USERDATA
+class QObjectPrivate : public QPtrVector<QObjectUserData>
+{
+public:
+    QObjectPrivate( uint s ) : QPtrVector<QObjectUserData>(s){ setAutoDelete( TRUE ); }
+};
+#else
+class QObjectPrivate {
+}
+#endif
+
+class QSenderObjectList : public QObjectList, public QShared
+{
+public:
+    QSenderObjectList():currentSender(0){}
+    QObject *currentSender;
+};
 
 /*!
     \class Qt qnamespace.h
@@ -95,11 +107,12 @@ class QObjectPrivate
 
     \ingroup objectmodel
     \mainclass
+    \reentrant
 
     QObject is the heart of the \link object.html Qt object model.
     \endlink The central feature in this model is a very powerful
     mechanism for seamless object communication called \link
-    signalsandslots.html signals and slots \endlink. You can can
+    signalsandslots.html signals and slots \endlink. You can
     connect a signal to a slot with connect() and destroy the
     connection with disconnect(). To avoid never ending notification
     loops you can temporarily block signals with blockSignals(). The
@@ -150,31 +163,6 @@ class QObjectPrivate
 */
 
 
-static QObject * sigSender = 0;
-
-/* (no '!' on purpose since this is an internal class)
-  \class QSenderObject qobject.h
-  \brief Internal object used for sending signals.
-
-  \internal
-
-  It is generally a very bad idea to use this class directly in
-  application programs.
-
-  In particular, you cannot not use it to send signals from classes
-  that do not inherit QObject. If you wish to do that, make an
-  internal class that inherits QObject and has the necessary signals
-  and slots. Alternatively, you can use the QSignal class.
-*/
-
-/* \internal */
-
-void QSenderObject::setSender( QObject *s )
-{
-    sigSender=s;
-}
-
-
 //
 // Remove white space from SIGNAL and SLOT names.
 // Internal for QObject::connect() and QObject::disconnect()
@@ -215,6 +203,7 @@ static QCString qt_rmWS( const char *s )
 	if ( *s && isIdentChar(*s) && isIdentChar(last) )
 	    last = *d++ = ' ';
     }
+    *d = '\0';
     result.truncate( (int)(d - result.data()) );
     int void_pos = result.find("(void)");
     if ( void_pos >= 0 )
@@ -229,8 +218,6 @@ int   qStartTimer( int interval, QObject *obj );
 bool  qKillTimer( int id );
 bool  qKillTimer( QObject *obj );
 
-void  qRemovePostedEvents( QObject* );
-
 static void removeObjFromList( QObjectList *objList, const QObject *obj,
 			       bool single=FALSE )
 {
@@ -280,30 +267,47 @@ void *qt_find_obj_child( QObject *parent, const char *type, const char *name )
 
 
 static QObjectList* object_trees = 0;
+
+#ifdef QT_THREAD_SUPPORT
+static QMutex *obj_trees_mutex = 0;
+#endif
+
 static void cleanup_object_trees()
 {
     delete object_trees;
     object_trees = 0;
+#ifdef QT_THREAD_SUPPORT
+    delete obj_trees_mutex;
+    obj_trees_mutex = 0;
+#endif
 }
 
 static void ensure_object_trees()
 {
-    if ( object_trees )
-	return;
     object_trees = new QObjectList;
     qAddPostRoutine( cleanup_object_trees );
 }
 
 static void insert_tree( QObject* obj )
 {
-    ensure_object_trees();
+#ifdef QT_THREAD_SUPPORT
+    if ( !obj_trees_mutex )
+	obj_trees_mutex = new QMutex();
+    QMutexLocker locker( obj_trees_mutex );
+#endif
+    if ( !object_trees )
+	ensure_object_trees();
     object_trees->insert(0, obj );
 }
 
 static void remove_tree( QObject* obj )
 {
-    if ( object_trees )
+    if ( object_trees ) {
+#ifdef QT_THREAD_SUPPORT
+	QMutexLocker locker( obj_trees_mutex );
+#endif
 	object_trees->removeRef( obj );
+    }
 }
 
 
@@ -333,32 +337,31 @@ static void remove_tree( QObject* obj )
 */
 
 QObject::QObject( QObject *parent, const char *name )
-    : d( 0 )
+    :
+    isSignal( FALSE ),				// assume not a signal object
+    isWidget( FALSE ), 				// assume not a widget object
+    pendTimer( FALSE ),				// no timers yet
+    blockSig( FALSE ),      			// not blocking signals
+    wasDeleted( FALSE ),       			// double-delete catcher
+    isTree( FALSE ), 				// no tree yet
+    objname( name ? qstrdup(name) : 0 ),        // set object name
+    parentObj( 0 ),				// no parent yet. It is set by insertChild()
+    childObjects( 0 ), 				// no children yet
+    connections( 0 ),				// no connections yet
+    senderObjects( 0 ),        			// no signals connected yet
+    eventFilters( 0 ), 				// no filters installed
+    postedEvents( 0 ), 				// no events posted
+    d( 0 )
 {
     if ( !metaObj )				// will create object dict
 	(void) staticMetaObject();
-    objname       = name ? qstrdup(name) : 0;   // set object name
-    childObjects  = 0;				// no children yet
-    connections   = 0;				// no connections yet
-    senderObjects = 0;				// no signals connected yet
-    eventFilters  = 0;				// no filters installed
-    postedEvents  = 0;				// no events posted
-    isSignal   = FALSE;				// assume not a signal object
-    isWidget   = FALSE;				// assume not a widget object
-    pendTimer  = FALSE;				// no timers yet
-    blockSig   = FALSE;				// not blocking signals
-    wasDeleted = FALSE;				// double-delete catcher
-    isTree = FALSE;				// no tree yet
-    parentObj  = parent;			// to avoid root checking in insertChild()
+
     if ( parent ) {				// add object to parent
 	parent->insertChild( this );
     } else {
 	insert_tree( this );
 	isTree = TRUE;
     }
-
-    // null out sigSender... this may be wrong.
-    sigSender = 0;
 }
 
 
@@ -373,6 +376,13 @@ QObject::QObject( QObject *parent, const char *name )
     outside the parent. If you still do, the QObject::destroyed()
     signal gives you an opportunity to detect when an object is
     destroyed.
+
+    \warning Deleting a QObject while pending events are waiting to be
+    delivered can cause a crash.  You must not delete the QObject
+    directly from a thread that is not the GUI thread.  Use the
+    QObject::deleteLater() method instead, which will cause the event
+    loop to delete the object after all pending events have been
+    delivered to the object.
 */
 
 QObject::~QObject()
@@ -384,6 +394,7 @@ QObject::~QObject()
 	return;
     }
     wasDeleted = 1;
+    blockSig = 0; // unblock signals to keep QGuardedPtr happy
     emit destroyed( this );
     emit destroyed();
     if ( objname )
@@ -391,8 +402,7 @@ QObject::~QObject()
     objname = 0;
     if ( pendTimer )				// might be pending timers
 	qKillTimer( this );
-    if ( postedEvents && qApp )
-	QApplication::removePostedEvents( this );
+    QApplication::removePostedEvents( this );
     if ( isTree ) {
 	remove_tree( this );		// remove from global root list
 	isTree = FALSE;
@@ -401,15 +411,15 @@ QObject::~QObject()
 	parentObj->removeChild( this );
     register QObject *obj;
     if ( senderObjects ) {			// disconnect from senders
-	QObjectList *tmp = senderObjects;
+	QSenderObjectList *tmp = senderObjects;
 	senderObjects = 0;
 	obj = tmp->first();
 	while ( obj ) {				// for all senders...
 	    obj->disconnect( this );
 	    obj = tmp->next();
 	}
-	delete tmp;
-	senderObjects = 0;
+	if ( tmp->deref() )
+	    delete tmp;
     }
     if ( connections ) {			// disconnect receivers
 	for ( int i = 0; i < (int) connections->size(); i++ ) {
@@ -441,6 +451,9 @@ QObject::~QObject()
 	}
 	delete childObjects;
     }
+
+    if ( d )
+	delete d;
 }
 
 
@@ -749,6 +762,49 @@ void QObject::customEvent( QCustomEvent * )
     the event \a e, out, i.e. stop it being handled further, return
     TRUE; otherwise return FALSE.
 
+    Example:
+    \code
+    class MyMainWindow : public QMainWindow
+    {
+    public:
+	MyMainWindow( QWidget *parent = 0, const char *name = 0 );
+
+    protected:
+	bool eventFilter( QObject *obj, QEvent *ev );
+
+    private:
+	QTextEdit *textEdit;
+    };
+
+    MyMainWindow::MyMainWindow( QWidget *parent, const char *name )
+	: QMainWindow( parent, name )
+    {
+	textEdit = new QTextEdit( this );
+	setCentralWidget( textEdit );
+	textEdit->installEventFilter( this );
+    }
+
+    bool MyMainWindow::eventFilter( QObject *obj, QEvent *ev )
+    {
+	if ( obj == textEdit ) {
+	    if ( e->type() == QEvent::KeyPress ) {
+		qDebug( "Ate key press %d", k->key() );
+		return TRUE;
+	    } else {
+		return FALSE;
+	    }
+	} else {
+	    // pass the event on to the parent class
+	    return QMainWindow::eventFilter( obj, ev );
+	}
+    }
+    \endcode
+
+    Notice in the example above that unhandled events are passed to
+    the base class's eventFilter() function, since the base class
+    might have reimplemented eventFilter() for its own internal
+    purposes.
+
     \warning If you delete the receiver object in this function, be
     sure to return TRUE. Otherwise, Qt will forward the event to the
     deleted object and the program might crash.
@@ -800,6 +856,8 @@ bool QObject::activate_filters( QEvent *e )
     block is FALSE.
 
     Emitted signals disappear into hyperspace if signals are blocked.
+    Note that the destroyed() signals will be emitted even if the signals
+    for this object have been blocked.
 */
 
 void QObject::blockSignals( bool block )
@@ -1124,14 +1182,13 @@ void QObject::insertChild( QObject *obj )
     if ( !childObjects ) {
 	childObjects = new QObjectList;
 	Q_CHECK_PTR( childObjects );
-    }
+    } else if ( obj->parentObj == this ) {
 #if defined(QT_CHECK_STATE)
-    else if ( childObjects->findRef(obj) >= 0 ) {
 	qWarning( "QObject::insertChild: Object %s::%s already in list",
 		 obj->className(), obj->name( "unnamed" ) );
+#endif
 	return;
     }
-#endif
     obj->parentObj = this;
     childObjects->append( obj );
 
@@ -1170,41 +1227,34 @@ void QObject::removeChild( QObject *obj )
 
 
 /*!
-    Installs an event filter \a obj on this object.
+    \fn void QObject::installEventFilter( const QObject *filterObj )
+
+    Installs an event filter \a filterObj on this object. For example:
+    \code
+    monitoredObj->installEventFilter( filterObj );
+    \endcode
 
     An event filter is an object that receives all events that are
     sent to this object. The filter can either stop the event or
-    forward it to this object. The event filter \a obj receives events
-    via its eventFilter() function. The eventFilter() function must
-    return TRUE if the event should be filtered, (i.e. stopped);
+    forward it to this object. The event filter \a filterObj receives
+    events via its eventFilter() function. The eventFilter() function
+    must return TRUE if the event should be filtered, (i.e. stopped);
     otherwise it must return FALSE.
 
     If multiple event filters are installed on a single object, the
     filter that was installed last is activated first.
 
-    Example:
+    Here's a \c KeyPressEater class that eats the key presses of its
+    monitored objects:
     \code
-    #include <qwidget.h>
-
-    class MyWidget : public QWidget
+    class KeyPressEater : public QObject
     {
-	Q_OBJECT
-    public:
-	MyWidget( QWidget *parent = 0, const char *name = 0 );
-
+	...
     protected:
-	bool eventFilter( QObject *, QEvent * );
+	bool eventFilter( QObject *o, QEvent *e );
     };
 
-    MyWidget::MyWidget( QWidget *parent, const char *name )
-	: QWidget( parent, name )
-    {
-	// install a filter on the parent (if any)
-	if ( parent )
-	    parent->installEventFilter( this );
-    }
-
-    bool MyWidget::eventFilter( QObject *o, QEvent *e )
+    bool KeyPressEater::eventFilter( QObject *o, QEvent *e )
     {
 	if ( e->type() == QEvent::KeyPress ) {
 	    // special processing for key press
@@ -1213,11 +1263,21 @@ void QObject::removeChild( QObject *obj )
 	    return TRUE; // eat event
 	} else {
 	    // standard event processing
-	    return QWidget::eventFilter( o, e );
+	    return FALSE;
 	}
     }
     \endcode
 
+    And here's how to install it on two widgets:
+    \code
+	KeyPressEater *keyPressEater = new KeyPressEater( this );
+	QPushButton *pushButton = new QPushButton( this );
+	QListView *listView = new QListView( this );
+
+	pushButton->installEventFilter( keyPressEater );
+	listView->installEventFilter( keyPressEater );
+    \endcode
+
     The QAccel class, for example, uses this technique to intercept
     accelerator key presses.
 
@@ -1366,8 +1426,7 @@ static void err_info_about_candidates( int code,
 
 /*!
     Returns a pointer to the object that sent the signal, if called in
-    a slot before any function call or signal emission. Returns an
-    undefined value in all other cases.
+    a slot activated by a signal; otherwise the return value is undefined.
 
     \warning This function will return something apparently correct in
     other cases as well. However, its value may change during any
@@ -1384,7 +1443,16 @@ static void err_info_about_candidates( int code,
 
 const QObject *QObject::sender()
 {
-    return sigSender;
+    if ( senderObjects &&
+	 senderObjects->currentSender &&
+	 /*currentSender may be a dangling pointer in case the object
+	  * it was pointing to was destructed from inside a slot. Thus
+	  * verify it still is contained inside the senderObjects list
+	  * which gets cleaned on both destruction and disconnect.
+	  */
+	 senderObjects->findRef( senderObjects->currentSender ) != -1 )
+	return senderObjects->currentSender;
+    return 0;
 }
 
 
@@ -1502,7 +1570,11 @@ QCString QObject::normalizeSignalSlot( const char *signalSlot )
     \endcode
 
     This example ensures that the label always displays the current
-    scroll bar value.
+    scroll bar value. Note that the signal and slots parameters must not
+    contain any variable names, only the type. E.g. the following would
+    not work and return FALSE:
+    QObject::connect( scroll, SIGNAL(valueChanged(int v)),
+                      label,  SLOT(setNum(int v)) );
 
     A signal can also be connected to another signal:
 
@@ -1543,6 +1615,10 @@ QCString QObject::normalizeSignalSlot( const char *signalSlot )
     existence of either \a signal or \a member, or if their signatures
     aren't compatible.
 
+    A signal is emitted for \e{every} connection you make, so if you
+    duplicate a connection, two signals will be emitted. You can
+    always break a connection using \c{disconnect()}.
+
     \sa disconnect()
 */
 
@@ -1635,6 +1711,11 @@ bool QObject::connect( const QObject *sender,	const char *signal,
 		    ri++;
 		else if ( !QUType::isEqual( sm->method->parameters[si++].type,
 					    rm->method->parameters[ri++].type ) ) {
+		    if ( ( QUType::isEqual( sm->method->parameters[si-1].type, &static_QUType_ptr )
+			 && QUType::isEqual( rm->method->parameters[ri-1].type, &static_QUType_varptr ) )
+			|| ( QUType::isEqual( sm->method->parameters[si-1].type, &static_QUType_varptr )
+			     && QUType::isEqual( rm->method->parameters[ri-1].type, &static_QUType_ptr ) ) )
+			continue; // varptr got introduced in 3.1 and is binary compatible with ptr
 		    qWarning( "QObject::connect: Incompatible sender/receiver marshalling"
 			      "\n\t%s::%s --> %s::%s",
 			      s->className(), signal,
@@ -1658,8 +1739,8 @@ void QObject::connectInternal( const QObject *sender, int signal_index, const QO
     QObject *s = (QObject*)sender;
     QObject *r = (QObject*)receiver;
 
-    if ( !s->connections ) {			// create connections dict
-	s->connections = new QSignalVec( 7 );
+    if ( !s->connections ) {			// create connections lookup table
+	s->connections = new QSignalVec( signal_index+1 );
 	Q_CHECK_PTR( s->connections );
 	s->connections->setAutoDelete( TRUE );
     }
@@ -1687,10 +1768,8 @@ void QObject::connectInternal( const QObject *sender, int signal_index, const QO
     QConnection *c = new QConnection( r, member_index, rm ? rm->name : "qt_invoke", membcode );
     Q_CHECK_PTR( c );
     clist->append( c );
-    if ( !r->senderObjects ) {			// create list of senders
-	r->senderObjects = new QObjectList;
-	Q_CHECK_PTR( r->senderObjects );
-    }
+    if ( !r->senderObjects )			// create list of senders
+	r->senderObjects = new QSenderObjectList;
     r->senderObjects->append( s );		// add sender to list
 }
 
@@ -1785,13 +1864,14 @@ bool QObject::disconnect( const QObject *sender,   const char *signal,
 	return FALSE;
     QCString signal_name;
     QCString member_name;
-    const QMetaData *rm = 0;
     QObject *s = (QObject *)sender;
     QObject *r = (QObject *)receiver;
+    int member_index = -1;
+    int membcode = -1;
     if ( member ) {
 	member_name = qt_rmWS( member );
 	member = member_name.data();
-	int membcode = member[0] - '0';
+	membcode = member[0] - '0';
 #if defined(QT_CHECK_RANGE)
 	if ( !check_member_code( membcode, r, member, "disconnect" ) )
 	    return FALSE;
@@ -1800,14 +1880,14 @@ bool QObject::disconnect( const QObject *sender,   const char *signal,
 	QMetaObject *rmeta = r->metaObject();
 
 	switch ( membcode ) {			// get receiver member
-	    case QSLOT_CODE:
-		rm = rmeta->slot( rmeta->findSlot( member, TRUE ), TRUE );
-		break;
-	    case QSIGNAL_CODE:
-		rm = rmeta->signal( rmeta->findSignal( member, TRUE ), TRUE );
-		break;
+	case QSLOT_CODE:
+	    member_index = rmeta->findSlot( member, TRUE );
+	    break;
+	case QSIGNAL_CODE:
+	    member_index = rmeta->findSignal( member, TRUE );
+	    break;
 	}
-	if ( !rm ) {				// no such member
+	if ( member_index < 0 ) {				// no such member
 #if defined(QT_CHECK_RANGE)
 	    err_member_notfound( membcode, r, member, "disconnect" );
 	    err_info_about_candidates( membcode, rmeta, member, "connect" );
@@ -1817,32 +1897,11 @@ bool QObject::disconnect( const QObject *sender,   const char *signal,
 	}
     }
 
-    QConnectionList *clist;
-    register QConnection *c;
     if ( signal == 0 ) {			// any/all signals
-	for ( int i = 0; i < (int) s->connections->size(); i++ ) {
-	    clist = (*s->connections)[i]; // for all signals...
-	    if ( !clist )
-		continue;
-	    c = clist->first();
-	    while ( c ) {			// for all receivers...
-		if ( r == 0 ) {			// remove all receivers
-		    removeObjFromList( c->object()->senderObjects, s );
-		    c = clist->next();
-		} else if ( r == c->object() &&
-			    (member == 0 ||
-			     qstrcmp(member,c->memberName()) == 0) ) {
-		    removeObjFromList( c->object()->senderObjects, s );
-		    clist->remove();
-		    c = clist->current();
-		} else {
-		    c = clist->next();
-		}
-	    }
-	    if ( r == 0 )			// disconnect all receivers
-		s->connections->insert( i, 0 );
-	}
-	s->disconnectNotify( 0 );
+	if ( disconnectInternal( s, -1, r, membcode, member_index ) )
+	    s->disconnectNotify( 0 );
+	else
+	    return FALSE;
     } else {					// specific signal
 	signal_name = qt_rmWS( signal );
 	signal = signal_name.data();
@@ -1863,6 +1922,67 @@ bool QObject::disconnect( const QObject *sender,   const char *signal,
 #endif
 		return FALSE;
 	}
+
+	/* compatibility and safety: If a receiver has several slots
+	 * with the same name, disconnect them all*/
+	bool res = FALSE;
+	if ( membcode == QSLOT_CODE && r ) {
+	    QMetaObject * rmeta = r->metaObject();
+	    do {
+		int mi = rmeta->findSlot( member );
+		if ( mi != -1 )
+		    res |= disconnectInternal( s, signal_index, r, membcode,  mi );
+	    } while ( (rmeta = rmeta->superClass()) );
+	} else {
+	    res = disconnectInternal( s, signal_index, r, membcode,  member_index );
+	}
+	if ( res )
+	    s->disconnectNotify( signal_name );
+	return res;
+    }
+    return TRUE;
+}
+
+/*! \internal */
+
+bool QObject::disconnectInternal( const QObject *sender, int signal_index,
+		  const QObject *receiver, int membcode, int member_index )
+{
+    QObject *s = (QObject*)sender;
+    QObject *r = (QObject*)receiver;
+
+    if ( !s->connections )
+	return FALSE;
+
+    bool success = FALSE;
+    QConnectionList *clist;
+    register QConnection *c;
+    if ( signal_index == -1 ) {
+	for ( int i = 0; i < (int) s->connections->size(); i++ ) {
+	    clist = (*s->connections)[i]; // for all signals...
+	    if ( !clist )
+		continue;
+	    c = clist->first();
+	    while ( c ) {			// for all receivers...
+		if ( r == 0 ) {			// remove all receivers
+		    removeObjFromList( c->object()->senderObjects, s );
+		    success = TRUE;
+		    c = clist->next();
+		} else if ( r == c->object() &&
+			    ( member_index == -1 ||
+			      member_index == c->member() && c->memberType() == membcode ) ) {
+		    removeObjFromList( c->object()->senderObjects, s );
+		    success = TRUE;
+		    clist->remove();
+		    c = clist->current();
+		} else {
+		    c = clist->next();
+		}
+	    }
+	    if ( r == 0 )			// disconnect all receivers
+		s->connections->insert( i, 0 );
+	}
+    } else {
 	clist = s->connections->at( signal_index );
 	if ( !clist )
 	    return FALSE;
@@ -1871,10 +1991,13 @@ bool QObject::disconnect( const QObject *sender,   const char *signal,
 	while ( c ) {				// for all receivers...
 	    if ( r == 0 ) {			// remove all receivers
 		removeObjFromList( c->object()->senderObjects, s, TRUE );
+		success = TRUE;
 		c = clist->next();
-	    } else if ( r == c->object() && (member == 0 ||
-				      qstrcmp(member,c->memberName()) == 0) ) {
+	    } else if ( r == c->object() &&
+			( member_index == -1 ||
+			  member_index == c->member() && c->memberType() == membcode ) ) {
 		removeObjFromList( c->object()->senderObjects, s, TRUE );
+		success = TRUE;
 		clist->remove();
 		c = clist->current();
 	    } else {
@@ -1883,16 +2006,19 @@ bool QObject::disconnect( const QObject *sender,   const char *signal,
 	}
 	if ( r == 0 )				// disconnect all receivers
 	    s->connections->insert( signal_index, 0 );
-	s->disconnectNotify( signal_name );
     }
-    return TRUE;
+    return success;
 }
 
-
 /*!
     \fn QObject::destroyed()
 
-    This signal is emitted immediately before the object is destroyed.
+    This signal is emitted when the object is being destroyed.
+
+    Note that the signal is emitted by the QObject destructor, so
+    the object's virtual table is already degenerated at this point,
+    and it is not safe to call any functions on the object emitting
+    the signal. This signal can not be blocked.
 
     All the objects's children are destroyed immediately after this
     signal is emitted.
@@ -1902,7 +2028,7 @@ bool QObject::disconnect( const QObject *sender,   const char *signal,
     \overload QObject::destroyed( QObject* obj)
 
     This signal is emitted immediately before the object \a obj is
-    destroyed.
+    destroyed, and can not be blocked.
 
     All the objects's children are destroyed immediately after this
     signal is emitted.
@@ -1933,32 +2059,44 @@ void QObject::cleanupEventFilter(QObject* obj)
 
 
 /*!
-    \fn const char *QObject::tr( const char *sourceText, const char * comment ) const
+    \fn QString QObject::tr( const char *sourceText, const char * comment )
+    \reentrant
 
     Returns a translated version of \a sourceText, or \a sourceText
     itself if there is no appropriate translated version. The
-    translation context is QObject with \a comment (null by default).
+    translation context is QObject with \a comment (0 by default).
     All QObject subclasses using the Q_OBJECT macro automatically have
     a reimplementation of this function with the subclass name as
     context.
 
+    \warning This method is reentrant only if all translators are
+    installed \e before calling this method. Installing or removing
+    translators while performing translations is not supported. Doing
+    so will probably result in crashes or other undesirable behavior.
+
     \sa trUtf8() QApplication::translate()
 	\link i18n.html Internationalization with Qt\endlink
 */
 
 /*!
-    \fn const char *QObject::trUtf8( const char *sourceText,
-				     const char *comment ) const
+    \fn QString QObject::trUtf8( const char *sourceText,
+                                 const char *comment )
+    \reentrant
 
     Returns a translated version of \a sourceText, or
     QString::fromUtf8(\a sourceText) if there is no appropriate
     version. It is otherwise identical to tr(\a sourceText, \a
     comment).
 
+    \warning This method is reentrant only if all translators are
+    installed \e before calling this method. Installing or removing
+    translators while performing translations is not supported. Doing
+    so will probably result in crashes or other undesirable behavior.
+
     \sa tr() QApplication::translate()
 */
 
-static QMetaObjectCleanUp cleanUp_Qt = QMetaObjectCleanUp();
+static QMetaObjectCleanUp cleanUp_Qt = QMetaObjectCleanUp( "QObject", &QObject::staticMetaObject );
 
 QMetaObject* QObject::staticQtMetaObject()
 {
@@ -1988,7 +2126,8 @@ QMetaObject* QObject::staticQtMetaObject()
     static const QMetaEnum::Item enum_2[] = {
 	{ "PlainText", (int) Qt::PlainText },
 	{ "RichText", (int) Qt::RichText },
-	{ "AutoText", (int) Qt::AutoText }
+	{ "AutoText", (int) Qt::AutoText },
+	{ "LogText", (int) Qt::LogText }
     };
 
     static const QMetaEnum::Item enum_3[] = {
@@ -2069,25 +2208,47 @@ void QObject::activate_signal( QConnectionList *clist, QUObject *o )
 	return;
 
     QObject *object;
+    QSenderObjectList* sol;
+    QObject* oldSender = 0;
     QConnection *c;
     if ( clist->count() == 1 ) { // save iterator
 	c = clist->first();
 	object = c->object();
-	sigSender = this;
+	sol = object->senderObjects;
+	if ( sol ) {
+	    oldSender = sol->currentSender;
+	    sol->ref();
+	    sol->currentSender = this;
+	}
 	if ( c->memberType() == QSIGNAL_CODE )
 	    object->qt_emit( c->member(), o );
 	else
 	    object->qt_invoke( c->member(), o );
+	if (sol ) {
+	    sol->currentSender = oldSender;
+	    if ( sol->deref() )
+		delete sol;
+	}
     } else {
 	QConnectionListIt it(*clist);
 	while ( (c=it.current()) ) {
 	    ++it;
 	    object = c->object();
-	    sigSender = this;
+	    sol = object->senderObjects;
+	    if ( sol ) {
+		oldSender = sol->currentSender;
+		sol->ref();
+		sol->currentSender = this;
+	    }
 	    if ( c->memberType() == QSIGNAL_CODE )
 		object->qt_emit( c->member(), o );
 	    else
 		object->qt_invoke( c->member(), o );
+	    if (sol ) {
+		sol->currentSender = oldSender;
+		if ( sol->deref() )
+		    delete sol;
+	    }
 	}
     }
 }
@@ -2263,8 +2424,11 @@ bool QObject::setProperty( const char *name, const QVariant& value )
 	return FALSE;
     int id = meta->findProperty( name, TRUE );
     const QMetaProperty* p = meta->property( id, TRUE );
-    if ( !p || !p->writable() )
+    if ( !p || !p->isValid() || !p->writable() ) {
+	qWarning( "%s::setProperty( \"%s\", value ) failed: property invalid, read-only or does not exist",
+		  className(), name );
 	return FALSE;
+    }
 
     if ( p->isEnumType() ) {
 	if ( v.type() == QVariant::String || v.type() == QVariant::CString ) {
@@ -2285,8 +2449,10 @@ bool QObject::setProperty( const char *name, const QVariant& value )
 	return qt_property( id, 0, &v );
     }
 
-    QVariant::Type type = QVariant::nameToType( p->type() );
-    if ( !(type == QVariant::Invalid && !qstrcmp( p->type(), "QVariant" )) && !v.canCast( type ) )
+    QVariant::Type type = (QVariant::Type)(p->flags >> 24);
+    if ( type == QVariant::Invalid )
+	type = QVariant::nameToType( p->type() );
+    if ( type != QVariant::Invalid && !v.canCast( type ) )
 	return FALSE;
     return qt_property( id, 0, &v );
 }
@@ -2310,8 +2476,11 @@ QVariant QObject::property( const char *name ) const
 	return v;
     int id = meta->findProperty( name, TRUE );
     const QMetaProperty* p = meta->property( id, TRUE );
-    if ( !p )
+    if ( !p || !p->isValid() ) {
+	qWarning( "%s::property( \"%s\" ) failed: property invalid or does not exist",
+		  className(), name );
 	return v;
+    }
     QObject* that = (QObject*) this; // moc ensures constness for the qt_property call
     that->qt_property( id, 1, &v );
     return v;
@@ -2319,3 +2488,40 @@ QVariant QObject::property( const char *name ) const
 
 #endif // QT_NO_PROPERTIES
 
+#ifndef QT_NO_USERDATA
+/*!\internal
+ */
+uint QObject::registerUserData()
+{
+    static int user_data_registration = 0;
+    return user_data_registration++;
+}
+
+/*!\internal
+ */
+QObjectUserData::~QObjectUserData()
+{
+}
+
+/*!\internal
+ */
+void QObject::setUserData( uint id, QObjectUserData* data)
+{
+    if ( !d )
+	d = new QObjectPrivate( id+1 );
+    if ( id >= d->size() )
+	d->resize( id+1 );
+    d->insert( id, data );
+}
+
+/*!\internal
+ */
+QObjectUserData* QObject::userData( uint id ) const
+{
+    if ( d && id < d->size() )
+	return d->at( id );
+    return 0;
+}
+
+#endif // QT_NO_USERDATA
+
