@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qpaintdevice_x11.cpp   3.0.5   edited Apr 23 14:15 $
+** $Id: qt/qpaintdevice_x11.cpp   3.1.2   edited Nov 7 16:55 $
 **
 ** Implementation of QPaintDevice class for X11
 **
@@ -97,12 +97,23 @@
 
 Display *QPaintDevice::x_appdisplay = 0;
 int	 QPaintDevice::x_appscreen;
-int	 QPaintDevice::x_appdepth;
-int	 QPaintDevice::x_appcells;
-Qt::HANDLE QPaintDevice::x_appcolormap;
-bool	 QPaintDevice::x_appdefcolormap;
-void	*QPaintDevice::x_appvisual;
-bool	 QPaintDevice::x_appdefvisual;
+
+int	  QPaintDevice::x_appdepth;
+int	  QPaintDevice::x_appcells;
+Qt::HANDLE  QPaintDevice::x_approotwindow;
+Qt::HANDLE  QPaintDevice::x_appcolormap;
+bool	  QPaintDevice::x_appdefcolormap;
+void	 *QPaintDevice::x_appvisual;
+bool	  QPaintDevice::x_appdefvisual;
+
+// ### in 4.0, remove the above, and use the below
+int	 *QPaintDevice::x_appdepth_arr;
+int	 *QPaintDevice::x_appcells_arr;
+Qt::HANDLE *QPaintDevice::x_approotwindow_arr;
+Qt::HANDLE *QPaintDevice::x_appcolormap_arr;
+bool	 *QPaintDevice::x_appdefcolormap_arr;
+void	**QPaintDevice::x_appvisual_arr;
+bool	 *QPaintDevice::x_appdefvisual_arr;
 
 /*!
     \enum QPaintDevice::PDevCmd
@@ -266,6 +277,13 @@ Qt::HANDLE QPaintDevice::handle() const
 }
 
 
+#ifndef QT_NO_XFTFREETYPE
+#ifndef QT_XFT2
+// Xft1 doesn't have XftDrawPicture, so we fake it in qtaddons_x11.cpp
+extern "C" Qt::HANDLE XftDrawPicture( XftDraw * );
+#endif // QT_XFT2
+#endif // QT_NO_XFTFREETYPE
+
 /*!
     Returns the window system handle of the paint device for XRender
     support. Use of this function is not portable. This function will
@@ -275,7 +293,11 @@ Qt::HANDLE QPaintDevice::handle() const
 */
 Qt::HANDLE QPaintDevice::x11RenderHandle() const
 {
-    return rendhd;
+#ifndef QT_NO_XFTFREETYPE
+    return rendhd ? XftDrawPicture( (XftDraw *) rendhd ) : 0;
+#else
+    return 0;
+#endif // QT_NO_XFTFREETYPE
 }
 
 
@@ -296,10 +318,12 @@ Qt::HANDLE QPaintDevice::x11RenderHandle() const
 */
 
 /*!
+    \overload
     \fn int QPaintDevice::x11AppDepth()
 
-    Returns the depth of the X display global to the application (X11
-    only). Using this function is not portable.
+    Returns the depth for the default screen of the X display global
+    to the application (X11 only). Using this function is not
+    portable.
 
     \sa QPixmap::defaultDepth()
 */
@@ -307,18 +331,27 @@ Qt::HANDLE QPaintDevice::x11RenderHandle() const
 /*!
     \fn int QPaintDevice::x11AppCells()
 
-    Returns the number of entries in the colormap of the X display
-    global to the application (X11 only). Using this function is not
-    portable.
+    Returns the number of entries in the colormap for the default
+    screen of the X display global to the application (X11
+    only). Using this function is not portable.
 
     \sa x11Colormap()
 */
 
+/*!
+    \fn HANDLE QPaintDevice::x11AppRootWindow()
+
+    Returns the root window for the default screen of the X display
+    global to the applicatoin (X11 only). Using this function is not
+    portable.
+*/
+
 /*!
     \fn HANDLE QPaintDevice::x11AppColormap()
 
-    Returns the colormap of the X display global to the application
-    (X11 only). Using this function is not portable.
+    Returns the colormap for the default screen of the X display
+    global to the application (X11 only). Using this function is not
+    portable.
 
     \sa x11Cells()
 */
@@ -326,8 +359,9 @@ Qt::HANDLE QPaintDevice::x11RenderHandle() const
 /*!
     \fn bool QPaintDevice::x11AppDefaultColormap ()
 
-    Returns the default colormap of the X display global to the
-    application (X11 only). Using this function is not portable.
+    Returns the default colormap for the default screen of the X
+    display global to the application (X11 only). Using this function
+    is not portable.
 
     \sa x11Cells()
 */
@@ -335,15 +369,85 @@ Qt::HANDLE QPaintDevice::x11RenderHandle() const
 /*!
     \fn void* QPaintDevice::x11AppVisual ()
 
-    Returns the Visual of the X display global to the application (X11
-    only). Using this function is not portable.
+    Returns the Visual for the default screen of the X display global
+    to the application (X11 only). Using this function is not
+    portable.
 */
 
 /*!
     \fn bool QPaintDevice::x11AppDefaultVisual ()
 
-    Returns the default Visual of the X display global to the
-    application (X11 only). Using this function is not portable.
+    Returns TRUE if the Visual used is the default for the default
+    screen of the X display global to the application (X11 only);
+    otherwise returns FALSE. Using this function is not portable.
+*/
+
+/*!
+    \fn int QPaintDevice::x11AppDepth( int screen )
+
+    Returns the depth for screen \a screen of the X display global to
+    the application (X11 only). Using this function is not portable.
+
+    \sa QPixmap::defaultDepth()
+*/
+
+/*!
+    \overload
+    \fn int QPaintDevice::x11AppCells( int screen )
+
+    Returns the number of entries in the colormap for screen \a screen
+    of the X display global to the application (X11 only). Using this
+    function is not portable.
+
+    \sa x11Colormap()
+*/
+
+/*!
+    \overload
+    \fn HANDLE QPaintDevice::x11AppRootWindow( int screen )
+
+    Returns the root window for screen \a screen of the X display
+    global to the applicatoin (X11 only). Using this function is not
+    portable.
+*/
+
+/*!
+    \overload
+    \fn HANDLE QPaintDevice::x11AppColormap( int screen )
+
+    Returns the colormap for screen \a screen of the X display global
+    to the application (X11 only). Using this function is not
+    portable.
+
+    \sa x11Cells()
+*/
+
+/*!
+    \overload
+    \fn bool QPaintDevice::x11AppDefaultColormap( int screen )
+
+    Returns the default colormap for screen \a screen of the X display
+    global to the application (X11 only). Using this function is not
+    portable.
+
+    \sa x11Cells()
+*/
+
+/*!
+    \overload
+    \fn void* QPaintDevice::x11AppVisual( int screen )
+
+    Returns the Visual for screen \a screen of the X display global to
+    the application (X11 only). Using this function is not portable.
+*/
+
+/*!
+    \overload
+    \fn bool QPaintDevice::x11AppDefaultVisual( int screen )
+
+    Returns TRUE if the Visual used is the default for screen
+    \a screen of the X display global to the application (X11 only);
+    otherwise returns FALSE. Using this function is not portable.
 */
 
 
@@ -413,81 +517,174 @@ Qt::HANDLE QPaintDevice::x11RenderHandle() const
     (X11 only). Using this function is not portable.
 */
 
-static int dpiX=0,dpiY=0;
+static int *dpisX=0, *dpisY=0;
+static void create_dpis()
+{
+    if ( dpisX )
+	return;
+
+    Display *dpy = QPaintDevice::x11AppDisplay();
+    if ( ! dpy )
+	return;
+
+    int i, screens =  ScreenCount( dpy );
+    dpisX = new int[ screens ];
+    dpisY = new int[ screens ];
+    Q_CHECK_PTR( dpisX );
+    Q_CHECK_PTR( dpisY );
+    for ( i = 0; i < screens; i++ ) {
+	dpisX[ i ] = (DisplayWidth(dpy,i) * 254 + DisplayWidthMM(dpy,i)*5)
+
+		     / (DisplayWidthMM(dpy,i)*10);
+	dpisY[ i ] = (DisplayHeight(dpy,i) * 254 + DisplayHeightMM(dpy,i)*5)
+		     / (DisplayHeightMM(dpy,i)*10);
+    }
+}
 extern void     qX11ClearFontNameCache(); // defined in qfont_x11.cpp
 
 /*!
-    Sets the value returned by x11AppDpiX() to \a dpi. The default is
-    determined by the display configuration. Changing this value will
-    alter the scaling of fonts and many other metrics and is not
-    recommended. Using this function is not portable.
+    Sets the value returned by x11AppDpiX() to \a dpi for screen
+    \a screen. The default is determined by the display configuration.
+    Changing this value will alter the scaling of fonts and many other
+    metrics and is not recommended. Using this function is not
+    portable.
 
     \sa x11SetAppDpiY()
 */
-void QPaintDevice::x11SetAppDpiX(int dpi)
+void QPaintDevice::x11SetAppDpiX(int dpi, int screen)
 {
-    dpiX = dpi;
+    create_dpis();
+    if ( ! dpisX )
+	return;
+    if ( screen < 0 )
+	screen = QPaintDevice::x11AppScreen();
+    if ( screen > ScreenCount( QPaintDevice::x11AppDisplay() ) )
+	return;
+    dpisX[ screen ] = dpi;
     qX11ClearFontNameCache();
 }
 
 /*!
-    Sets the value returned by x11AppDpiY() to \a dpi. The default is
-    determined by the display configuration. Changing this value will
-    alter the scaling of fonts and many other metrics and is not
-    recommended. Using this function is not portable.
+    \overload
+
+    Sets the value returned by x11AppDpiX() to \a dpi for the default
+    screen. The default is determined by the display configuration.
+    Changing this value will alter the scaling of fonts and many other
+    metrics and is not recommended. Using this function is not
+    portable.
+
+*/
+// ### REMOVE 4.0
+void QPaintDevice::x11SetAppDpiX( int dpi )
+{
+    QPaintDevice::x11SetAppDpiX( dpi, -1 );
+}
+
+/*!
+    Sets the value returned by x11AppDpiY() to \a dpi for screen
+    \a screen. The default is determined by the display configuration.
+    Changing this value will alter the scaling of fonts and many other
+    metrics and is not recommended. Using this function is not
+    portable.
 
     \sa x11SetAppDpiX()
 */
-void QPaintDevice::x11SetAppDpiY(int dpi)
+void QPaintDevice::x11SetAppDpiY(int dpi, int screen)
 {
-    dpiY = dpi;
+    create_dpis();
+    if ( ! dpisY )
+	return;
+    if ( screen < 0 )
+	screen = QPaintDevice::x11AppScreen();
+    if ( screen > ScreenCount( QPaintDevice::x11AppDisplay() ) )
+	return;
+    dpisY[ screen ] = dpi;
     qX11ClearFontNameCache();
 }
 
 /*!
-    Returns the horizontal DPI of the X display (X11 only). Using this
-    function is not portable. See QPaintDeviceMetrics for portable
-    access to related information. Using this function is not
-    portable.
+    \overload
 
-    \sa x11AppDpiY(), x11SetAppDpiX(), QPaintDeviceMetrics::logicalDpiX()
+    Sets the value returned by x11AppDpiY() to \a dpi for the default
+    screen.  The default is determined by the display configuration.
+    Changing this value will alter the scaling of fonts and many other
+    metrics and is not recommended. Using this function is not
+    portable.
 */
-int QPaintDevice::x11AppDpiX()
+// ### REMOVE 4.0
+void QPaintDevice::x11SetAppDpiY( int dpi )
 {
-    if ( !dpiX ) {
-	Display *dpy = x11AppDisplay();
-	int scr = x11AppScreen();
-	if ( dpy ) {
-	    dpiX =
-		(DisplayWidth(dpy,scr) * 254 + DisplayWidthMM(dpy,scr)*5)
-		       / (DisplayWidthMM(dpy,scr)*10);
-	}
-    }
-    return dpiX;
+    QPaintDevice::x11SetAppDpiY( dpi, -1 );
 }
 
 /*!
-    Returns the vertical DPI of the X11 display (X11 only). Using this
-    function is not portable. See QPaintDeviceMetrics for portable
-    access to related information. Using this function is not
-    portable.
+    Returns the horizontal DPI of the X display (X11 only) for screen
+    \a screen. Using this function is not portable. See
+    QPaintDeviceMetrics for portable access to related information.
+    Using this function is not portable.
+
+    \sa x11AppDpiY(), x11SetAppDpiX(), QPaintDeviceMetrics::logicalDpiX()
+*/
+int QPaintDevice::x11AppDpiX(int screen)
+{
+    create_dpis();
+    if ( ! dpisX )
+	return 0;
+    if ( screen < 0 )
+	screen = QPaintDevice::x11AppScreen();
+    if ( screen > ScreenCount( QPaintDevice::x11AppDisplay() ) )
+	return 0;
+    return dpisX[ screen ];
+}
+
+/*!
+    \overload
+
+    Returns the horizontal DPI of the X display (X11 only) for the
+    default screen. Using this function is not portable. See
+    QPaintDeviceMetrics for portable access to related information.
+    Using this function is not portable.
+*/
+int QPaintDevice::x11AppDpiX()
+{
+    return QPaintDevice::x11AppDpiX( -1 );
+}
+
+/*!
+    Returns the vertical DPI of the X11 display (X11 only) for screen
+    \a screen.  Using this function is not portable. See
+    QPaintDeviceMetrics for portable access to related information.
+    Using this function is not portable.
+
+    \sa x11AppDpiX(), x11SetAppDpiY(), QPaintDeviceMetrics::logicalDpiY()
+*/
+int QPaintDevice::x11AppDpiY( int screen )
+{
+    create_dpis();
+    if ( ! dpisY )
+	return 0;
+    if ( screen < 0 )
+	screen = QPaintDevice::x11AppScreen();
+    if ( screen > ScreenCount( QPaintDevice::x11AppDisplay() ) )
+	return 0;
+    return dpisY[ screen ];
+}
+
+/*!
+    \overload
+
+    Returns the vertical DPI of the X11 display (X11 only) for the
+    default screen.  Using this function is not portable. See
+    QPaintDeviceMetrics for portable access to related information.
+    Using this function is not portable.
 
     \sa x11AppDpiX(), x11SetAppDpiY(), QPaintDeviceMetrics::logicalDpiY()
 */
 int QPaintDevice::x11AppDpiY()
 {
-    if ( !dpiY ) {
-	Display *dpy = x11AppDisplay();
-	int scr = x11AppScreen();
-	if ( dpy )
-	    dpiY =
-		(DisplayHeight(dpy,scr) * 254 + DisplayHeightMM(dpy,scr)*5)
-		       / (DisplayHeightMM(dpy,scr)*10);
-    }
-    return dpiY;
+    return QPaintDevice::x11AppDpiY( -1 );
 }
 
-
 /*!
     \fn bool QPaintDevice::paintingActive() const
 
@@ -576,7 +773,7 @@ static mask_gc gc_vec[max_mask_gcs];
 
 static void cleanup_mask_gc()
 {
-    Display *dpy = qt_xdisplay();
+    Display *dpy = QPaintDevice::x11AppDisplay();
     init_mask_gc = FALSE;
     for ( int i=0; i<max_mask_gcs; i++ ) {
 	if ( gc_vec[i].gc )
@@ -876,8 +1073,8 @@ void bitBlt( QPaintDevice *dst, int dx, int dy,
 			    GCStipple | GCTileStipXOrigin | GCTileStipYOrigin;
 	if ( td == QInternal::Widget ) {	// set GC colors
 	    QWidget *w = (QWidget *)dst;
-	    gcvals.background = w->backgroundColor().pixel();
-	    gcvals.foreground = w->foregroundColor().pixel();
+	    gcvals.background = w->backgroundColor().pixel( dst->x11Screen() );
+	    gcvals.foreground = w->foregroundColor().pixel( dst->x11Screen() );
 	    if ( include_inferiors ) {
 		valmask |= GCSubwindowMode;
 		gcvals.subwindow_mode = IncludeInferiors;
@@ -886,8 +1083,8 @@ void bitBlt( QPaintDevice *dst, int dx, int dy,
 	    gcvals.background = 0;
 	    gcvals.foreground = 1;
 	} else {
-	    gcvals.background = Qt::white.pixel();
-	    gcvals.foreground = Qt::black.pixel();
+	    gcvals.background = Qt::white.pixel( dst->x11Screen() );
+	    gcvals.foreground = Qt::black.pixel( dst->x11Screen() );
 	}
 
 	gcvals.fill_style  = FillOpaqueStippled;
