@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qpixmap.cpp   3.0.5   edited Feb 14 1999 $
+** $Id: qt/qpixmap.cpp   3.1.2   edited Feb 11 10:29 $
 **
 ** Implementation of QPixmap class
 **
 ** Created : 950301
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -46,6 +46,9 @@
 #include "qobjectlist.h"
 #include "qapplication.h"
 #include <private/qinternal_p.h>
+#include "qmime.h"
+#include "qdragobject.h"
+#include "qfile.h"
 
 /*!
     \class QPixmap qpixmap.h
@@ -376,6 +379,32 @@ QPixmap::~QPixmap()
     deref();
 }
 
+/*! Convenience function. Gets the data associated with the absolute
+  name \a abs_name from the default mime source factory and decodes it
+  to a pixmap.
+
+  \sa QMimeSourceFactory, QImage::fromMimeSource(), QImageDrag::decode()
+*/
+
+#ifndef QT_NO_MIME
+QPixmap QPixmap::fromMimeSource( const QString &abs_name )
+{
+    const QMimeSource *m = QMimeSourceFactory::defaultFactory()->data( abs_name );
+    if ( !m ) {
+	if ( QFile::exists( abs_name ) )
+	    return QPixmap( abs_name );
+#if defined(QT_CHECK_STATE)
+	if ( !abs_name.isEmpty() )
+	    qWarning( "QPixmap::fromMimeSource: Cannot find pixmap \"%s\" in the mime source factory",
+		      abs_name.latin1() );
+#endif
+	return QPixmap();
+    }
+    QPixmap pix;
+    QImageDrag::decode( m, pix );
+    return pix;
+}
+#endif
 
 /*!
     Returns a \link shclass.html deep copy\endlink of the pixmap using
@@ -396,21 +425,25 @@ QPixmap QPixmap::copy( bool ignoreMask ) const
     if ( !pm.isNull() ) {			// copy the bitmap
 #if defined(Q_WS_X11)
 	pm.cloneX11Data( this );
-#if !defined(QT_NO_XRENDER)
+#endif
+#if defined(Q_WS_MAC) || (defined(Q_WS_X11) && !defined(QT_NO_XFTFREETYPE))
 	QPixmap* save_alpha = data->alphapm;
 	data->alphapm = 0;
-#endif // !QT_NO_XRENDER
-#endif // Q_WS_X11
+#endif // !QT_NO_XFTFREETYPE
 	bitBlt( &pm, 0,0, this, 0,0, data->w, data->h, CopyROP, TRUE );
-#if defined(Q_WS_X11) && !defined(QT_NO_XRENDER)
+#if defined(Q_WS_MAC) || (defined(Q_WS_X11) && !defined(QT_NO_XFTFREETYPE))
 	data->alphapm = save_alpha;
-#endif // Q_WS_X11 && !QT_NO_XRENDER
+#endif // Q_WS_X11 && !QT_NO_XFTFREETYPE
 	if ( !ignoreMask ) {
-#if defined(Q_WS_X11) && !defined(QT_NO_XRENDER)
+#if defined(Q_WS_MAC)
+	    if(data->alphapm)
+		qt_mac_copy_alpha_pixmap(&pm, this);
+	    else
+#elif defined(Q_WS_X11) && !defined(QT_NO_XFTFREETYPE)
 	    if ( data->alphapm )
 		qt_x11_copy_alpha_pixmap(&pm, this);
 	    else
-#endif // Q_WS_X11 && !QT_NO_XRENDER
+#endif // Q_WS_X11 && !QT_NO_XFTFREETYPE
 		if ( data->mask )		// copy the mask
 		    pm.setMask( data->selfmask ? *((QBitmap*)&pm) : *data->mask );
 	}
@@ -441,7 +474,11 @@ QPixmap &QPixmap::operator=( const QPixmap &pixmap )
 	if ( !isNull() ) {
 	    bitBlt( this, 0, 0, &pixmap, 0, 0, pixmap.width(), pixmap.height(),
 		    CopyROP, TRUE );
-#if defined(Q_WS_X11) && !defined(QT_NO_XRENDER)
+#if defined(Q_WS_MAC)
+	    if ( pixmap.data->alphapm )
+		qt_mac_copy_alpha_pixmap(this, &pixmap);
+	    else
+#elif defined(Q_WS_X11) && !defined(QT_NO_XFTFREETYPE)
 	    if ( pixmap.data->alphapm )
 		qt_x11_copy_alpha_pixmap(this, &pixmap);
 	    else
@@ -593,36 +630,19 @@ QPixmap &QPixmap::operator=( const QImage &image )
 void QPixmap::fill( const QWidget *widget, int xofs, int yofs )
 {
     const QPixmap* bgpm = widget->backgroundPixmap();
+    fill( widget->backgroundColor() );
     if ( bgpm ) {
 	if ( !bgpm->isNull() ) {
-	    if ( !widget->isTopLevel() ) {
-		switch ( widget->backgroundOrigin() ) {
-		case QWidget::ParentOrigin:
-		    xofs += widget->x();
-		    yofs += widget->y();
-		    break;
-		case QWidget::WindowOrigin: {
-		    const QWidget *topl = widget;
-		    while(!topl->isTopLevel() && !topl->testWFlags(WSubWindow))
-			topl = topl->parentWidget(TRUE);
-		    QPoint p = widget->mapTo( ((QWidget *)topl), QPoint(0,0) );
-		    xofs += p.x();
-		    yofs += p.y();
-		    } break;
-		default:
-		    break;
-		}
-	    }
+	    QPoint ofs = widget->backgroundOffset();
+	    xofs += ofs.x();
+	    yofs += ofs.y();
+
 	    QPainter p;
 	    p.begin( this );
 	    p.setPen( NoPen );
-	    p.setBrush( QBrush( Qt::black,*widget->backgroundPixmap() ) );
-	    p.setBrushOrigin( -xofs, -yofs );
-	    p.drawRect( 0, 0, width(), height() );
+	    p.drawTiledPixmap( 0, 0, width(), height(), *widget->backgroundPixmap(), xofs, yofs );
 	    p.end();
 	}
-    } else {
-	fill( widget->backgroundColor() );
     }
 }
 
@@ -656,11 +676,18 @@ void QPixmap::resize( int w, int h )
 	d = isQBitmap() ? 1 : -1;
     // Create new pixmap
     QPixmap pm( w, h, d, data->bitmap, data->optim );
+#ifdef Q_WS_X11
+    pm.x11SetScreen( x11Screen() );
+#endif // Q_WS_X11
     if ( !data->uninit && !isNull() )		// has existing pixmap
 	bitBlt( &pm, 0, 0, this, 0, 0,		// copy old pixmap
 		QMIN(width(), w),
 		QMIN(height(),h), CopyROP, TRUE );
-#if defined(Q_WS_X11) && !defined(QT_NO_XRENDER)
+#if defined(Q_WS_MAC)
+    if(data->alphapm) {
+	data->alphapm->resize(w, h);
+    } else
+#elif defined(Q_WS_X11) && !defined(QT_NO_XFTFREETYPE)
     if (data->alphapm)
 	qWarning("QPixmap::resize: TODO: resize alpha data");
     else
@@ -732,11 +759,11 @@ void QPixmap::setMask( const QBitmap &newmask )
 #endif
 	return;
     }
-#if defined(Q_WS_X11) && !defined(QT_NO_XRENDER)
+#if defined(Q_WS_MAC) || (defined(Q_WS_X11) && !defined(QT_NO_XFTFREETYPE))
     // when setting the mask, we get rid of the alpha channel completely
     delete data->alphapm;
     data->alphapm = 0;
-#endif // Q_WS_X11 && !QT_NO_XRENDER
+#endif // Q_WS_X11 && !QT_NO_XFTFREETYPE
 
     delete data->mask;
     QBitmap* newmaskcopy;
@@ -880,6 +907,12 @@ bool QPixmap::load( const QString &fileName, const char *format,
 
 bool QPixmap::convertFromImage( const QImage &image, ColorMode mode )
 {
+    if ( image.isNull() ) {
+	// convert null image to null pixmap
+	*this = QPixmap();
+	return TRUE;
+    }
+
     int conversion_flags = 0;
     switch (mode) {
       case Color:
@@ -1051,6 +1084,8 @@ void QPixmap::setDefaultOptimization( Optimization optimization )
 static QPixmap grabChildWidgets( QWidget * w )
 {
     QPixmap res( w->width(), w->height() );
+    if ( res.isNull() && w->width() )
+	return res;
     res.fill( w, QPoint( 0, 0 ) );
     QPainter::redirect( w, &res ); // ### overwrites earlier redirect
     bool dblbfr = QSharedDoubleBuffer::isDisabled();
@@ -1080,7 +1115,13 @@ static QPixmap grabChildWidgets( QWidget * w )
 		// make sure to evaluate pos() first - who knows what
 		// the paint event(s) inside grabChildWidgets() will do.
 		QPoint childpos = ((QWidget *)child)->pos();
-		p.drawPixmap( childpos, grabChildWidgets( (QWidget *)child ) );
+		QPixmap cpm = grabChildWidgets( (QWidget *)child );
+		if ( cpm.isNull() ) {
+		    // Some child pixmap failed - abort and reset
+		    res.resize( 0, 0 );
+		    break;
+		}
+		p.drawPixmap( childpos, cpm);
 	    }
 	}
     }
@@ -1118,6 +1159,10 @@ static QPixmap grabChildWidgets( QWidget * w )
     is a superset of \a widget, the areas outside \a widget are
     covered with the widget's background.
 
+    If an error occurs when trying to grab the widget, such as the
+    size of the widget being too large to fit in memory, an isNull()
+    pixmap is returned.
+
     \sa grabWindow() QPainter::redirect() QWidget::paintEvent()
 */
 
@@ -1139,12 +1184,20 @@ QPixmap QPixmap::grabWidget( QWidget * widget, int x, int y, int w, int h )
 	return res;
 
     res.resize( w, h );
+    if( res.isNull() )
+	return res;
     res.fill( widget, QPoint( w,h ) );
     QPixmap tmp( grabChildWidgets( widget ) );
+    if( tmp.isNull() )
+	return tmp;
     ::bitBlt( &res, 0, 0, &tmp, x, y, w, h );
     return res;
 }
 
+/*!
+    \fn QPixmap::Q_DUMMY_COMPARISON_OPERATOR( QPixmap )
+    \internal
+*/
 
 /*!
     Returns the actual matrix used for transforming a pixmap with \a w
@@ -1193,8 +1246,10 @@ QWMatrix QPixmap::trueMatrix( const QWMatrix &matrix, int w, int h )
     if ( x3 > xmax ) xmax = x3;
     if ( x4 > xmax ) xmax = x4;
 
-    xmin -= xmin/(xmax-xmin);
-    ymin -= ymin/(ymax-ymin);
+    if ( xmax-xmin > 1.0 )
+	xmin -= xmin/(xmax-xmin);
+    if ( ymax-ymin > 1.0 )
+	ymin -= ymin/(ymax-ymin);
 
     mat.setMatrix( matrix.m11(), matrix.m12(), matrix.m21(), matrix.m22(), -xmin, -ymin );
     return mat;
@@ -1220,15 +1275,7 @@ QWMatrix QPixmap::trueMatrix( const QWMatrix &matrix, int w, int h )
 
 QDataStream &operator<<( QDataStream &s, const QPixmap &pixmap )
 {
-    QImageIO io;
-    io.setIODevice( s.device() );
-    if ( s.version() == 1 )
-	io.setFormat( "BMP" );
-    else
-	io.setFormat( "PNG" );
-
-    io.setImage( pixmap.convertToImage() );
-    io.write();
+    s << pixmap.convertToImage();
     return s;
 }
 
@@ -1243,9 +1290,9 @@ QDataStream &operator<<( QDataStream &s, const QPixmap &pixmap )
 
 QDataStream &operator>>( QDataStream &s, QPixmap &pixmap )
 {
-    QImageIO io( s.device(), 0 );
-    if ( io.read() )
-	pixmap.convertFromImage( io.image() );
+    QImage img;
+    s >> img;
+    pixmap.convertFromImage( img );
     return s;
 }
 
@@ -1317,8 +1364,8 @@ bool qt_xForm_helper( const QWMatrix &trueMat, int xoffset,
     int dx  = (int)(trueMat.dx() *65536.0);
     int dy  = (int)(trueMat.dy() *65536.0);
 
-    int m21ydx = dx + (xoffset<<16);
-    int m22ydy = dy;
+    int m21ydx = dx + (xoffset<<16) + QABS(m11)/2;
+    int m22ydy = dy + QABS(m22)/2;
     uint trigx;
     uint trigy;
     uint maxws = sWidth<<16;
@@ -1387,14 +1434,14 @@ bool qt_xForm_helper( const QWMatrix &trueMat, int xoffset,
 	    switch ( type ) {
 		case QT_XFORM_TYPE_MSBFIRST:
 		    while ( dptr < maxp ) {
-			IWX_MSB(1);
-			IWX_MSB(2);
-			IWX_MSB(4);
-			IWX_MSB(8);
-			IWX_MSB(16);
-			IWX_MSB(32);
-			IWX_MSB(64);
 			IWX_MSB(128);
+			IWX_MSB(64);
+			IWX_MSB(32);
+			IWX_MSB(16);
+			IWX_MSB(8);
+			IWX_MSB(4);
+			IWX_MSB(2);
+			IWX_MSB(1);
 			dptr++;
 		    }
 		    break;
