@@ -1,5 +1,5 @@
 /**********************************************************************
-** $Id$
+** $Id: qt/qprinter.cpp   3.1.2   edited Nov 7 16:55 $
 **
 ** Implementation of QPrinter class
 **
@@ -263,9 +263,9 @@
     in color or not.
 
     \value Color  print in color if available, otherwise in grayscale.
-    
+
     \value GrayScale  print in grayscale, even on color printers.
-    Might be a little faster than \c Color.  This is the default.
+    Might be a little faster than \c Color. This is the default.
 */
 
 /*!
@@ -516,6 +516,9 @@ void QPrinter::setPageSize( PageSize newPageSize )
         return;
     }
     page_size = newPageSize;
+#if defined(Q_WS_WIN)
+    reinit();
+#endif
 }
 
 
@@ -553,7 +556,7 @@ QPrinter::PageOrder QPrinter::pageOrder() const
 
 /*!
     Sets the printer's color mode to \a newColorMode, which can be
-    either \c Color (the default) or \c GrayScale.
+    either \c Color or \c GrayScale (the default).
 
     \sa colorMode()
 */
@@ -683,6 +686,13 @@ void QPrinter::setMinMax( int minPage, int maxPage )
 
     Returns the number of copies to be printed. The default value is 1.
 
+    After a call to setup(), this value will return the number of
+    times the application is required to print in order to match the 
+    number specified in the printer setup dialog. This has been done since
+    some printer drivers are not capable of buffering up the copies and
+    the application in those cases have to make an explicit call to the
+    print code for each copy.
+
     \sa setNumCopies()
 */
 
