@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qrect.cpp   3.0.5   edited Apr 26 11:46 $
+** $Id: qt/qrect.cpp   3.1.2   edited Nov 7 16:55 $
 **
 ** Implementation of QRect class
 **
@@ -40,7 +40,7 @@
 #include "qdatastream.h"
 
 /*!
-    \class QRect qrect.h
+    \class QRect
     \brief The QRect class defines a rectangle in the plane.
 
     \ingroup images
@@ -178,6 +178,9 @@ QRect::QRect( const QPoint &topLeft, const QSize &size )
 
     A valid rectangle has a left() \<= right() and top() \<= bottom().
 
+    Note that non-trivial operations like intersections are not defined
+    for invalid rectangles.
+
     \c{isValid() == !isEmpty()}
 
     \sa isNull(), isEmpty(), normalize()
@@ -220,7 +223,7 @@ QRect QRect::normalize() const
 
     Returns the left coordinate of the rectangle. Identical to x().
 
-    \sa x(), top(), right(), setLeft(), topLeft(), bottomLeft()
+    \sa setLeft(), right(), topLeft(), bottomLeft()
 */
 
 /*!
@@ -228,7 +231,7 @@ QRect QRect::normalize() const
 
     Returns the top coordinate of the rectangle. Identical to y().
 
-    \sa y(), left(), bottom(), setTop(), topLeft(), topRight()
+    \sa setTop(), bottom(), topLeft(), topRight()
 */
 
 /*!
@@ -236,7 +239,7 @@ QRect QRect::normalize() const
 
     Returns the right coordinate of the rectangle.
 
-    \sa left(), setRight(), topRight(), bottomRight()
+    \sa setRight(), left(), topRight(), bottomRight()
 */
 
 /*!
@@ -244,7 +247,7 @@ QRect QRect::normalize() const
 
     Returns the bottom coordinate of the rectangle.
 
-    \sa top(), setBottom(), bottomLeft(), bottomRight()
+    \sa setBottom(), top(), bottomLeft(), bottomRight()
 */
 
 /*!
@@ -359,12 +362,64 @@ QRect QRect::normalize() const
     \sa y(), setX()
 */
 
+/*!
+    Set the top-left corner of the rectangle to \a p. May change
+    the size, but will the never change the bottom-right corner of
+    the rectangle.
+
+    \sa topLeft(), moveTopLeft(), setBottomRight(), setTopRight(), setBottomLeft()
+*/
+void QRect::setTopLeft( const QPoint &p )
+{
+    setLeft( p.x() );
+    setTop( p.y() );
+}
+
+/*!
+    Set the bottom-right corner of the rectangle to \a p. May change
+    the size, but will the never change the top-left corner of
+    the rectangle.
+
+    \sa bottomRight(), moveBottomRight(), setTopLeft(), setTopRight(), setBottomLeft()
+*/
+void QRect::setBottomRight( const QPoint &p )
+{
+    setRight( p.x() );
+    setBottom( p.y() );
+}
+
+/*!
+    Set the top-right corner of the rectangle to \a p. May change
+    the size, but will the never change the bottom-left corner of
+    the rectangle.
+
+    \sa topRight(), moveTopRight(), setTopLeft(), setBottomRight(), setBottomLeft()
+*/
+void QRect::setTopRight( const QPoint &p )
+{
+    setRight( p.x() );
+    setTop( p.y() );
+}
+
+/*!
+    Set the bottom-left corner of the rectangle to \a p. May change
+    the size, but will the never change the top-right corner of
+    the rectangle.
+
+    \sa bottomLeft(), moveBottomLeft(), setTopLeft(), setBottomRight(), setTopRight()
+*/
+void QRect::setBottomLeft( const QPoint &p )
+{
+    setLeft( p.x() );
+    setBottom( p.y() );
+}
+
 /*!
     \fn QPoint QRect::topLeft() const
 
     Returns the top-left position of the rectangle.
 
-    \sa moveTopLeft(), topRight(), bottomLeft(), bottomRight(), left(), top()
+    \sa setTopLeft(), moveTopLeft(), bottomRight(), left(), top()
 */
 
 /*!
@@ -372,7 +427,7 @@ QRect QRect::normalize() const
 
     Returns the bottom-right position of the rectangle.
 
-    \sa moveBottomRight(), bottomLeft(), topLeft(), topRight(), bottom(), right()
+    \sa setBottomRight(), moveBottomRight(), topLeft(), right(), bottom()
 */
 
 /*!
@@ -380,7 +435,7 @@ QRect QRect::normalize() const
 
     Returns the top-right position of the rectangle.
 
-    \sa moveTopRight(), topLeft(), bottomLeft(), bottomRight(), top(), right()
+    \sa setTopRight(), moveTopRight(), bottomLeft(), top(), right()
 */
 
 /*!
@@ -388,7 +443,7 @@ QRect QRect::normalize() const
 
     Returns the bottom-left position of the rectangle.
 
-    \sa moveBottomLeft(), bottomRight(), topLeft(), topRight(), bottom(), left()
+    \sa setBottomLeft(), moveBottomLeft(), topRight(), bottom(), left()
 */
 
 /*!
@@ -396,7 +451,7 @@ QRect QRect::normalize() const
 
     Returns the center point of the rectangle.
 
-    \sa moveCenter(), topLeft(), topRight(), bottomLeft(), bottomRight()
+    \sa moveCenter(), topLeft(), bottomRight(), topRight(), bottomLeft()
 */
 
 
@@ -430,68 +485,108 @@ void QRect::coords( int *xp1, int *yp1, int *xp2, int *yp2 ) const
     *yp2 = y2;
 }
 
+
+/*!
+    Sets the left position of the rectangle to \a pos, leaving the
+    size unchanged.
+
+    \sa left(), setLeft(), moveTop(), moveRight(), moveBottom()
+*/
+void QRect::moveLeft( int pos )
+{
+    x2 += (QCOORD)(pos - x1);
+    x1 = (QCOORD)pos;
+}
+
+/*!
+    Sets the top position of the rectangle to \a pos, leaving the
+    size unchanged.
+
+    \sa top(), setTop(), moveLeft(), moveRight(), moveBottom()
+*/
+
+void QRect::moveTop( int pos )
+{
+    y2 += (QCOORD)(pos - y1);
+    y1 = (QCOORD)pos;
+}
+
+/*!
+    Sets the right position of the rectangle to \a pos, leaving the
+    size unchanged.
+
+    \sa right(), setRight(), moveLeft(), moveTop(), moveBottom()
+*/
+
+void QRect::moveRight( int pos )
+{
+    x1 += (QCOORD)(pos - x2);
+    x2 = (QCOORD)pos;
+}
+
+/*!
+    Sets the bottom position of the rectangle to \a pos, leaving the
+    size unchanged.
+
+    \sa bottom(), setBottom(), moveLeft(), moveTop(), moveRight()
+*/
+
+void QRect::moveBottom( int pos )
+{
+    y1 += (QCOORD)(pos - y2);
+    y2 = (QCOORD)pos;
+}
+
 /*!
     Sets the top-left position of the rectangle to \a p, leaving the
     size unchanged.
 
-    \sa topLeft(), moveTopRight(), moveBottomLeft(), moveBottomRight(),
-    setTop(), setLeft()
+    \sa topLeft(), setTopLeft(), moveBottomRight(), moveTopRight(), moveBottomLeft()
 */
 
 void QRect::moveTopLeft( const QPoint &p )
 {
-    x2 += (QCOORD)(p.x() - x1);
-    y2 += (QCOORD)(p.y() - y1);
-    x1 = (QCOORD)p.x();
-    y1 = (QCOORD)p.y();
+    moveLeft( p.x() );
+    moveTop( p.y() );
 }
 
 /*!
     Sets the bottom-right position of the rectangle to \a p, leaving
     the size unchanged.
 
-    \sa bottomRight(), moveBottomLeft(), moveTopLeft(), moveTopRight(),
-    setBottom(), setRight()
+    \sa bottomRight(), setBottomRight(), moveTopLeft(), moveTopRight(), moveBottomLeft()
 */
 
 void QRect::moveBottomRight( const QPoint &p )
 {
-    x1 += (QCOORD)(p.x() - x2);
-    y1 += (QCOORD)(p.y() - y2);
-    x2 = (QCOORD)p.x();
-    y2 = (QCOORD)p.y();
+    moveRight( p.x() );
+    moveBottom( p.y() );
 }
 
 /*!
     Sets the top-right position of the rectangle to \a p, leaving the
     size unchanged.
 
-    \sa topRight(), moveTopLeft(), moveBottomLeft(), moveBottomRight(),
-    setTop(), setRight()
+    \sa topRight(), setTopRight(), moveTopLeft(), moveBottomRight(), moveBottomLeft()
 */
 
 void QRect::moveTopRight( const QPoint &p )
 {
-    x1 += (QCOORD)(p.x() - x2);
-    y2 += (QCOORD)(p.y() - y1);
-    x2 = (QCOORD)p.x();
-    y1 = (QCOORD)p.y();
+    moveRight( p.x() );
+    moveTop( p.y() );
 }
 
 /*!
     Sets the bottom-left position of the rectangle to \a p, leaving
     the size unchanged.
 
-    \sa bottomLeft(), moveBottomRight(), moveTopLeft(), moveTopRight(),
-    setBottom(), setLeft()
+    \sa bottomLeft(), setBottomLeft(), moveTopLeft(), moveBottomRight(), moveTopRight()
 */
 
 void QRect::moveBottomLeft( const QPoint &p )
 {
-    x2 += (QCOORD)(p.x() - x1);
-    y1 += (QCOORD)(p.y() - y2);
-    x1 = (QCOORD)p.x();
-    y2 = (QCOORD)p.y();
+    moveLeft( p.x() );
+    moveBottom( p.y() );
 }
 
 
@@ -499,7 +594,7 @@ void QRect::moveBottomLeft( const QPoint &p )
     Sets the center point of the rectangle to \a p, leaving the size
     unchanged.
 
-    \sa center(), moveTopLeft(), moveTopRight(), moveBottomLeft(), moveBottomRight()
+    \sa center(), moveTopLeft(), moveBottomRight(), moveTopRight(), moveBottomLeft()
 */
 
 void QRect::moveCenter( const QPoint &p )
@@ -514,11 +609,12 @@ void QRect::moveCenter( const QPoint &p )
 
 
 /*!
-    Moves the rectangle \a dx along the X axis and \a dy along the Y
-    axis, relative to the current position. (Positive values move the
-    rectangle right and/or down.)
-*/
+    Moves the rectangle \a dx along the x axis and \a dy along the y
+    axis, relative to the current position. Positive values move the
+    rectangle to the right and down.
 
+    \sa moveTopLeft()
+*/
 
 void QRect::moveBy( int dx, int dy )
 {
@@ -663,6 +759,13 @@ bool QRect::contains( const QPoint &p, bool proper ) const
     is entirely inside (not on the edge).
 */
 
+/*!
+    \overload bool QRect::contains( int x, int y ) const
+
+    Returns TRUE if the point \a x, \a y is inside this rectangle;
+    otherwise returns FALSE.
+*/
+
 /*!
     \overload
 
