@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qstylesheet.cpp   3.0.5   edited Jul 3 14:12 $
+** $Id: qt/qstylesheet.cpp   3.1.2   edited Feb 11 10:13 $
 **
 ** Implementation of the QStyleSheet class
 **
@@ -174,7 +174,19 @@ QStyleSheetItem::~QStyleSheetItem()
     delete d;
 }
 
-
+/*!
+  Assignment. Assings a copy of \a other that is not bound to any style sheet.
+  Unbounds first from previous style sheet.
+ */
+QStyleSheetItem& QStyleSheetItem::operator=( const QStyleSheetItem& other )
+{
+    if ( &other == this )
+	return *this;
+    delete d;
+    d = new QStyleSheetItemData;
+    *d = *other.d;
+    return *this;
+}
 
 /*!
     Returns the style sheet this item is in.
@@ -682,7 +694,6 @@ void QStyleSheetItem::setMargin(Margin m, int v)
 	d->margin[1] = v;
 	d->margin[2] = v;
 	d->margin[3] = v;
-	d->margin[4] = v;
     } else if (m == MarginVertical ) {
 	d->margin[MarginTop] = v;
 	d->margin[MarginBottom] = v;
@@ -797,7 +808,8 @@ void QStyleSheetItem::setSelfNesting( bool nesting )
     d->selfnest = nesting;
 }
 
-/*
+/*!
+   \internal
     Sets the linespacing to be at least \a ls pixels.
 
     For compatibility with previous Qt releases, small values get
@@ -844,7 +856,7 @@ int QStyleSheetItem::lineSpacing() const
     definition of a set of tags. This definition will be used by the
     internal rich text rendering system to parse and display text
     documents to which the style sheet applies. Rich text is normally
-    visualized in a QTextView or a QTextBrowser. However, QLabel,
+    visualized in a QTextEdit or a QTextBrowser. However, QLabel,
     QWhatsThis and QMessageBox also support it, and other classes are
     likely to follow. With QSimpleRichText it is possible to use the
     rich text renderer for custom widgets as well.
@@ -863,7 +875,7 @@ int QStyleSheetItem::lineSpacing() const
 	    attributes:
 	    \list
 	    \i \c title -- The caption of the document. This attribute is
-		easily accessible with QTextView::documentTitle().
+		easily accessible with QTextEdit::documentTitle().
 	    \i \c type -- The type of the document. The default type is \c
 		page. It indicates that the document is displayed in a
 		page of its own. Another style is \c detail, which can be
@@ -952,7 +964,7 @@ int QStyleSheetItem::lineSpacing() const
 	 \i A smaller font size.
     \row \i \c{<code>...</code>}
 	 \i Indicates code. By default this is the same as
-	    \c{<tt>...</tt>} (typewriter). For larger junks of code
+	    \c{<tt>...</tt>} (typewriter). For larger chunks of code
 	    use the block-tag \c{<}\c{pre>}.
     \row \i \c{<tt>...</tt>}
 	 \i Typewriter font style.
@@ -1037,7 +1049,9 @@ int QStyleSheetItem::lineSpacing() const
 	    \i \c rowspan -- Specifies how many rows this cell spans. The
 		default is 1.
 	    \i \c align -- Alignment; possible values are \c left, \c
-		right, and \c center. The default is left.
+		right, and \c center. The default is \c left.
+	    \i \c valign -- Vertical alignment; possible values are \c
+	    top, \c middle, and \c bottom. The default is \c middle.
 	    \endlist
     \endtable
 */
@@ -1191,6 +1205,7 @@ void QStyleSheet::init()
     style->setFontFamily( QString::fromLatin1("courier") );
     style->setDisplayMode(QStyleSheetItem::DisplayBlock);
     style->setWhiteSpaceMode(QStyleSheetItem::WhiteSpacePre);
+    style-> setMargin(QStyleSheetItem::MarginVertical, 12);
 
     style = new QStyleSheetItem( this, QString::fromLatin1("blockquote") );
     style->setDisplayMode(QStyleSheetItem::DisplayBlock);
@@ -1336,6 +1351,7 @@ const QStyleSheetItem* QStyleSheet::item( const QString& name) const
 
     This function should not be used in application code.
 */
+#ifndef QT_NO_TEXTCUSTOMITEM
 QTextCustomItem* QStyleSheet::tag(  const QString& name,
 				   const QMap<QString, QString> &attr,
 				   const QString& context,
@@ -1352,6 +1368,7 @@ QTextCustomItem* QStyleSheet::tag(  const QString& name,
 	return new QTextHorizontalLine( doc, attr, context, (QMimeSourceFactory&)factory  );
    return 0;
 }
+#endif
 
 
 /*!  Auxiliary function. Converts the plain text string \a plain to a
@@ -1454,6 +1471,9 @@ QString QStyleSheet::escape( const QString& plain)
     \value AutoText The text string is interpreted as for \c RichText
 	if QStyleSheet::mightBeRichText() returns TRUE, otherwise as
 	\c PlainText.
+	
+    \value LogText A special, limited text format which is only used
+    by QTextEdit in an optimized mode.
 */
 
 /*!
@@ -1469,16 +1489,20 @@ bool QStyleSheet::mightBeRichText( const QString& text)
 {
     if ( text.isEmpty() )
 	return FALSE;
-    if ( text.left(5).lower() == "<!doc" )
+    int start = 0;
+
+    while ( start < int(text.length()) && text[start].isSpace() )
+	++start;
+    if ( text.mid( start, 5 ).lower() == "<!doc" )
 	return TRUE;
-    int open = 0;
+    int open = start;
     while ( open < int(text.length()) && text[open] != '<'
-	    && text[open] != '\n' && text[open] != '&')
-	++open;
-    if ( text[open] == '&' ) {
-	if ( text.mid(open+1,3) == "lt;" )
+	    && text[open] != '\n' ) {
+	if ( text[open] == '&' &&  text.mid(open+1,3) == "lt;" )
 	    return TRUE; // support desperate attempt of user to see <...>
-    } else if ( text[open] == '<' ) {
+	++open;
+    }
+    if ( text[open] == '<' ) {
 	int close = text.find('>', open);
 	if ( close > -1 ) {
 	    QString tag;
@@ -1541,7 +1565,7 @@ void QStyleSheet::scaleFont( QFont& font, int logicalSize ) const
     int s;
     switch ( logicalSize ) {
     case 1:
-	s =  baseSize/2;
+	s =  7*baseSize/10;
 	break;
     case 2:
 	s = (8 * baseSize) / 10;
