@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qthread_unix.cpp   3.1.2   edited Feb 13 10:38 $
 **
 ** QThread class for Unix
 **
@@ -45,218 +45,96 @@
 #endif
 
 #include "qthread.h"
-#include "qapplication.h"
+
 #include "qmutex.h"
 #include "qwaitcondition.h"
-#include "qptrlist.h"
+#include <private/qmutexpool_p.h>
 
 #ifndef QT_H
-#if defined( QWS ) || defined( Q_WS_MACX )
-#include "qptrdict.h"
-#else
-#include "qintdict.h"
-#endif
+#  include "qapplication.h"
+#  include "qptrlist.h"
 #endif // QT_H
 
 #include <errno.h>
 
-static QMutex *dictMutex = 0;
-#if defined( QWS ) || defined( Q_WS_MACX )
-static QPtrDict<QThread> *thrDict = 0;
-#else
-static QIntDict<QThread> *thrDict = 0;
-#endif
+static QMutexPool *qt_thread_mutexpool = 0;
 
 
-extern "C" { static void *start_thread(void *t); }
-
-
-#ifdef    Q_OS_SOLARIS
-// Solaris
-typedef thread_t Q_THREAD_T;
-
-// helpers
-#define Q_THREAD_SELF()    thr_self()
-#define Q_THREAD_EXIT(a)   thr_exit((a))
-#define Q_THREAD_CREATE(a) (a) = thr_create(0, 0, start_thread, that, THR_DETACHED, \
-                                            &thread_id);
-#else // !Q_OS_SOLARIS
-// Pthreads
-typedef pthread_t Q_THREAD_T;
-
-// helpers
-#define Q_THREAD_SELF()    pthread_self()
-#define Q_THREAD_EXIT(a)   pthread_exit((a))
-#define Q_THREAD_CREATE(a) pthread_attr_t attr; \
-                           pthread_attr_init(&attr); \
-	                   pthread_attr_setinheritsched(&attr, PTHREAD_INHERIT_SCHED); \
-                           pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED); \
-	                   (a) = pthread_create(&thread_id, &attr, start_thread, \
-                                                (void *) that); \
-	                   pthread_attr_destroy(&attr);
-#endif // Q_OS_SOLARIS
+extern "C" { static void *start_thread( void *_arg ); }
+extern "C" { static void finish_thread( void *arg ); }
 
 
 class QThreadPrivate {
 public:
-    Q_THREAD_T thread_id;
-    QWaitCondition thread_done;      // Used for QThread::wait()
-    bool finished, running;
+    QWaitCondition thread_done;
+    pthread_t thread_id;
+    size_t stacksize;
+    void *args[2];
+    bool finished : 1;
+    bool running  : 1;
+    bool orphan   : 1;
 
-    QThreadPrivate()
-	: thread_id(0), finished(FALSE), running(FALSE)
-    {
-	if (! dictMutex)
-	    dictMutex = new QMutex;
-	if (! thrDict) {
-#if defined( QWS ) || defined( Q_WS_MACX )
-	    thrDict = new QPtrDict<QThread>;
-#else
-	    thrDict = new QIntDict<QThread>;
-#endif
-	}
-    }
+    QThreadPrivate( size_t ss = 0);
 
-    ~QThreadPrivate()
-    {
-	dictMutex->lock();
-	if (thread_id)
-	    thrDict->remove((Qt::HANDLE) thread_id);
-	dictMutex->unlock();
-
-	thread_id = 0;
-    }
-
-    void init(QThread *that)
-    {
-	that->d->running = TRUE;
-	that->d->finished = FALSE;
-
-	int ret;
-	Q_THREAD_CREATE(ret);
-
-#ifdef QT_CHECK_RANGE
-	if (ret)
-	    qWarning("QThread::start: thread creation error: %s", strerror(ret));
-#endif
-    }
-
-    static void internalRun(QThread *that)
-    {
-	dictMutex->lock();
-	thrDict->insert(QThread::currentThread(), that);
-	dictMutex->unlock();
-
-	that->run();
-
-	dictMutex->lock();
-
-	QThread *there = thrDict->find(QThread::currentThread());
-	if (there) {
-	    there->d->running = FALSE;
-	    there->d->finished = TRUE;
-
-	    there->d->thread_done.wakeAll();
-	}
-
-	thrDict->remove(QThread::currentThread());
-	dictMutex->unlock();
-    }
+    static inline void start( QThread *thread ) { thread->run(); }
 };
 
+inline QThreadPrivate::QThreadPrivate( size_t ss )
+    : thread_id( 0 ), stacksize( ss ),
+      finished( FALSE ), running( FALSE ), orphan( FALSE )
+{
+    // threads have not been initialized yet, do it now
+    if ( ! qt_thread_mutexpool ) QThread::initialize();
+}
+
 extern "C" {
-    static void *start_thread(void *t)
+    static void *start_thread( void *_arg )
     {
-	QThreadPrivate::internalRun( (QThread *) t );
+	void **arg = (void **) _arg;
+
+	pthread_cleanup_push( finish_thread, arg[1] );
+	pthread_testcancel();
+
+	QThreadPrivate::start( (QThread *) arg[0] );
+
+	pthread_cleanup_pop( TRUE );
+
 	return 0;
     }
-}
 
-/**************************************************************************
- ** QThreadQtEvent
- *************************************************************************/
-
-// this is for the magic QThread::postEvent()
-class QThreadQtEvent
-{
-public:
-    QThreadQtEvent(QObject *r, QEvent *e)
-	: receiver(r), event(e)
+    static void finish_thread( void *arg )
     {
-    }
-    QObject *receiver;
-    QEvent *event;
-};
+	QThreadPrivate *d = (QThreadPrivate *) arg;
 
-
-class QThreadPostEventPrivate : public QObject
-{
-    Q_OBJECT
-public:
-    QThreadPostEventPrivate();
-
-    QPtrList<QThreadQtEvent> events;
-    QMutex eventmutex;
-
-public slots:
-    void sendEvents();
-};
-
-
-QThreadPostEventPrivate::QThreadPostEventPrivate()
-{
-    events.setAutoDelete( TRUE );
-    connect( qApp, SIGNAL( guiThreadAwake() ), this, SLOT( sendEvents() ) );
-}
-
-
-// this is called from the QApplication::guiThreadAwake signal, and the
-// application mutex is already locked
-void QThreadPostEventPrivate::sendEvents()
-{
-    eventmutex.lock();
-
-    QThreadQtEvent *qte;
-    for ( qte = events.first(); qte != 0; qte = events.next() ) {
-	if ( qte->receiver )
-	    qApp->postEvent( qte->receiver, qte->event );
-    }
-    events.clear();
-
-    eventmutex.unlock();
-}
-
-
-static QThreadPostEventPrivate * qthreadposteventprivate = 0;
-
-
-void qthread_removePostedEvents( QObject *receiver )
-{
-    if ( !qthreadposteventprivate )
-	return;
-    qthreadposteventprivate->eventmutex.lock();
-
-    QThreadQtEvent *qte;
-    for ( qte = qthreadposteventprivate->events.first(); qte != 0;
-	  qte = qthreadposteventprivate->events.next() ) {
-	if ( qte->receiver == receiver ) {
-	    qte->receiver = 0;
-	    delete qte->event;
-	    qte->event = 0;
+	if ( ! d ) {
+#ifdef QT_CHECK_STATE
+	    qWarning( "QThread: internal error: zero data for running thread." );
+#endif // QT_CHECK_STATE
+	    return;
 	}
+
+	QMutexLocker locker( qt_thread_mutexpool->get( d ) );
+	d->running = FALSE;
+	d->finished = TRUE;
+	d->thread_id = 0;
+
+	d->thread_done.wakeAll();
+
+	d->args[0] = d->args[1] = 0;
+
+	if ( d->orphan )
+	    delete d;
     }
-
-    qthreadposteventprivate->eventmutex.unlock();
 }
 
 
-
 /**************************************************************************
  ** QThread
  *************************************************************************/
 
 /*!
     \class QThread qthread.h
+    \threadsafe
     \brief The QThread class provides platform-independent threads.
 
     \ingroup thread
@@ -318,7 +196,7 @@ void qthread_removePostedEvents( QObject *receiver )
 */
 Qt::HANDLE QThread::currentThread()
 {
-    return (HANDLE) Q_THREAD_SELF();
+    return (HANDLE) pthread_self();
 }
 
 
@@ -327,8 +205,10 @@ Qt::HANDLE QThread::currentThread()
 */
 void QThread::initialize()
 {
-    if( !qthreadposteventprivate )
-	qthreadposteventprivate = new QThreadPostEventPrivate();
+    if ( ! qt_global_mutexpool )
+	qt_global_mutexpool = new QMutexPool( TRUE );
+    if ( ! qt_thread_mutexpool )
+	qt_thread_mutexpool = new QMutexPool( FALSE );
 }
 
 
@@ -337,85 +217,41 @@ void QThread::initialize()
 */
 void QThread::cleanup()
 {
-    delete qthreadposteventprivate;
-    qthreadposteventprivate = 0;
+    delete qt_global_mutexpool;
+    delete qt_thread_mutexpool;
+    qt_global_mutexpool = 0;
+    qt_thread_mutexpool = 0;
 }
 
 
-/*!
-    Provides a way of posting an event from a thread which is not the
-    event thread to an object.
+/*! \obsolete
 
-    This is achieved as follows:
-    \list
-    \i The \a event is put into a queue;
-    \i the event thread is woken up;
-    \i the event thread sends the event to the \a receiver object.
-    \endlist
-    It is important to note that the event handler for the event, when
-    called, will be called from the event thread and not from the
-    thread calling QThread::postEvent().
-
-    Since QThread::postEvent() posts events into the event queue of
-    QApplication, you must create a QApplication object before calling
-    QThread::postEvent().
-
-    The event must be allocated on the heap since the post event queue
-    will take ownership of the event and delete it once it has been
-    posted.
-
-    \sa QApplication::postEvent()
+    Use QApplication::postEvent() instead.
 */
 void QThread::postEvent( QObject * receiver, QEvent * event )
 {
-#if defined(QT_CHECK_STATE)
-    if (! qthreadposteventprivate) {
-	qWarning("QThread::postEvent: cannot post event - threads not initialized.");
-	return;
-    }
-#endif // QT_CHECK_STATE
-
-    qthreadposteventprivate->eventmutex.lock();
-    qthreadposteventprivate->events.append( new QThreadQtEvent(receiver, event) );
-    qApp->wakeUpGuiThread();
-    qthreadposteventprivate->eventmutex.unlock();
+    QApplication::postEvent( receiver, event );
 }
 
 
 // helper function to do thread sleeps, since usleep()/nanosleep() aren't reliable
 // enough (in terms of behavior and availability)
-static void thread_sleep(struct timespec *ti)
+static void thread_sleep( struct timespec *ti )
 {
-#ifdef    Q_OS_SOLARIS
-    mutex_t mtx;
-    cond_t cnd;
-
-    mutex_init(&mtx, 0, 0);
-    cond_init(&cnd, 0, 0);
-
-    mutex_lock(&mtx);
-    (void) cond_timedwait(&cnd, &mtx, ti);
-    mutex_unlock(&mtx);
-
-    cond_destroy(&cnd);
-    mutex_destroy(&mtx);
-#else // !Q_OS_SOLARIS
     pthread_mutex_t mtx;
     pthread_cond_t cnd;
 
     pthread_mutex_init(&mtx, 0);
     pthread_cond_init(&cnd, 0);
 
-    pthread_mutex_lock(&mtx);
-    (void) pthread_cond_timedwait(&cnd, &mtx, ti);
-    pthread_mutex_unlock(&mtx);
+    pthread_mutex_lock( &mtx );
+    (void) pthread_cond_timedwait( &cnd, &mtx, ti );
+    pthread_mutex_unlock( &mtx );
 
-    pthread_cond_destroy(&cnd);
-    pthread_mutex_destroy(&mtx);
-#endif // Q_OS_SOLARIS
+    pthread_cond_destroy( &cnd );
+    pthread_mutex_destroy( &mtx );
 }
 
-
 /*!
     System independent sleep. This causes the current thread to sleep
     for \a secs seconds.
@@ -423,14 +259,13 @@ static void thread_sleep(struct timespec *ti)
 void QThread::sleep( unsigned long secs )
 {
     struct timeval tv;
-    gettimeofday(&tv, 0);
+    gettimeofday( &tv, 0 );
     struct timespec ti;
     ti.tv_sec = tv.tv_sec + secs;
-    ti.tv_nsec = (tv.tv_usec * 1000);
-    thread_sleep(&ti);
+    ti.tv_nsec = ( tv.tv_usec * 1000 );
+    thread_sleep( &ti );
 }
 
-
 /*!
     System independent sleep. This causes the current thread to sleep
     for \a msecs milliseconds
@@ -438,14 +273,14 @@ void QThread::sleep( unsigned long secs )
 void QThread::msleep( unsigned long msecs )
 {
     struct timeval tv;
-    gettimeofday(&tv, 0);
+    gettimeofday( &tv, 0 );
     struct timespec ti;
-    ti.tv_nsec = (tv.tv_usec * 1000) + (msecs % 1000) * 1000000;
-    ti.tv_sec = tv.tv_sec + (msecs / 1000) + (ti.tv_nsec / 1000000000);
-    ti.tv_nsec %= 1000000000;
-    thread_sleep(&ti);
-}
 
+    ti.tv_nsec = ( tv.tv_usec + ( msecs % 1000 ) * 1000 ) * 1000;
+    ti.tv_sec = tv.tv_sec + ( msecs / 1000 ) + ( ti.tv_nsec / 1000000000 );
+    ti.tv_nsec %= 1000000000;
+    thread_sleep( &ti );
+}
 
 /*!
     System independent sleep. This causes the current thread to sleep
@@ -454,14 +289,14 @@ void QThread::msleep( unsigned long msecs )
 void QThread::usleep( unsigned long usecs )
 {
     struct timeval tv;
-    gettimeofday(&tv, 0);
+    gettimeofday( &tv, 0 );
     struct timespec ti;
-    ti.tv_nsec = (tv.tv_usec * 1000) + (usecs % 1000000) * 1000;
-    ti.tv_sec = tv.tv_sec + (usecs / 1000000) + (ti.tv_nsec / 1000000000);
-    ti.tv_nsec %= 1000000000;
-    thread_sleep(&ti);
-}
 
+    ti.tv_nsec = ( tv.tv_usec + ( usecs % 1000000 ) ) * 1000;
+    ti.tv_sec = tv.tv_sec + ( usecs / 1000000 ) + ( ti.tv_nsec / 1000000000 );
+    ti.tv_nsec %= 1000000000;
+    thread_sleep( &ti );
+}
 
 /*!
     Constructs a new thread. The thread does not begin executing until
@@ -470,8 +305,26 @@ void QThread::usleep( unsigned long usecs )
 QThread::QThread()
 {
     d = new QThreadPrivate;
+    Q_CHECK_PTR( d );
 }
 
+/*!
+    Constructs a new thread. The thread does not begin executing until
+    start() is called.
+
+    If \a stackSize is greater than zero, the maximum stack size is
+    set to \a stackSize bytes, otherwise the maximum stack size is
+    automatically determined by the operating system.
+
+    \warning Most operating systems place minimum and maximum limits
+    on thread stack sizes. The thread will fail to start if the stack
+    size is outside these limits.
+*/
+QThread::QThread( unsigned int stackSize )
+{
+    d = new QThreadPrivate( stackSize );
+    Q_CHECK_PTR( d );
+}
 
 /*!
     QThread destructor.
@@ -483,14 +336,18 @@ QThread::QThread()
 */
 QThread::~QThread()
 {
-#ifndef QT_NO_DEBUG
-    if( d->running && !d->finished ) {
+    QMutexLocker locker( qt_thread_mutexpool ? qt_thread_mutexpool->get( d ) : 0 );
+    if ( d->running && !d->finished ) {
+#ifdef QT_CHECK_STATE
 	qWarning("QThread object destroyed while thread is still running.");
-    } else
 #endif
-	delete d;
-}
 
+	d->orphan = TRUE;
+	return;
+    }
+
+    delete d;
+}
 
 /*!
     Ends the execution of the calling thread and wakes up any threads
@@ -498,22 +355,9 @@ QThread::~QThread()
 */
 void QThread::exit()
 {
-    dictMutex->lock();
-
-    QThread *there = thrDict->find(QThread::currentThread());
-    if (there) {
-	there->d->running = FALSE;
-	there->d->finished = TRUE;
-
-	there->d->thread_done.wakeAll();
-    }
-
-    dictMutex->unlock();
-
-    Q_THREAD_EXIT(0);
+    pthread_exit( 0 );
 }
 
-
 /*!
     This begins the execution of the thread by calling run(), which
     should be reimplemented in a QThread subclass to contain your
@@ -523,17 +367,77 @@ void QThread::exit()
 */
 void QThread::start()
 {
-    if (d->running) {
-#ifdef QT_CHECK_RANGE
-	qWarning("Attempt to start a thread already running");
-#endif
+    QMutexLocker locker( qt_thread_mutexpool->get( d ) );
 
-	wait();
+    if ( d->running ) {
+#ifdef QT_CHECK_STATE
+	qWarning( "Attempt to start a thread already running" );
+#endif // QT_CHECK_STATE
+
+	d->thread_done.wait( locker.mutex() );
     }
 
-    d->init(this);
+    d->running = TRUE;
+    d->finished = FALSE;
+
+    int ret;
+    pthread_attr_t attr;
+    pthread_attr_init( &attr );
+    pthread_attr_setinheritsched( &attr, PTHREAD_INHERIT_SCHED );
+    pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_DETACHED );
+    if ( d->stacksize > 0 ) {
+	ret = pthread_attr_setstacksize( &attr, d->stacksize );
+	if ( ret ) {
+#ifdef QT_CHECK_STATE
+	    qWarning( "QThread::start: thread stack size error: %s", strerror( ret ) ) ;
+#endif // QT_CHECK_STATE
+
+	    // we failed to set the stacksize, and as the documentation states,
+	    // the thread will fail to run...
+	    d->running = FALSE;
+	    d->finished = FALSE;
+	    return;
+	}
+    }
+    d->args[0] = this;
+    d->args[1] = d;
+    ret = pthread_create( &d->thread_id, &attr, start_thread, d->args );
+    pthread_attr_destroy( &attr );
+
+    if ( ret ) {
+#ifdef QT_CHECK_STATE
+	qWarning( "QThread::start: thread creation error: %s", strerror( ret ) );
+#endif // QT_CHECK_STATE
+
+	d->running = FALSE;
+	d->finished = FALSE;
+    }
 }
 
+/*!
+    This function terminates the execution of the thread. The thread
+    may or may not be terminated immediately, depending on the
+    operating systems scheduling policies. Use QThread::wait()
+    after terminate() for synchronous termination.
+
+    When the thread is terminated, all threads waiting for the
+    the thread to finish will be woken up.
+
+    \warning This function is dangerous, and its use is discouraged.
+    The thread can be terminate at any point in its code path.  Threads
+    can be terminated while modifying data.  There is no chance for
+    the thread to cleanup after itself, unlock any held mutexes, etc.
+    In short, use this function only if \e absolutely necessary.
+*/
+void QThread::terminate()
+{
+    QMutexLocker private_locker( qt_thread_mutexpool->get( d ) );
+    if ( d->finished || !d->running )
+	return;
+    if ( ! d->thread_id )
+	return;
+    pthread_cancel( d->thread_id );
+}
 
 /*!
     This provides similar functionality to POSIX pthread_join. A thread
@@ -549,33 +453,32 @@ void QThread::start()
 	wait timed out.
     \endlist
 */
-bool QThread::wait(unsigned long time)
+bool QThread::wait( unsigned long time )
 {
-    if (d->finished || ! d->running)
+    QMutexLocker locker( qt_thread_mutexpool->get( d ) );
+    if ( d->finished || ! d->running )
 	return TRUE;
-
-    return d->thread_done.wait(time);
+    return d->thread_done.wait( locker.mutex(), time );
 }
 
-
 /*!
     Returns TRUE is the thread is finished; otherwise returns FALSE.
 */
 bool QThread::finished() const
 {
+    QMutexLocker locker( qt_thread_mutexpool->get( d ) );
     return d->finished;
 }
 
-
 /*!
     Returns TRUE if the thread is running; otherwise returns FALSE.
 */
 bool QThread::running() const
 {
+    QMutexLocker locker( qt_thread_mutexpool->get( d ) );
     return d->running;
 }
 
-
 /*!
     \fn void QThread::run()
 
@@ -586,7 +489,4 @@ bool QThread::running() const
     \sa wait()
 */
 
-#include "qthread_unix.moc"
-
 #endif // QT_THREAD_SUPPORT
-
