@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qtimer.cpp   3.0.5   edited Apr 30 14:26 $
+** $Id: qt/qtimer.cpp   3.1.2   edited Nov 7 16:55 $
 **
 ** Implementation of QTimer class
 **
@@ -141,9 +141,17 @@ QTimer::~QTimer()
     FALSE.
 */
 
+/*!
+    \fn int QTimer::timerId() const
+
+    Returns the ID of the timer if the timer is running; otherwise returns
+    -1.
+*/
+
 
 /*!
-    Starts the timer with a \a msec milliseconds timeout.
+    Starts the timer with a \a msec milliseconds timeout, and returns
+    the ID of the timer, or zero when starting the timer failed.
 
     If \a sshot is TRUE, the timer will be activated only once;
     otherwise it will continue until it is stopped.
