@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qurl.cpp   3.1.2   edited Nov 7 16:55 $
 **
 ** Implementation of QUrl class
 **
@@ -37,8 +37,7 @@
 
 #include "qurl.h"
 
-#ifndef QT_NO_NETWORKPROTOCOL
-#ifndef QT_NO_DIR
+#ifndef QT_NO_URL
 
 #include "qdir.h"
 
@@ -664,13 +663,13 @@ bool QUrl::parse( const QString& url )
     relPath = FALSE;
     bool forceRel = FALSE;
 
-    // if :/ is at pos 1, we have only one letter
+    // If ':' is at pos 1, we have only one letter
     // before that separator => that's a drive letter!
-    int cs = url_.find( ":/" );
-    if ( cs == 1 )
+    if ( url_.length() >= 2 && url_[1] == ':' )
 	relPath = forceRel = TRUE;
 
     int hasNoHost = -1;
+    int cs = url_.find( ":/" );
     if ( cs != -1 ) // if a protocol is there, find out if there is a host or directly the path after it
 	hasNoHost = url_.find( "///", cs );
     table[ 4 ][ 1 ] = User;
@@ -1036,8 +1035,6 @@ QString QUrl::path( bool correct ) const
 		if ( share ) {
 		    check = FALSE;
 		    d->cleanPath.prepend( "/" );
-		    if ( !d->cleanPath.endsWith( "/" ) )
-			d->cleanPath.append( "/" );
 		}
 	    }
 #endif
@@ -1318,5 +1315,4 @@ bool QUrl::cdUp()
     return TRUE;
 }
 
-#endif
-#endif // QT_NO_NETWORKPROTOCOL
+#endif // QT_NO_URL
