@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qvariant.cpp   3.0.5   edited Jul 1 12:44 $
+** $Id: qt/qvariant.cpp   3.1.2   edited Feb 5 09:24 $
 **
 ** Implementation of QVariant class
 **
@@ -35,6 +35,8 @@
 **
 **********************************************************************/
 
+#include <float.h>
+
 #include "qvariant.h"
 #ifndef QT_NO_VARIANT
 #include "qstring.h"
@@ -59,10 +61,16 @@
 #include "qshared.h"
 #include "qbitarray.h"
 #include "qkeysequence.h"
+#include "qpen.h"
+
+#ifndef DBL_DIG
+#define DBL_DIG 10
+#endif //DBL_DIG
 
 // Uncomment to test for memory leaks or to run qt/test/qvariant/main.cpp
 // #define QVARIANT_DEBUG
 
+
 #ifdef QVARIANT_DEBUG
 int qv_count = 0;
 int get_qv_count() { return qv_count; }
@@ -74,6 +82,7 @@ QVariant::Private::Private()
     qv_count++;
 #endif
     typ = QVariant::Invalid;
+    is_null = TRUE;
 }
 
 QVariant::Private::Private( Private* d )
@@ -178,6 +187,9 @@ QVariant::Private::Private( Private* d )
 	    value.ptr = new QKeySequence( *((QKeySequence*)d->value.ptr) );
 	    break;
 #endif
+	case QVariant::Pen:
+	    value.ptr = new QPen( *((QPen*)d->value.ptr) );
+	    break;
 	case QVariant::Int:
 	    value.i = d->value.i;
 	    break;
@@ -201,6 +213,7 @@ QVariant::Private::Private( Private* d )
 	}
 
     typ = d->typ;
+    is_null = d->is_null;
 }
 
 QVariant::Private::~Private()
@@ -306,6 +319,9 @@ void QVariant::Private::clear()
 	    delete (QKeySequence*)value.ptr;
 	    break;
 #endif
+	case QVariant::Pen:
+	    delete (QPen*)value.ptr;
+	    break;
 	case QVariant::Invalid:
 	case QVariant::Int:
 	case QVariant::UInt:
@@ -315,6 +331,7 @@ void QVariant::Private::clear()
 	}
 
     typ = QVariant::Invalid;
+    is_null = TRUE;
 }
 
 /*!
@@ -385,6 +402,14 @@ void QVariant::Private::clear()
     very powerful and versatile, but may prove less memory and speed
     efficient than storing specific types in standard data structures.
 
+    QVariant also supports the notion of NULL values, where you have a
+    defined type with no value set.
+    \code
+    QVariant x, y( QString() ), z( QString("") );
+    x.asInt();
+    // x.isNull() == TRUE, y.isNull() == TRUE, z.isNull() == FALSE
+    \endcode
+
     See the \link collection.html Collection Classes\endlink.
 */
 
@@ -414,6 +439,7 @@ void QVariant::Private::clear()
     \value List  a QValueList<QVariant>
     \value Map  a QMap<QString,QVariant>
     \value Palette  a QPalette
+    \value Pen  a QPen
     \value Pixmap  a QPixmap
     \value Point  a QPoint
     \value PointArray  a QPointArray
@@ -461,8 +487,8 @@ QVariant::~QVariant()
 */
 QVariant::QVariant( const QVariant& p )
 {
-    d = new Private;
-    *this = p;
+    p.d->ref();
+    d = p.d;
 }
 
 #ifndef QT_NO_DATASTREAM
@@ -524,6 +550,7 @@ QVariant::QVariant( const QStringList& val )
     d = new Private;
     d->typ = StringList;
     d->value.ptr = new QStringList( val );
+    d->is_null = FALSE;
 }
 #endif // QT_NO_STRINGLIST
 
@@ -536,6 +563,7 @@ QVariant::QVariant( const QMap<QString,QVariant>& val )
     d = new Private;
     d->typ = Map;
     d->value.ptr = new QMap<QString,QVariant>( val );
+    d->is_null = FALSE;
 }
 #endif
 /*!
@@ -546,6 +574,7 @@ QVariant::QVariant( const QFont& val )
     d = new Private;
     d->typ = Font;
     d->value.ptr = new QFont( val );
+    d->is_null = FALSE;
 }
 
 /*!
@@ -581,6 +610,7 @@ QVariant::QVariant( const QBrush& val )
     d = new Private;
     d->typ = Brush;
     d->value.ptr = new QBrush( val );
+    d->is_null = FALSE;
 }
 
 /*!
@@ -621,6 +651,7 @@ QVariant::QVariant( const QColor& val )
     d = new Private;
     d->typ = Color;
     d->value.ptr = new QColor( val );
+    d->is_null = FALSE;
 }
 
 #ifndef QT_NO_PALETTE
@@ -632,6 +663,7 @@ QVariant::QVariant( const QPalette& val )
     d = new Private;
     d->typ = Palette;
     d->value.ptr = new QPalette( val );
+    d->is_null = FALSE;
 }
 
 /*!
@@ -642,6 +674,7 @@ QVariant::QVariant( const QColorGroup& val )
     d = new Private;
     d->typ = ColorGroup;
     d->value.ptr = new QColorGroup( val );
+    d->is_null = FALSE;
 }
 #endif //QT_NO_PALETTE
 #ifndef QT_NO_ICONSET
@@ -685,6 +718,7 @@ QVariant::QVariant( const QCursor& val )
     d = new Private;
     d->typ = Cursor;
     d->value.ptr = new QCursor( val );
+    d->is_null = FALSE;
 }
 
 /*!
@@ -761,10 +795,21 @@ QVariant::QVariant( const QKeySequence& val )
     d = new Private;
     d->typ = KeySequence;
     d->value.ptr = new QKeySequence( val );
+    d->is_null = FALSE;
 }
 
 #endif
 
+/*!
+    Constructs a new variant with a pen value, \a val.
+*/
+QVariant::QVariant( const QPen& val )
+{
+    d = new Private;
+    d->typ = Pen;
+    d->value.ptr = new QPen( val );
+}
+
 /*!
     Constructs a new variant with an integer value, \a val.
 */
@@ -773,6 +818,7 @@ QVariant::QVariant( int val )
     d = new Private;
     d->typ = Int;
     d->value.i = val;
+    d->is_null = FALSE;
 }
 
 /*!
@@ -783,6 +829,7 @@ QVariant::QVariant( uint val )
     d = new Private;
     d->typ = UInt;
     d->value.u = val;
+    d->is_null = FALSE;
 }
 
 /*!
@@ -795,6 +842,7 @@ QVariant::QVariant( bool val, int )
     d = new Private;
     d->typ = Bool;
     d->value.b = val;
+    d->is_null = FALSE;
 }
 
 
@@ -806,6 +854,7 @@ QVariant::QVariant( double val )
     d = new Private;
     d->typ = Double;
     d->value.d = val;
+    d->is_null = FALSE;
 }
 
 #ifndef QT_NO_TEMPLATE_VARIANT
@@ -817,6 +866,7 @@ QVariant::QVariant( const QValueList<QVariant>& val )
     d = new Private;
     d->typ = List;
     d->value.ptr = new QValueList<QVariant>( val );
+    d->is_null = FALSE;
 }
 #endif
 
@@ -828,6 +878,7 @@ QVariant::QVariant( QSizePolicy val )
     d = new Private;
     d->typ = SizePolicy;
     d->value.ptr = new QSizePolicy( val );
+    d->is_null = FALSE;
 }
 
 /*!
@@ -870,7 +921,7 @@ void QVariant::detach()
 */
 const char* QVariant::typeName() const
 {
-    return typeToName( d->typ );
+    return typeToName( (Type) d->typ );
 }
 
 /*!
@@ -896,7 +947,7 @@ void QVariant::clear()
 
    (Search for the word 'Attention' in moc.y.)
 */
-static const int ntypes = 32;
+static const int ntypes = 33;
 static const char* const type_map[ntypes] =
 {
     0,
@@ -930,7 +981,8 @@ static const char* const type_map[ntypes] =
     "QDateTime",
     "QByteArray",
     "QBitArray",
-    "QKeySequence"
+    "QKeySequence",
+    "QPen"
 };
 
 
@@ -977,7 +1029,10 @@ void QVariant::load( QDataStream& s )
 
     switch( t ) {
     case Invalid:
-	d->typ = t;
+	{
+	    d->typ = t;
+	    d->is_null = TRUE;
+	}
 	break;
 #ifndef QT_NO_TEMPLATE_VARIANT
     case Map:
@@ -985,6 +1040,7 @@ void QVariant::load( QDataStream& s )
 	    QMap<QString,QVariant>* x = new QMap<QString,QVariant>;
 	    s >> *x;
 	    d->value.ptr = x;
+	    d->is_null = FALSE;
 	}
 	break;
     case List:
@@ -992,6 +1048,7 @@ void QVariant::load( QDataStream& s )
 	    QValueList<QVariant>* x = new QValueList<QVariant>;
 	    s >> *x;
 	    d->value.ptr = x;
+	    d->is_null = FALSE;
 	}
 	break;
 #endif
@@ -1001,6 +1058,7 @@ void QVariant::load( QDataStream& s )
 	    QCursor* x = new QCursor;
 	    s >> *x;
 	    d->value.ptr = x;
+	    d->is_null = FALSE;
 #endif
 	}
 	break;
@@ -1047,6 +1105,7 @@ void QVariant::load( QDataStream& s )
 	    QStringList* x = new QStringList;
 	    s >> *x;
 	    d->value.ptr = x;
+	    d->is_null = FALSE;
 	}
 	break;
 #endif // QT_NO_STRINGLIST
@@ -1055,6 +1114,7 @@ void QVariant::load( QDataStream& s )
 	    QFont* x = new QFont;
 	    s >> *x;
 	    d->value.ptr = x;
+	    d->is_null = FALSE;
 	}
 	break;
     case Pixmap:
@@ -1080,6 +1140,7 @@ void QVariant::load( QDataStream& s )
 	    QBrush* x = new QBrush;
 	    s >> *x;
 	    d->value.ptr = x;
+	    d->is_null = FALSE;
 	}
 	break;
     case Rect:
@@ -1108,6 +1169,7 @@ void QVariant::load( QDataStream& s )
 	    QColor* x = new QColor;
 	    s >> *x;
 	    d->value.ptr = x;
+	    d->is_null = FALSE;
 	}
 	break;
 #ifndef QT_NO_PALETTE
@@ -1116,6 +1178,7 @@ void QVariant::load( QDataStream& s )
 	    QPalette* x = new QPalette;
 	    s >> *x;
 	    d->value.ptr = x;
+	    d->is_null = FALSE;
 	}
 	break;
     case ColorGroup:
@@ -1123,6 +1186,7 @@ void QVariant::load( QDataStream& s )
 	    QColorGroup* x = new QColorGroup;
 	    s >> *x;
 	    d->value.ptr = x;
+	    d->is_null = FALSE;
 	}
 	break;
 #endif
@@ -1140,6 +1204,7 @@ void QVariant::load( QDataStream& s )
 	    int x;
 	    s >> x;
 	    d->value.i = x;
+	    d->is_null = FALSE;
 	}
 	break;
     case UInt:
@@ -1147,6 +1212,7 @@ void QVariant::load( QDataStream& s )
 	    uint x;
 	    s >> x;
 	    d->value.u = x;
+	    d->is_null = FALSE;
 	}
 	break;
     case Bool:
@@ -1154,6 +1220,7 @@ void QVariant::load( QDataStream& s )
 	    Q_INT8 x;
 	    s >> x;
 	    d->value.b = x;
+	    d->is_null = FALSE;
 	}
 	break;
     case Double:
@@ -1161,6 +1228,7 @@ void QVariant::load( QDataStream& s )
 	    double x;
 	    s >> x;
 	    d->value.d = x;
+	    d->is_null = FALSE;
 	}
 	break;
     case SizePolicy:
@@ -1171,6 +1239,7 @@ void QVariant::load( QDataStream& s )
 	    d->value.ptr = new QSizePolicy( (QSizePolicy::SizeType)h,
 					    (QSizePolicy::SizeType)v,
 					    (bool) hfw);
+	    d->is_null = FALSE;
 	}
 	break;
     case Date:
@@ -1210,11 +1279,22 @@ void QVariant::load( QDataStream& s )
 	break;
 #ifndef QT_NO_ACCEL
     case KeySequence:
-	QKeySequence* x = new QKeySequence;
- 	s >> *x;
-	d->value.ptr = x;
+	{
+	    QKeySequence* x = new QKeySequence;
+	    s >> *x;
+	    d->value.ptr = x;
+	    d->is_null = FALSE;
+	}
 	break;
 #endif // QT_NO_ACCEL
+    case Pen:
+	{
+	    QPen* x = new QPen;
+	    s >> *x;
+	    d->value.ptr = x;
+	    d->is_null = FALSE;
+	}
+	break;
     }
     d->typ = t;
 }
@@ -1339,9 +1419,14 @@ void QVariant::save( QDataStream& s ) const
     case BitArray:
 	s << *((QBitArray*)d->value.ptr);
 	break;
+#ifndef QT_NO_ACCEL
     case KeySequence:
 	s << *((QKeySequence*)d->value.ptr);
 	break;
+#endif
+    case Pen:
+	s << *((QPen*)d->value.ptr);
+	break;
     case Invalid:
 	s << QString(); // ### looks wrong.
 	break;
@@ -1463,40 +1548,47 @@ QDataStream& operator<< ( QDataStream& s, const QVariant::Type p )
 
 /*!
     Returns the variant as a QString if the variant has type() String,
-    CString, ByteArray, Int, Uint, Bool, Double, Date, Time, or
-    DateTime; otherwise returns QString::null.
+    CString, ByteArray, Int, Uint, Bool, Double, Date, Time, DateTime,
+    KeySequence, Font or Color; otherwise returns QString::null.
 
     \sa asString()
 */
 const QString QVariant::toString() const
 {
-    if ( d->typ == CString )
+    switch( d->typ ) {
+    case CString:
 	return QString::fromLatin1( toCString() );
-    if ( d->typ == Int )
+    case Int:
 	return QString::number( toInt() );
-    if ( d->typ == UInt )
+    case UInt:
 	return QString::number( toUInt() );
-    if ( d->typ == Double )
-	return QString::number( toDouble() );
-#ifndef QT_NO_SPRINTF
-    if ( d->typ == Date )
+    case Double:
+	return QString::number( toDouble(), 'g', DBL_DIG );
+#if !defined(QT_NO_SPRINTF) && !defined(QT_NO_DATESTRING)
+    case Date:
 	return toDate().toString( Qt::ISODate );
-    if ( d->typ == Time )
+    case Time:
 	return toTime().toString( Qt::ISODate );
-    if ( d->typ == DateTime )
+    case DateTime:
 	return toDateTime().toString( Qt::ISODate );
 #endif
-    if ( d->typ == Bool )
-	return QString::number( toInt() );
+    case Bool:
+	return toInt() ? "true" : "false";
 #ifndef QT_NO_ACCEL
-    if ( d->typ == KeySequence )
+    case KeySequence:
 	return (QString) *( (QKeySequence*)d->value.ptr );
 #endif
-    if ( d->typ == ByteArray )
+    case ByteArray:
 	return QString( *((QByteArray*)d->value.ptr) );
-    if ( d->typ != String )
+    case Font:
+	return toFont().toString();
+    case Color:
+	return toColor().name();
+    case String:
+	return *((QString*)d->value.ptr);
+    default:
 	return QString::null;
-    return *((QString*)d->value.ptr);
+    }
 }
 /*!
     Returns the variant as a QCString if the variant has type()
@@ -1506,12 +1598,14 @@ const QString QVariant::toString() const
 */
 const QCString QVariant::toCString() const
 {
-    if ( d->typ == CString )
+    switch ( d->typ ) {
+    case CString:
 	return *((QCString*)d->value.ptr);
-    if ( d->typ == String )
+    case String:
 	return ((QString*)d->value.ptr)->latin1();
-
-    return 0;
+    default:
+	return 0;
+    }
 }
 
 
@@ -1536,23 +1630,26 @@ const QCString QVariant::toCString() const
 */
 const QStringList QVariant::toStringList() const
 {
-    if ( d->typ == StringList )
+    switch ( d->typ ) {
+    case StringList:
 	return *((QStringList*)d->value.ptr);
 #ifndef QT_NO_TEMPLATE_VARIANT
-    if ( d->typ == List ) {
-	QStringList lst;
-	QValueList<QVariant>::ConstIterator it = listBegin();
-	QValueList<QVariant>::ConstIterator end = listEnd();
-	while( it != end ) {
-	    QString tmp = (*it).toString();
-	    ++it;
-	    lst.append( tmp );
+    case List:
+	{
+	    QStringList lst;
+	    QValueList<QVariant>::ConstIterator it = listBegin();
+	    QValueList<QVariant>::ConstIterator end = listEnd();
+	    while( it != end ) {
+		QString tmp = (*it).toString();
+		++it;
+		lst.append( tmp );
+	    }
+	    return lst;
 	}
-	return lst;
-    }
 #endif
-
-    return QStringList();
+    default:
+	return QStringList();
+    }
 }
 #endif //QT_NO_STRINGLIST
 
@@ -1590,10 +1687,18 @@ const QMap<QString, QVariant> QVariant::toMap() const
 */
 const QFont QVariant::toFont() const
 {
-    if ( d->typ != Font )
+    switch ( d->typ ) {
+    case String:
+	{
+	    QFont fnt;
+	    fnt.fromString( toString() );
+	    return fnt;
+	}
+    case Font:
+	return *((QFont*)d->value.ptr);
+    default:
 	return QFont();
-
-    return *((QFont*)d->value.ptr);
+    }
 }
 
 /*!
@@ -1688,10 +1793,18 @@ const QSize QVariant::toSize() const
 */
 const QColor QVariant::toColor() const
 {
-    if ( d->typ != Color )
+    switch ( d->typ ) {
+    case String:
+	{
+	    QColor col;
+	    col.setNamedColor( toString() );
+	    return col;
+	}
+    case Color:
+	return *((QColor*)d->value.ptr);
+    default:
 	return QColor();
-
-    return *((QColor*)d->value.ptr);
+    }
 }
 #ifndef QT_NO_PALETTE
 /*!
@@ -1806,13 +1919,18 @@ const QCursor QVariant::toCursor() const
 */
 const QDate QVariant::toDate() const
 {
-    if ( d->typ == Date )
+    switch ( d->typ ) {
+    case Date:
 	return *((QDate*)d->value.ptr);
-    if ( d->typ == DateTime )
+    case DateTime:
 	return ((QDateTime*)d->value.ptr)->date();
-    if ( d->typ == String )
+#ifndef QT_NO_DATESTRING
+    case String:
 	return QDate::fromString( *((QString*)d->value.ptr), Qt::ISODate );
-    return QDate();
+#endif
+    default:
+	return QDate();
+    }
 }
 
 /*!
@@ -1826,13 +1944,18 @@ const QDate QVariant::toDate() const
 */
 const QTime QVariant::toTime() const
 {
-    if ( d->typ == Time )
+    switch ( d->typ ) {
+    case Time:
 	return *((QTime*)d->value.ptr);
-    if ( d->typ == DateTime )
+    case DateTime:
 	return ((QDateTime*)d->value.ptr)->time();
-    if ( d->typ == String )
+#ifndef QT_NO_DATESTRING
+    case String:
 	return QTime::fromString( *((QString*)d->value.ptr), Qt::ISODate );
-    return QTime();
+#endif
+    default:
+	return QTime();
+    }
 }
 
 /*!
@@ -1847,18 +1970,23 @@ const QTime QVariant::toTime() const
 */
 const QDateTime QVariant::toDateTime() const
 {
-    if ( d->typ == DateTime )
+    switch ( d->typ ) {
+    case DateTime:
 	return *((QDateTime*)d->value.ptr);
-    if ( d->typ == String )
+#ifndef QT_NO_DATESTRING
+    case String:
 	return QDateTime::fromString( *((QString*)d->value.ptr), Qt::ISODate );
-    if ( d->typ == Date )
+#endif
+    case Date:
 	return QDateTime( *((QDate*)d->value.ptr) );
-    return QDateTime();
+    default:
+	return QDateTime();
+    }
 }
 
 /*!
     Returns the variant as a QByteArray if the variant has type()
-    ByteArray; otherwise returns an empty bytearray.
+    ByteArray or CString; otherwise returns an empty bytearray.
 
     \sa asByteArray()
 */
@@ -1866,6 +1994,8 @@ const QByteArray QVariant::toByteArray() const
 {
     if ( d->typ == ByteArray )
 	return *((QByteArray*)d->value.ptr);
+    if ( d->typ == CString )
+	return *((QByteArray*)d->value.ptr);
     return QByteArray();
 }
 
@@ -1896,17 +2026,34 @@ const QBitArray QVariant::toBitArray() const
 */
 const QKeySequence QVariant::toKeySequence() const
 {
-    if ( d->typ == KeySequence )
+    switch ( d->typ ) {
+    case KeySequence:
 	return *((QKeySequence*)d->value.ptr);
-    if ( d->typ == String )
+    case String:
 	return QKeySequence( toString() );
-    if ( d->typ == Int )
+    case Int:
 	return QKeySequence( toInt() );
-    return QKeySequence();
+    default:
+	return QKeySequence();
+    }
 }
 
 #endif // QT_NO_ACCEL
 
+/*!
+    Returns the variant as a QPen if the variant has type()
+    Pen; otherwise returns an empty QPen.
+
+    \sa asPen()
+*/
+const QPen QVariant::toPen() const
+{
+    if ( d->typ != Pen )
+	return QPen();
+
+    return *((QPen*)d->value.ptr);
+}
+
 /*!
     Returns the variant as an int if the variant has type() String,
     CString, Int, UInt, Double, Bool or KeySequence; otherwise returns
@@ -1919,26 +2066,29 @@ const QKeySequence QVariant::toKeySequence() const
 */
 int QVariant::toInt( bool * ok ) const
 {
-    if( d->typ == String )
-	return ((QString*)d->value.ptr)->toInt( ok );
-    if ( d->typ == CString )
-	return ((QCString*)d->value.ptr)->toInt( ok );
     if ( ok )
 	*ok = canCast( UInt );
-    if( d->typ == Int )
+
+    switch ( d->typ ) {
+    case String:
+	return ((QString*)d->value.ptr)->toInt( ok );
+    case CString:
+	return ((QCString*)d->value.ptr)->toInt( ok );
+    case Int:
 	return d->value.i;
-    if( d->typ == UInt )
+    case UInt:
 	return (int)d->value.u;
-    if ( d->typ == Double )
+    case Double:
 	return (int)d->value.d;
-    if ( d->typ == Bool )
+    case Bool:
 	return (int)d->value.b;
 #ifndef QT_NO_ACCEL
-    if ( d->typ == KeySequence )
+    case KeySequence:
 	return (int) *( (QKeySequence*)d->value.ptr );
 #endif
-
-    return 0;
+    default:
+	return 0;
+    }
 }
 
 /*!
@@ -1952,22 +2102,25 @@ int QVariant::toInt( bool * ok ) const
 */
 uint QVariant::toUInt( bool * ok ) const
 {
-    if( d->typ == String )
-	return ((QString*)d->value.ptr)->toUInt( ok );
-    if ( d->typ == CString )
-	return ((QCString*)d->value.ptr)->toUInt( ok );
     if ( ok )
 	*ok = canCast( UInt );
-    if( d->typ == Int )
-	return d->value.i;
-    if( d->typ == UInt )
-	return (int)d->value.u;
-    if ( d->typ == Double )
-	return (int)d->value.d;
-    if ( d->typ == Bool )
-	return (int)d->value.b;
 
-    return 0;
+    switch( d->typ ) {
+    case String:
+	return ((QString*)d->value.ptr)->toUInt( ok );
+    case CString:
+	return ((QCString*)d->value.ptr)->toUInt( ok );
+    case Int:
+	return d->value.i;
+    case UInt:
+	return (int)d->value.u;
+    case Double:
+	return (int)d->value.d;
+    case Bool:
+	return (int)d->value.b;
+    default:
+	return 0;
+    }
 }
 
 /*!
@@ -1980,16 +2133,18 @@ uint QVariant::toUInt( bool * ok ) const
 */
 bool QVariant::toBool() const
 {
-    if ( d->typ == Bool )
+    switch( d->typ ) {
+    case Bool:
 	return d->value.b;
-    if ( d->typ == Double )
+    case Double:
 	return d->value.d != 0.0;
-    if ( d->typ == Int )
+    case Int:
 	return d->value.i != 0;
-    if ( d->typ == UInt )
+    case UInt:
 	return d->value.u != 0;
-
-    return FALSE;
+    default:
+	return FALSE;
+    }
 }
 
 /*!
@@ -2003,21 +2158,25 @@ bool QVariant::toBool() const
 */
 double QVariant::toDouble( bool * ok ) const
 {
-    if( d->typ == String )
-	return ((QString*)d->value.ptr)->toDouble( ok );
-    if ( d->typ == CString )
-	return ((QCString*)d->value.ptr)->toDouble( ok );
     if ( ok )
 	*ok = canCast( Double );
-    if ( d->typ == Double )
+
+    switch ( d->typ ) {
+    case String:
+	return ((QString*)d->value.ptr)->toDouble( ok );
+    case CString:
+	return ((QCString*)d->value.ptr)->toDouble( ok );
+    case Double:
 	return d->value.d;
-    if ( d->typ == Int )
+    case Int:
 	return (double)d->value.i;
-    if ( d->typ == Bool )
+    case Bool:
 	return (double)d->value.b;
-    if ( d->typ == UInt )
+    case UInt:
 	return (double)d->value.u;
-    return 0.0;
+    default:
+	return 0.0;
+    }
 }
 
 #ifndef QT_NO_TEMPLATE_VARIANT
@@ -2107,6 +2266,7 @@ Q_VARIANT_AS(BitArray)
 #ifndef QT_NO_ACCEL
 Q_VARIANT_AS(KeySequence)
 #endif
+Q_VARIANT_AS(Pen)
 
 /*!
     \fn QString& QVariant::asString()
@@ -2392,13 +2552,23 @@ Q_VARIANT_AS(KeySequence)
     \sa toKeySequence()
 */
 
+/*! \fn QPen& QVariant::asPen()
+
+  Tries to convert the variant to hold a QPen value. If that
+  is not possible then the variant is set to an empty pen.
+
+  Returns a reference to the stored pen.
+
+  \sa toPen()
+*/
+
 /*!
     Returns the variant's value as int reference.
 */
 int& QVariant::asInt()
 {
-    detach();
     if ( d->typ != Int ) {
+	detach();
 	int i = toInt();
 	d->clear();
  	d->value.i = i;
@@ -2412,8 +2582,8 @@ int& QVariant::asInt()
 */
 uint& QVariant::asUInt()
 {
-    detach();
     if ( d->typ != UInt ) {
+	detach();
 	uint u = toUInt();
 	d->clear();
 	d->value.u = u;
@@ -2427,8 +2597,8 @@ uint& QVariant::asUInt()
 */
 bool& QVariant::asBool()
 {
-    detach();
     if ( d->typ != Bool ) {
+	detach();
 	bool b = toBool();
 	d->clear();
 	d->value.b = b;
@@ -2443,6 +2613,7 @@ bool& QVariant::asBool()
 double& QVariant::asDouble()
 {
     if ( d->typ != Double ) {
+	detach();
 	double dbl = toDouble();
 	d->clear();
 	d->value.d = dbl;
@@ -2504,59 +2675,74 @@ QMap<QString, QVariant>& QVariant::asMap()
     \table
     \header \i Type \i Automatically Cast To
     \row \i Bool \i Double, Int, UInt
-    \row \i CString \i String
-    \row \i Date \i String
+    \row \i Color \i String
+    \row \i Date \i String, DateTime
     \row \i DateTime \i String, Date, Time
     \row \i Double \i String, Int, Bool, UInt
+    \row \i Font \i String
     \row \i Int \i String, Double, Bool, UInt
-    \row \i List \i StringList (if the list contains strings or something
-    \row     that can be cast to a string)
-    \row \i String \i CString, Int, Uint, Double, Date, Time, DateTime
+    \row \i List \i StringList (if the list contains strings or
+    something that can be cast to a string)
+    \row \i String \i CString, Int, Uint, Bool, Double, Date, 
+    Time, DateTime, KeySequence, Font, Color
+    \row \i CString \i String
     \row \i StringList \i List
     \row \i Time \i String
     \row \i UInt \i String, Double, Bool, Int
+    \row \i KeySequence \i String, Int
     \endtable
 */
 bool QVariant::canCast( Type t ) const
 {
-    if ( d->typ == t )
-	return TRUE;
-    if ( t == Bool && ( d->typ == Double || d->typ == Int || d->typ == UInt ) )
-	 return TRUE;
-    if ( t == Int && ( d->typ == String || d->typ == Double || d->typ == Bool || d->typ == UInt  || d->typ == KeySequence ) )
-	return TRUE;
-    if ( t == UInt && ( d->typ == String || d->typ == Double || d->typ == Bool || d->typ == Int ) )
-	return TRUE;
-    if ( t == Double && ( d->typ == String || d->typ == Int || d->typ == Bool || d->typ == UInt ) )
-	return TRUE;
-    if ( t == CString && d->typ == String )
-	return TRUE;
-    if ( t == String && ( d->typ == CString || d->typ == Int || d->typ == UInt || d->typ == Double || d->typ == Date || d->typ == Time || d->typ == DateTime || d->typ == KeySequence ) )
-	return TRUE;
-    if ( t == Date && ( d->typ == String || d->typ == DateTime ) )
-	return TRUE;
-    if ( t == Time && ( d->typ == String || d->typ == DateTime ) )
-	return TRUE;
-    if ( t == DateTime && d->typ == String )
-	return TRUE;
-    if ( ( t == KeySequence && ( d->typ == String || d->typ == Int ) ) )
+    if ( Type( d->typ ) == t )
 	return TRUE;
+    switch ( t ) {
+    case Bool:
+	return d->typ == Double || d->typ == Int || d->typ == UInt;
+    case Int:
+	return d->typ == String || d->typ == Double || d->typ == Bool || d->typ == UInt  || d->typ == KeySequence;
+    case UInt:
+	return d->typ == String || d->typ == Double || d->typ == Bool || d->typ == Int;
+    case Double:
+	return d->typ == String || d->typ == Int || d->typ == Bool || d->typ == UInt;
+    case CString:
+	return d->typ == String;
+    case String:
+	return d->typ == CString || d->typ == ByteArray || d->typ == Int || d->typ == UInt || d->typ == Bool || d->typ == Double || d->typ == Date || d->typ == Time || d->typ == DateTime || d->typ == KeySequence || d->typ == Font || d->typ == Color;
+    case ByteArray:
+	return d->typ == CString;
+    case Date:
+	return d->typ == String || d->typ == DateTime;
+    case Time:
+	return d->typ == String || d->typ == DateTime;
+    case DateTime:
+	return d->typ == String || d->typ == Date;
+    case KeySequence:
+	return d->typ == String || d->typ == Int;
+    case Font:
+	return d->typ == String;
+    case Color:
+	return d->typ == String;
 #ifndef QT_NO_STRINGLIST
-    if ( t == List && d->typ == StringList )
-	return TRUE;
+    case List:
+	return d->typ == StringList;
 #endif
 #ifndef QT_NO_TEMPLATE_VARIANT
-    if ( t == StringList && d->typ == List ) {
-	QValueList<QVariant>::ConstIterator it = listBegin();
-	QValueList<QVariant>::ConstIterator end = listEnd();
-	for( ; it != end; ++it ) {
-	    if ( !(*it).canCast( String ) )
-		return FALSE;
+    case StringList:
+	if ( d->typ == List ) {
+	    QValueList<QVariant>::ConstIterator it = listBegin();
+	    QValueList<QVariant>::ConstIterator end = listEnd();
+	    for( ; it != end; ++it ) {
+		if ( !(*it).canCast( String ) )
+		    return FALSE;
+	    }
+	    return TRUE;
 	}
-	return TRUE;
-    }
+	return FALSE;
 #endif
-    return FALSE;
+    default:
+	return FALSE;
+    }
 }
 
 /*!
@@ -2676,6 +2862,9 @@ bool QVariant::cast( Type t )
 	asKeySequence();
 	break;
 #endif
+    case QVariant::Pen:
+	asPen();
+	break;
     default:
     case QVariant::Invalid:
 	(*this) = QVariant();
@@ -2711,9 +2900,11 @@ bool QVariant::operator==( const QVariant &v ) const
 	    return FALSE;
 	QMap<QString, QVariant>::ConstIterator it = v.toMap().begin();
 	QMap<QString, QVariant>::ConstIterator it2 = toMap().begin();
-	for ( ; it != v.toMap().end(); ++it ) {
+	while ( it != v.toMap().end() ) {
 	    if ( *it != *it2 )
 		return FALSE;
+	    ++it;
+	    ++it2;
 	}
 	return TRUE;
     }
@@ -2777,6 +2968,8 @@ bool QVariant::operator==( const QVariant &v ) const
     case KeySequence:
 	return v.toKeySequence() == toKeySequence();
 #endif
+    case Pen:
+	return v.toPen() == toPen();
     case Invalid:
 	break;
     }
@@ -2792,4 +2985,104 @@ bool QVariant::operator!=( const QVariant &v ) const
 {
     return !( v == *this );
 }
+
+
+/*! \internal
+
+  Reads or sets the variant type and ptr
+ */
+void* QVariant::rawAccess( void* ptr, Type typ, bool deepCopy )
+{
+    if ( ptr ) {
+	clear();
+	d->typ = typ;
+	d->value.ptr = ptr;
+	d->is_null = FALSE;
+	if ( deepCopy ) {
+	    QVariant::Private* p = new Private( d );
+	    d->typ = Invalid;
+	    delete d;
+	    d = p;
+	}
+    }
+
+    if ( !deepCopy )
+	return d->value.ptr;
+    QVariant::Private* p = new Private( d );
+    void *ret = (void*)p->value.ptr;
+    p->typ = Invalid;
+    delete p;
+    return ret;
+}
+
+/*!
+  Returns TRUE if this is a NULL variant, FALSE otherwise.
+*/
+bool QVariant::isNull() const
+{
+    switch( d->typ )
+	{
+	case QVariant::Bitmap:
+	    return ((QBitmap*) d->value.ptr)->isNull();
+	case QVariant::Region:
+	    return ((QRegion*) d->value.ptr)->isNull();
+	case QVariant::PointArray:
+	    return ((QPointArray*) d->value.ptr)->isNull();
+	case QVariant::String:
+	    return ((QString*) d->value.ptr)->isNull();
+	case QVariant::CString:
+	    return ((QCString*) d->value.ptr)->isNull();
+	case QVariant::Pixmap:
+	    return ((QPixmap*) d->value.ptr)->isNull();
+	case QVariant::Image:
+	    return ((QImage*) d->value.ptr)->isNull();
+	case QVariant::Point:
+	    return ((QPoint*) d->value.ptr)->isNull();
+	case QVariant::Rect:
+	    return ((QRect*) d->value.ptr)->isNull();
+	case QVariant::Size:
+	    return ((QSize*) d->value.ptr)->isNull();
+#ifndef QT_NO_ICONSET
+	case QVariant::IconSet:
+	    return ((QIconSet*) d->value.ptr)->isNull();
+#endif
+	case QVariant::Date:
+	    return ((QDate*) d->value.ptr)->isNull();
+	case QVariant::Time:
+	    return ((QTime*) d->value.ptr)->isNull();
+	case QVariant::DateTime:
+	    return ((QDateTime*) d->value.ptr)->isNull();
+	case QVariant::ByteArray:
+	    return ((QByteArray*) d->value.ptr)->isNull();
+	case QVariant::BitArray:
+	    return ((QBitArray*) d->value.ptr)->isNull();
+	case QVariant::Cursor:
+#ifndef QT_NO_STRINGLIST
+	case QVariant::StringList:
+#endif //QT_NO_STRINGLIST
+	case QVariant::Font:
+	case QVariant::Brush:
+	case QVariant::Color:
+#ifndef QT_NO_PALETTE
+	case QVariant::Palette:
+	case QVariant::ColorGroup:
+#endif
+#ifndef QT_NO_TEMPLATE_VARIANT
+	case QVariant::Map:
+	case QVariant::List:
+#endif
+	case QVariant::SizePolicy:
+#ifndef QT_NO_ACCEL
+	case QVariant::KeySequence:
+#endif
+	case QVariant::Pen:
+	case QVariant::Invalid:
+	case QVariant::Int:
+	case QVariant::UInt:
+	case QVariant::Bool:
+	case QVariant::Double:
+	    break;
+	}
+    return d->is_null;
+}
 #endif //QT_NO_VARIANT
