@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qcommonstyle.cpp   3.1.2   edited Dec 3 12:13 $
 **
 ** Implementation of the QCommonStyle class
 **
@@ -60,19 +60,20 @@
 #include "qbitmap.h"
 #include "qprogressbar.h"
 #include <limits.h>
+#include <qpixmap.h>
 #include "../widgets/qtitlebar_p.h"
 
-// NOT REVISED
 /*!
-  \class QCommonStyle qcommonstyle.h
-  \brief The QCommonStyle class encapsulates the common Look and Feel of a GUI.
+    \class QCommonStyle qcommonstyle.h
+    \brief The QCommonStyle class encapsulates the common Look and Feel of a GUI.
 
-  \ingroup appearance
+    \ingroup appearance
 
-  This abstract class implements some of the widget's look and feel
-  that is common to all GUI styles provided and shipped as part of Qt.
+    This abstract class implements some of the widget's look and feel
+    that is common to all GUI styles provided and shipped as part of
+    Qt.
 
-  All the functions are documented in \l QStyle.
+    All the functions are documented in \l QStyle.
 */
 
 /*!
@@ -88,7 +89,7 @@
 static QPainter *activePainter = 0;
 
 /*!
-  Constructs a QCommonStyle.
+    Constructs a QCommonStyle.
 */
 QCommonStyle::QCommonStyle() : QStyle()
 {
@@ -102,6 +103,29 @@ QCommonStyle::~QCommonStyle()
 }
 
 
+static const char * const check_list_controller_xpm[] = {
+"16 16 4 1",
+"	c None",
+".	c #000000000000",
+"X	c #FFFFFFFF0000",
+"o	c #C71BC30BC71B",
+"                ",
+"                ",
+" ..........     ",
+" .XXXXXXXX.     ",
+" .XXXXXXXX.oo   ",
+" .XXXXXXXX.oo   ",
+" .XXXXXXXX.oo   ",
+" .XXXXXXXX.oo   ",
+" .XXXXXXXX.oo   ",
+" .XXXXXXXX.oo   ",
+" .XXXXXXXX.oo   ",
+" ..........oo   ",
+"   oooooooooo   ",
+"   oooooooooo   ",
+"                ",
+"                "};
+
 /*! \reimp */
 void QCommonStyle::drawPrimitive( PrimitiveElement pe,
 				  QPainter *p,
@@ -113,6 +137,108 @@ void QCommonStyle::drawPrimitive( PrimitiveElement pe,
     activePainter = p;
 
     switch (pe) {
+#ifndef QT_NO_LISTVIEW
+    case PE_CheckListController: {
+	p->drawPixmap(r, QPixmap((const char **)check_list_controller_xpm));
+	break; }
+    case PE_CheckListExclusiveIndicator: {
+	QCheckListItem *item = opt.checkListItem();
+	QListView *lv = item->listView();
+	if(!item)
+	    return;
+	int x = r.x(), y = r.y();
+#define QCOORDARRLEN(x) sizeof(x)/(sizeof(QCOORD)*2)
+	static const QCOORD pts1[] = {		// dark lines
+	    1,9, 1,8, 0,7, 0,4, 1,3, 1,2, 2,1, 3,1, 4,0, 7,0, 8,1, 9,1 };
+	static const QCOORD pts2[] = {		// black lines
+	    2,8, 1,7, 1,4, 2,3, 2,2, 3,2, 4,1, 7,1, 8,2, 9,2 };
+	static const QCOORD pts3[] = {		// background lines
+	    2,9, 3,9, 4,10, 7,10, 8,9, 9,9, 9,8, 10,7, 10,4, 9,3 };
+	static const QCOORD pts4[] = {		// white lines
+	    2,10, 3,10, 4,11, 7,11, 8,10, 9,10, 10,9, 10,8, 11,7,
+	    11,4, 10,3, 10,2 };
+	// static const QCOORD pts5[] = {		// inner fill
+	//    4,2, 7,2, 9,4, 9,7, 7,9, 4,9, 2,7, 2,4 };
+	//QPointArray a;
+	//	p->eraseRect( x, y, w, h );
+
+	if ( flags & Style_Enabled )
+	    p->setPen( cg.text() );
+	else
+	    p->setPen( QPen( lv->palette().color( QPalette::Disabled, QColorGroup::Text ) ) );
+	QPointArray a( QCOORDARRLEN(pts1), pts1 );
+	a.translate( x, y );
+	//p->setPen( cg.dark() );
+	p->drawPolyline( a );
+	a.setPoints( QCOORDARRLEN(pts2), pts2 );
+	a.translate( x, y );
+	p->drawPolyline( a );
+	a.setPoints( QCOORDARRLEN(pts3), pts3 );
+	a.translate( x, y );
+	//		p->setPen( black );
+	p->drawPolyline( a );
+	a.setPoints( QCOORDARRLEN(pts4), pts4 );
+	a.translate( x, y );
+	//			p->setPen( blue );
+	p->drawPolyline( a );
+	//		a.setPoints( QCOORDARRLEN(pts5), pts5 );
+	//		a.translate( x, y );
+	//	QColor fillColor = isDown() ? g.background() : g.base();
+	//	p->setPen( fillColor );
+	//	p->setBrush( fillColor );
+	//	p->drawPolygon( a );
+	if ( flags & Style_On ) {
+	    p->setPen( NoPen );
+	    p->setBrush( cg.text() );
+	    p->drawRect( x+5, y+4, 2, 4 );
+	    p->drawRect( x+4, y+5, 4, 2 );
+	}
+	break; }
+    case PE_CheckListIndicator: {
+	QCheckListItem *item = opt.checkListItem();
+	QListView *lv = item->listView();
+	if(!item)
+	    return;
+	int x = r.x(), y = r.y(), w = r.width(), h = r.width(), marg = lv->itemMargin();
+
+	if ( flags & Style_Enabled )
+	    p->setPen( QPen( cg.text(), 2 ) );
+	else
+	    p->setPen( QPen( lv->palette().color( QPalette::Disabled, QColorGroup::Text ),
+			     2 ) );
+	bool parentControl = FALSE;
+	if ( item->parent() && item->parent()->rtti() == 1  &&
+	     ((QCheckListItem*) item->parent())->type() == QCheckListItem::Controller )
+	    parentControl = TRUE;
+	if ( flags & Style_Selected && !lv->rootIsDecorated() && !parentControl ) {
+	    p->fillRect( 0, 0, x + marg + w + 4, item->height(),
+			 cg.brush( QColorGroup::Highlight ) );
+	    if ( item->isEnabled() )
+		p->setPen( QPen( cg.highlightedText(), 2 ) );
+	}
+
+	p->drawRect( x+marg, y+2, w-4, h-4 );
+	/////////////////////
+	x++;
+	y++;
+	if ( flags & Style_On ) {
+	    QPointArray a( 7*2 );
+	    int i, xx = x+1+marg, yy=y+5;
+	    for ( i=0; i<3; i++ ) {
+		a.setPoint( 2*i,   xx, yy );
+		a.setPoint( 2*i+1, xx, yy+2 );
+		xx++; yy++;
+	    }
+	    yy -= 2;
+	    for ( i=3; i<7; i++ ) {
+		a.setPoint( 2*i,   xx, yy );
+		a.setPoint( 2*i+1, xx, yy+2 );
+		xx++; yy--;
+	    }
+	    p->drawLineSegments( a );
+	}
+	break; }
+#endif
     case PE_HeaderArrow:
 	p->save();
 	if ( flags & Style_Down ) {
@@ -426,57 +552,18 @@ void QCommonStyle::drawPrimitive( PrimitiveElement pe,
 	p->drawLineSegments( a );
 	break; }
 
+    case PE_PanelGroupBox: //We really do not need PE_GroupBoxFrame anymore, nasty holdover ###
+	drawPrimitive( PE_GroupBoxFrame, p, r, cg, flags, opt );
+	break;
     case PE_GroupBoxFrame: {
 #ifndef QT_NO_FRAME
 	if ( opt.isDefault() )
 	    break;
-
-	QFrame::Shape	type 	= (QFrame::Shape)opt.frameShape();
-	QFrame::Shadow	cstyle 	= (QFrame::Shadow)opt.frameShadow();
-	int 		lwidth 	= opt.lineWidth();
-	int 		mlwidth = opt.midLineWidth();
-
-	int x = r.x();
-	int y = r.y();
-	int w = r.width();
-	int h = r.height();
-
-	switch ( type ) {
-	case QFrame::Box:
-	    if ( cstyle == QFrame::Plain )
-		qDrawPlainRect( p, x, y, w, h, cg.foreground(), lwidth );
-	    else
-		qDrawShadeRect( p, x, y, w, h, cg, cstyle == QFrame::Sunken,
-				lwidth, mlwidth );
-	    break;
-
-	case QFrame::StyledPanel:
-	    if ( cstyle == QFrame::Plain )
-		qDrawPlainRect( p, x, y, w, h, cg.foreground(), lwidth );
-	    break;
-
-	case QFrame::PopupPanel:
-	    if ( cstyle == QFrame::Plain )
-		qDrawPlainRect( p, x, y, w, h, cg.foreground(), lwidth );
-	    break;
-
-	case QFrame::Panel:
-	    if ( cstyle == QFrame::Plain )
-		qDrawPlainRect( p, x, y, w, h, cg.foreground(), lwidth );
-	    else
-		qDrawShadePanel( p, x, y, w, h, cg, cstyle == QFrame::Sunken,
-				 lwidth );
-	    break;
-
-	case QFrame::WinPanel:
-	    if ( cstyle == QFrame::Plain )
-		qDrawPlainRect( p, x, y, w, h, cg.foreground(), 2 );
-	    else
-		qDrawWinPanel( p, x, y, w, h, cg, cstyle == QFrame::Sunken );
-	    break;
-	default:
-	    break;
-	}
+	int lwidth = opt.lineWidth(), mlwidth = opt.midLineWidth();
+	if ( flags & (Style_Sunken|Style_Raised))
+	    qDrawShadeRect( p, r.x(), r.y(), r.width(), r.height(), cg, flags & Style_Sunken, lwidth, mlwidth );
+	else
+	    qDrawPlainRect( p, r.x(), r.y(), r.width(), r.height(), cg.foreground(), lwidth );
 #endif
 	break; }
 
@@ -517,6 +604,14 @@ void QCommonStyle::drawControl( ControlElement element,
     activePainter = p;
 
     switch (element) {
+    case CE_MenuBarEmptyArea: {
+	QRegion reg;
+	if(p->hasClipping()) //ick
+	    reg = p->clipRegion();
+	else
+	    reg = r;
+	((QWidget *)widget)->erase( reg );
+	break; }
     case CE_PushButton:
 	{
 #ifndef QT_NO_PUSHBUTTON
@@ -702,6 +797,11 @@ void QCommonStyle::drawControl( ControlElement element,
     case CE_ProgressBarContents:
 	{
 	    const QProgressBar *progressbar = (const QProgressBar *) widget;
+	    // Correct the highlight color if same as background,
+	    // or else we cannot see the progress...
+	    QColorGroup cgh = cg;
+	    if ( cgh.highlight() == cgh.background() )
+		cgh.setColor( QColorGroup::Highlight, progressbar->palette().active().highlight() );
 	    bool reverse = QApplication::reverseLayout();
 	    int fw = 2;
 	    int w = r.width() - 2*fw;
@@ -711,7 +811,7 @@ void QCommonStyle::drawControl( ControlElement element,
 		if (x > w)
 		    x = 2 * w - x;
 		x = reverse ? r.right() - x : x + r.x();
-		p->setPen( QPen(cg.highlight(), 4) );
+		p->setPen( QPen(cgh.highlight(), 4) );
 		p->drawLine(x, r.y() + 1, x, r.height() - fw);
 	    } else {
 		const int unit_width = pixelMetric(PM_ProgressBarChunkWidth, widget);
@@ -746,7 +846,7 @@ void QCommonStyle::drawControl( ControlElement element,
 		for (int i=0; i<nu; i++) {
 		    drawPrimitive( PE_ProgressBarChunk, p,
 				   QRect( x0+x, r.y(), unit_width, r.height() ),
-				   cg, Style_Default, opt );
+				   cgh, Style_Default, opt );
 		    x += reverse ? -unit_width: unit_width;
 		}
 
@@ -757,7 +857,7 @@ void QCommonStyle::drawControl( ControlElement element,
 		    int offset = reverse ? x0+x+unit_width-pixels_left : x0+x;
 		    drawPrimitive( PE_ProgressBarChunk, p,
 				   QRect( offset, r.y(), pixels_left,
-					  r.height() ), cg, Style_Default,
+					  r.height() ), cgh, Style_Default,
 				   opt );
 		}
 	    }
@@ -840,16 +940,28 @@ void QCommonStyle::drawControl( ControlElement element,
 		    pm = toolbutton->iconSet().pixmap( size, mode, state );
 
 		    if ( toolbutton->usesTextLabel() ) {
-			p->setFont( toolbutton->font() );
+			if ( toolbutton->textPosition() == QToolButton::Under ) {
+			    p->setFont( toolbutton->font() );
 
-			QRect pr = rect, tr = rect;
-			int fh = p->fontMetrics().height();
-			pr.addCoords( 0, 1, 0, -fh-3 );
-			tr.addCoords( 0, pr.bottom(), 0, -3 );
-			drawItem( p, pr, AlignCenter, cg, TRUE, &pm, QString::null );
-			drawItem( p, tr, AlignCenter | ShowPrefix, cg,
-				  flags & Style_Enabled, 0, toolbutton->textLabel(),
-				  toolbutton->textLabel().length(), &btext);
+			    QRect pr = rect, tr = rect;
+			    int fh = p->fontMetrics().height();
+			    pr.addCoords( 0, 1, 0, -fh-3 );
+			    tr.addCoords( 0, pr.bottom(), 0, -3 );
+			    drawItem( p, pr, AlignCenter, cg, TRUE, &pm, QString::null );
+			    drawItem( p, tr, AlignCenter | ShowPrefix, cg,
+				      flags & Style_Enabled, 0, toolbutton->textLabel(),
+				      toolbutton->textLabel().length(), &btext);
+			} else {
+			    p->setFont( toolbutton->font() );
+
+			    QRect pr = rect, tr = rect;
+			    pr.setWidth( pm.width() + 8 );
+			    tr.addCoords( pr.right(), 0, 0, 0 );
+			    drawItem( p, pr, AlignCenter, cg, TRUE, &pm, QString::null );
+			    drawItem( p, tr, AlignLeft | AlignVCenter | ShowPrefix, cg,
+				      flags & Style_Enabled, 0, toolbutton->textLabel(),
+				      toolbutton->textLabel().length(), &btext);
+			}
 		    } else
 			drawItem( p, rect, AlignCenter, cg, TRUE, &pm, QString::null );
 		}
@@ -858,7 +970,6 @@ void QCommonStyle::drawControl( ControlElement element,
 	    break;
 	}
 #endif // QT_NO_TOOLBUTTON
-
     default:
 	break;
     }
@@ -955,9 +1066,9 @@ QRect QCommonStyle::subRect(SubRect r, const QWidget *widget) const
 
     case SR_CheckBoxIndicator:
 	{
-	    int h = pixelMetric( PM_IndicatorHeight );
+	    int h = pixelMetric( PM_IndicatorHeight, widget );
 	    rect.setRect(0, ( wrect.height() - h ) / 2,
-			 pixelMetric( PM_IndicatorWidth ), h );
+			 pixelMetric( PM_IndicatorWidth, widget ), h );
 	    break;
 	}
 
@@ -965,8 +1076,8 @@ QRect QCommonStyle::subRect(SubRect r, const QWidget *widget) const
 	{
 #ifndef QT_NO_CHECKBOX
 	    QRect ir = subRect(SR_CheckBoxIndicator, widget);
-	    rect.setRect(ir.right() + 10, wrect.y(),
-			 wrect.width() - ir.width() - 10, wrect.height());
+	    rect.setRect(ir.right() + 6, wrect.y(),
+			 wrect.width() - ir.width() - 6, wrect.height());
 #endif
 	    break;
 	}
@@ -975,6 +1086,11 @@ QRect QCommonStyle::subRect(SubRect r, const QWidget *widget) const
 	{
 #ifndef QT_NO_CHECKBOX
 	    const QCheckBox *checkbox = (const QCheckBox *) widget;
+	    if ( !checkbox->pixmap() && checkbox->text().isEmpty() ) {
+		rect = subRect( SR_CheckBoxIndicator, widget );
+		rect.addCoords( 1, 1, -1, -1 );
+		break;
+	    }
 	    QRect cr = subRect(SR_CheckBoxContents, widget);
 
 	    // don't create a painter if we have an active one
@@ -997,17 +1113,17 @@ QRect QCommonStyle::subRect(SubRect r, const QWidget *widget) const
 
     case SR_RadioButtonIndicator:
 	{
-	    int h = pixelMetric( PM_ExclusiveIndicatorHeight );
+	    int h = pixelMetric( PM_ExclusiveIndicatorHeight, widget );
 	    rect.setRect(0, ( wrect.height() - h ) / 2,
-			 pixelMetric( PM_ExclusiveIndicatorWidth ), h );
+			 pixelMetric( PM_ExclusiveIndicatorWidth, widget ), h );
 	    break;
 	}
 
     case SR_RadioButtonContents:
 	{
 	    QRect ir = subRect(SR_RadioButtonIndicator, widget);
-	    rect.setRect(ir.right() + 10, wrect.y(),
-			 wrect.width() - ir.width() - 10, wrect.height());
+	    rect.setRect(ir.right() + 6, wrect.y(),
+			 wrect.width() - ir.width() - 6, wrect.height());
 	    break;
 	}
 
@@ -1015,6 +1131,11 @@ QRect QCommonStyle::subRect(SubRect r, const QWidget *widget) const
 	{
 #ifndef QT_NO_RADIOBUTTON
 	    const QRadioButton *radiobutton = (const QRadioButton *) widget;
+	    if ( !radiobutton->pixmap() && radiobutton->text().isEmpty() ) {
+		rect = subRect( SR_RadioButtonIndicator, widget );
+		rect.addCoords( 1, 1, -1, -1 );
+		break;
+	    }
 	    QRect cr = subRect(SR_RadioButtonContents, widget);
 
 	    // don't create a painter if we have an active one
@@ -1340,7 +1461,7 @@ void QCommonStyle::drawComplexControl( ComplexControl control,
 		QRect ir = visualRect( querySubControlMetrics( CC_TitleBar, widget, SC_TitleBarLabel ), widget );
 
 		p->setPen( cgroup.highlightedText() );
-		p->drawText(ir.x()+2, ir.y(), ir.width(), ir.height(),
+		p->drawText(ir.x()+2, ir.y(), ir.width()-2, ir.height(),
 			    AlignAuto | AlignVCenter | SingleLine, titlebar->visibleText() );
 	    }
 
@@ -1360,7 +1481,8 @@ void QCommonStyle::drawComplexControl( ComplexControl control,
 
 		p->save();
 		if( down )
-		    p->translate( pixelMetric(PM_ButtonShiftHorizontal), pixelMetric(PM_ButtonShiftVertical) );
+		    p->translate( pixelMetric(PM_ButtonShiftHorizontal, widget),
+				  pixelMetric(PM_ButtonShiftVertical, widget) );
 		drawItem( p, ir, AlignCenter, titlebar->colorGroup(), TRUE, &pm, QString::null );
 		p->restore();
 	    }
@@ -1376,7 +1498,8 @@ void QCommonStyle::drawComplexControl( ComplexControl control,
 
 		    p->save();
 		    if( down )
-			p->translate( pixelMetric(PM_ButtonShiftHorizontal), pixelMetric(PM_ButtonShiftVertical) );
+			p->translate( pixelMetric(PM_ButtonShiftHorizontal, widget),
+				      pixelMetric(PM_ButtonShiftVertical, widget) );
     		    drawItem( p, ir, AlignCenter, titlebar->colorGroup(), TRUE, &pm, QString::null );
 		    p->restore();
 		}
@@ -1396,7 +1519,8 @@ void QCommonStyle::drawComplexControl( ComplexControl control,
 
 		    p->save();
 		    if( down )
-			p->translate( pixelMetric(PM_ButtonShiftHorizontal), pixelMetric(PM_ButtonShiftVertical) );
+			p->translate( pixelMetric(PM_ButtonShiftHorizontal, widget),
+				      pixelMetric(PM_ButtonShiftVertical, widget) );
 		    drawItem( p, ir, AlignCenter, titlebar->colorGroup(), TRUE, &pm, QString::null );
 		    p->restore();
 		}
@@ -1410,7 +1534,8 @@ void QCommonStyle::drawComplexControl( ComplexControl control,
 				  down ? Style_Down : Style_Raised);
 		    p->save();
 		    if( down )
-			p->translate( pixelMetric(PM_ButtonShiftHorizontal), pixelMetric(PM_ButtonShiftVertical) );
+			p->translate( pixelMetric(PM_ButtonShiftHorizontal, widget),
+				      pixelMetric(PM_ButtonShiftVertical, widget) );
 		    drawItem( p, ir, AlignCenter, titlebar->colorGroup(), TRUE, &pm, QString::null );
 		    p->restore();
 		}
@@ -1424,7 +1549,8 @@ void QCommonStyle::drawComplexControl( ComplexControl control,
 				  down ? Style_Down : Style_Raised);
 		    p->save();
 		    if( down )
-			p->translate( pixelMetric(PM_ButtonShiftHorizontal), pixelMetric(PM_ButtonShiftVertical) );
+			p->translate( pixelMetric(PM_ButtonShiftHorizontal, widget),
+				      pixelMetric(PM_ButtonShiftVertical, widget) );
 		    drawItem( p, ir, AlignCenter, titlebar->colorGroup(), TRUE, &pm, QString::null );
 		    p->restore();
 		}
@@ -1507,7 +1633,7 @@ void QCommonStyle::drawComplexControl( ComplexControl control,
 		    interval = sl->pageStep();
 	    }
 
-	    int fudge = len / 2 + 1;
+	    int fudge = len / 2;
 	    int pos;
 
 	    if ( ticks & QSlider::Above ) {
@@ -1595,19 +1721,15 @@ void QCommonStyle::drawComplexControlMask( ComplexControl control,
 QRect QCommonStyle::querySubControlMetrics( ComplexControl control,
 					    const QWidget *widget,
 					    SubControl sc,
-					    const QStyleOption&  ) const
+					    const QStyleOption &opt ) const
 {
-
-    QRect rect;
-
 #if defined(QT_CHECK_STATE)
     if (! widget) {
 	qWarning("QCommonStyle::querySubControlMetrics: widget parameter cannot be zero!");
-	return rect;
+	return QRect();
     }
 #endif
 
-
     switch ( control ) {
     case CC_SpinWidget: {
 	int fw = pixelMetric( PM_SpinBoxFrameWidth, widget);
@@ -1624,23 +1746,18 @@ QRect QCommonStyle::querySubControlMetrics( ComplexControl control,
 	rx = x - fw;
 	switch ( sc ) {
 	case SC_SpinWidgetUp:
-	    rect.setRect(x, y, bs.width(), bs.height());
-	    break;
+	    return QRect(x, y, bs.width(), bs.height());
 	case SC_SpinWidgetDown:
-	    rect.setRect(x, y + bs.height(), bs.width(), bs.height());
-	    break;
+	    return QRect(x, y + bs.height(), bs.width(), bs.height());
 	case SC_SpinWidgetButtonField:
-	    rect.setRect(x, y, bs.width(), widget->height() - 2*fw);
-	    break;
+	    return QRect(x, y, bs.width(), widget->height() - 2*fw);
 	case SC_SpinWidgetEditField:
-	    rect.setRect(lx, fw, rx, widget->height() - 2*fw);
-	    break;
+	    return QRect(lx, fw, rx, widget->height() - 2*fw);
 	case SC_SpinWidgetFrame:
-	    rect = widget->rect();
+	    return widget->rect();
 	default:
 	    break;
 	}
-
 	break; }
 
     case CC_ComboBox: {
@@ -1650,18 +1767,16 @@ QRect QCommonStyle::querySubControlMetrics( ComplexControl control,
 
 	switch ( sc ) {
 	case SC_ComboBoxFrame:
-	    rect = widget->rect();
-	    break;
+	    return widget->rect();
 	case SC_ComboBoxArrow:
-	    rect.setRect(xpos, y+2, 16, he-4);
-	    break;
+	    return QRect(xpos, y+2, 16, he-4);
 	case SC_ComboBoxEditField:
-	    rect.setRect(x+3, y+3, wi-6-16, he-6);
-	    break;
+	    return QRect(x+3, y+3, wi-6-16, he-6);
+	case SC_ComboBoxListBoxPopup:
+	    return opt.rect();
 	default:
 	    break;
 	}
-
 	break; }
 
 #ifndef QT_NO_SCROLLBAR
@@ -1690,193 +1805,161 @@ QRect QCommonStyle::querySubControlMetrics( ComplexControl control,
 	    sliderlen = maxlen;
 
 	switch (sc) {
-	case SC_ScrollBarSubLine:
-	    // top/left button
-	    rect.setRect(0, 0, sbextent, sbextent);
-	    break;
+	case SC_ScrollBarSubLine:	    // top/left button
+	    if (scrollbar->orientation() == Qt::Horizontal) {
+		int buttonWidth = QMIN(scrollbar->width()/2, sbextent);
+		return QRect( 0, 0, buttonWidth, sbextent );
+	    } else {
+		int buttonHeight = QMIN(scrollbar->height()/2, sbextent);
+		return QRect( 0, 0, sbextent, buttonHeight );
+	    }
 
-	case SC_ScrollBarAddLine:
-	    // bottom/right button
-	    if (scrollbar->orientation() == Qt::Horizontal)
-		rect.setRect(scrollbar->width() - sbextent, 0, sbextent, sbextent);
-	    else
-		rect.setRect(0, scrollbar->height() - sbextent, sbextent, sbextent);
-	    break;
+	case SC_ScrollBarAddLine:	    // bottom/right button
+	    if (scrollbar->orientation() == Qt::Horizontal) {
+		int buttonWidth = QMIN(scrollbar->width()/2, sbextent);
+		return QRect( scrollbar->width() - buttonWidth, 0, buttonWidth, sbextent );
+	    } else {
+		int buttonHeight = QMIN(scrollbar->height()/2, sbextent);
+		return QRect( 0, scrollbar->height() - buttonHeight, sbextent, buttonHeight );
+	    }
 
-	case SC_ScrollBarSubPage:
-	    // between top/left button and slider
+	case SC_ScrollBarSubPage:	    // between top/left button and slider
 	    if (scrollbar->orientation() == Qt::Horizontal)
-		rect.setRect(sbextent, 0, sliderstart - sbextent, sbextent);
-	    else
-		rect.setRect(0, sbextent, sbextent, sliderstart - sbextent);
-	    break;
+		return QRect(sbextent, 0, sliderstart - sbextent, sbextent);
+	    return QRect(0, sbextent, sbextent, sliderstart - sbextent);
 
-	case SC_ScrollBarAddPage:
-	    // between bottom/right button and slider
+	case SC_ScrollBarAddPage:	    // between bottom/right button and slider
 	    if (scrollbar->orientation() == Qt::Horizontal)
-		rect.setRect(sliderstart + sliderlen, 0,
+		return QRect(sliderstart + sliderlen, 0,
 			     maxlen - sliderstart - sliderlen + sbextent, sbextent);
-	    else
-		rect.setRect(0, sliderstart + sliderlen,
-			     sbextent, maxlen - sliderstart - sliderlen + sbextent);
-	    break;
+	    return QRect(0, sliderstart + sliderlen,
+			 sbextent, maxlen - sliderstart - sliderlen + sbextent);
 
 	case SC_ScrollBarGroove:
 	    if (scrollbar->orientation() == Qt::Horizontal)
-		rect.setRect(sbextent, 0, scrollbar->width() - sbextent * 2,
+		return QRect(sbextent, 0, scrollbar->width() - sbextent * 2,
 			     scrollbar->height());
-	    else
-		rect.setRect(0, sbextent, scrollbar->width(),
-			     scrollbar->height() - sbextent * 2);
-	    break;
+	    return QRect(0, sbextent, scrollbar->width(),
+			 scrollbar->height() - sbextent * 2);
 
 	case SC_ScrollBarSlider:
 	    if (scrollbar->orientation() == Qt::Horizontal)
-		rect.setRect(sliderstart, 0, sliderlen, sbextent);
-	    else
-		rect.setRect(0, sliderstart, sbextent, sliderlen);
+		return QRect(sliderstart, 0, sliderlen, sbextent);
+	    return QRect(0, sliderstart, sbextent, sliderlen);
 
-	    break;
-
-	default:
-	    break;
+	default: break;
 	}
 
 	break; }
 #endif // QT_NO_SCROLLBAR
 
 #ifndef QT_NO_SLIDER
-    case CC_Slider:
-	{
+    case CC_Slider: {
 	    const QSlider * sl = (const QSlider *) widget;
 	    int tickOffset = pixelMetric( PM_SliderTickmarkOffset, sl );
 	    int thickness = pixelMetric( PM_SliderControlThickness, sl );
 
 	    switch ( sc ) {
-	    case SC_SliderHandle:
-		{
+	    case SC_SliderHandle: {
 		    int sliderPos = 0;
 		    int len   = pixelMetric( PM_SliderLength, sl );
 
 		    sliderPos = sl->sliderStart();
 
 		    if ( sl->orientation() == Horizontal )
-			rect.setRect( sliderPos, tickOffset, len, thickness );
-		    else
-			rect.setRect( tickOffset, sliderPos, thickness, len );
-		    break;
-		}
-
-	    case SC_SliderGroove:
-		{
-		    if ( sl->orientation() == Horizontal )
-			rect.setRect( 0, tickOffset, sl->width(), thickness );
-		    else
-			rect.setRect( tickOffset, 0, thickness, sl->height() );
-		    break;
-		}
+			return QRect( sliderPos, tickOffset, len, thickness );
+		    return QRect( tickOffset, sliderPos, thickness, len ); }
+	    case SC_SliderGroove: {
+		if ( sl->orientation() == Horizontal )
+		    return QRect( 0, tickOffset, sl->width(), thickness );
+		return QRect( tickOffset, 0, thickness, sl->height() );	}
 
 	    default:
 		break;
 	    }
-
-	    break;
-	}
+	    break; }
 #endif // QT_NO_SLIDER
 
 #if !defined(QT_NO_TOOLBUTTON) && !defined(QT_NO_POPUPMENU)
-    case CC_ToolButton:
-	{
+    case CC_ToolButton: {
 	    const QToolButton *toolbutton = (const QToolButton *) widget;
 	    int mbi = pixelMetric(PM_MenuButtonIndicator, widget);
 
-	    rect = toolbutton->rect();
-
+	    QRect rect = toolbutton->rect();
 	    switch (sc) {
 	    case SC_ToolButton:
 		if (toolbutton->popup() && ! toolbutton->popupDelay())
 		    rect.addCoords(0, 0, -mbi, 0);
-		break;
+		return rect;
 
 	    case SC_ToolButtonMenu:
 		if (toolbutton->popup() && ! toolbutton->popupDelay())
 		    rect.addCoords(rect.width() - mbi, 0, 0, 0);
-		break;
+		return rect;
 
-	    default:
-		break;
+	    default: break;
 	    }
 	    break;
 	}
 #endif // QT_NO_TOOLBUTTON && QT_NO_POPUPMENU
 
 #ifndef QT_NO_TITLEBAR
-    case CC_TitleBar:
-	{
+    case CC_TitleBar: {
 	    const QTitleBar *titlebar = (const QTitleBar *) widget;
 	    const int controlTop = 2;
 	    const int controlHeight = widget->height() - controlTop * 2;
 
 	    switch (sc) {
-	    case SC_TitleBarLabel:
-		{
-		    const QTitleBar *titlebar = (QTitleBar*)widget;
-		    QRect ir( 0, 0, titlebar->width(), titlebar->height() );
-		    if ( titlebar->testWFlags( WStyle_Tool ) ) {
-			if ( titlebar->testWFlags( WStyle_SysMenu ) )
-			    ir.addCoords( 0, 0, -controlHeight-3, 0 );
-			if ( titlebar->testWFlags( WStyle_MinMax ) )
-			    ir.addCoords( 0, 0, -controlHeight-2, 0 );
-		    } else {
-			if ( titlebar->testWFlags( WStyle_SysMenu ) )
-			    ir.addCoords( controlHeight+3, 0, -controlHeight-3, 0 );
-			if ( titlebar->testWFlags( WStyle_Minimize ) )
-			    ir.addCoords( 0, 0, -controlHeight-2, 0 );
-			if ( titlebar->testWFlags( WStyle_Maximize ) )
-			    ir.addCoords( 0, 0, -controlHeight-2, 0 );
-		    }
-		    rect = ir;
+	    case SC_TitleBarLabel: {
+		const QTitleBar *titlebar = (QTitleBar*)widget;
+		QRect ir( 0, 0, titlebar->width(), titlebar->height() );
+		if ( titlebar->testWFlags( WStyle_Tool ) ) {
+		    if ( titlebar->testWFlags( WStyle_SysMenu ) )
+			ir.addCoords( 0, 0, -controlHeight-3, 0 );
+		    if ( titlebar->testWFlags( WStyle_MinMax ) )
+			ir.addCoords( 0, 0, -controlHeight-2, 0 );
+		} else {
+		    if ( titlebar->testWFlags( WStyle_SysMenu ) )
+			ir.addCoords( controlHeight+3, 0, -controlHeight-3, 0 );
+		    if ( titlebar->testWFlags( WStyle_Minimize ) )
+			ir.addCoords( 0, 0, -controlHeight-2, 0 );
+		    if ( titlebar->testWFlags( WStyle_Maximize ) )
+			ir.addCoords( 0, 0, -controlHeight-2, 0 );
 		}
-		break;
+		return ir; }
 
 	    case SC_TitleBarCloseButton:
-		rect.setRect( titlebar->width() - ( controlHeight + controlTop ), controlTop, controlHeight, controlHeight );
-		break;
+		return QRect( titlebar->width() - ( controlHeight + controlTop ),
+			      controlTop, controlHeight, controlHeight );
 
 	    case SC_TitleBarMaxButton:
 	    case SC_TitleBarShadeButton:
 	    case SC_TitleBarUnshadeButton:
-		rect.setRect( titlebar->width() - ( ( controlHeight + controlTop ) * 2 ), controlTop, controlHeight, controlHeight );
-		break;
+		return QRect( titlebar->width() - ( ( controlHeight + controlTop ) * 2 ),
+			      controlTop, controlHeight, controlHeight );
 
 	    case SC_TitleBarMinButton:
-	    case SC_TitleBarNormalButton:
-		{
-		    int offset = controlHeight + controlTop;
-		    if ( !titlebar->testWFlags( WStyle_Maximize ) )
-			offset *= 2;
-		    else
-			offset *= 3;
-		    rect.setRect( titlebar->width() - offset, controlTop, controlHeight, controlHeight );
-		}
-		break;
+	    case SC_TitleBarNormalButton: {
+		int offset = controlHeight + controlTop;
+		if ( !titlebar->testWFlags( WStyle_Maximize ) )
+		    offset *= 2;
+		else
+		    offset *= 3;
+		return QRect( titlebar->width() - offset, controlTop, controlHeight, controlHeight );
+	    }
 
 	    case SC_TitleBarSysMenu:
-		rect.setRect( 3, controlTop, controlHeight, controlHeight);
-		break;
+		return QRect( 3, controlTop, controlHeight, controlHeight);
 
-	    default:
-		break;
+	    default: break;
 	    }
-	    break;
-	}
+	    break; }
 #endif //QT_NO_TITLEBAR
 
     default:
 	break;
     }
-
-    return rect;
+    return QRect();
 }
 
 
@@ -1961,6 +2044,9 @@ int QCommonStyle::pixelMetric(PixelMetric m, const QWidget *widget) const
     int ret;
 
     switch (m) {
+    case PM_CheckListButtonSize:
+	ret = 16;
+	break;
     case PM_TitleBarHeight: {
 	if ( widget && widget->testWFlags( WStyle_Tool ) ) {
 	    ret = QMAX( widget->fontMetrics().lineSpacing(), 16 );
@@ -2173,7 +2259,8 @@ QSize QCommonStyle::sizeFromContents(ContentsType contents,
 	    const QCheckBox *checkbox = (const QCheckBox *) widget;
 	    QRect irect = subRect(SR_CheckBoxIndicator, widget);
 	    int h = pixelMetric( PM_IndicatorHeight, widget );
-	    sz += QSize(irect.right() + (checkbox->text().isEmpty() ? 0 : 10), 4 );
+	    int margins = (!checkbox->pixmap() && checkbox->text().isEmpty()) ? 0 : 10;
+	    sz += QSize(irect.right() + margins, 4 );
 	    sz.setHeight( QMAX( sz.height(), h ) );
 #endif
 	    break;
@@ -2185,7 +2272,8 @@ QSize QCommonStyle::sizeFromContents(ContentsType contents,
 	    const QRadioButton *radiobutton = (const QRadioButton *) widget;
 	    QRect irect = subRect(SR_RadioButtonIndicator, widget);
 	    int h = pixelMetric( PM_ExclusiveIndicatorHeight, widget );
-	    sz += QSize(irect.right() + (radiobutton->text().isEmpty() ? 0 : 10), 4 );
+	    int margins = (!radiobutton->pixmap() && radiobutton->text().isEmpty()) ? 0 : 10;
+	    sz += QSize(irect.right() + margins, 4 );
 	    sz.setHeight( QMAX( sz.height(), h ) );
 #endif
 	    break;
@@ -2254,6 +2342,9 @@ QSize QCommonStyle::sizeFromContents(ContentsType contents,
 	    break;
 	}
 
+    case CT_LineEdit:
+    case CT_Header:
+    case CT_Slider:
     case CT_ProgressBar:
 	// just return the contentsSize for now
 	// fall through intended
@@ -2267,11 +2358,19 @@ QSize QCommonStyle::sizeFromContents(ContentsType contents,
 
 
 /*! \reimp */
-int QCommonStyle::styleHint(StyleHint sh, const QWidget *, const QStyleOption &, QStyleHintReturn *) const
+int QCommonStyle::styleHint(StyleHint sh, const QWidget * w, const QStyleOption &, QStyleHintReturn *) const
 {
     int ret;
 
     switch (sh) {
+    case SH_GroupBox_TextLabelVerticalAlignment:
+	ret = Qt::AlignVCenter;
+	break;
+
+    case SH_GroupBox_TextLabelColor:
+	ret = (int) ( w ? w->paletteForegroundColor().rgb() : 0 );
+	break;
+
     case SH_TabBar_SelectMouseType:
 	ret = QEvent::MouseButtonPress;
 	break;
@@ -2309,7 +2408,6 @@ int QCommonStyle::styleHint(StyleHint sh, const QWidget *, const QStyleOption &,
     return ret;
 }
 
-
 /*! \reimp */
 QPixmap QCommonStyle::stylePixmap(StylePixmap, const QWidget *, const QStyleOption&) const
 {
