@@ -1,9 +1,9 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qlibrary_p.h   3.1.2   edited Nov 7 16:55 $
 **
 ** Definition of an internal QLibrary class
 **
-** Created : 2000-01-01
+** Created : 000101
 **
 ** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
 **
@@ -52,6 +52,8 @@
 
 #include "qlibrary.h"
 
+#ifndef QT_NO_LIBRARY
+
 #ifndef QT_H
 #include "qwindowdefs.h"
 #endif // QT_H
@@ -75,4 +77,5 @@ private:
     QLibrary *library;
 };
 
+#endif // QT_NO_LIBRARY
 #endif // QLIBRARY_P_H
