@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qgrid.cpp   3.0.5   edited Nov 23 2001 $
+** $Id: qt/qgrid.cpp   3.1.2   edited Nov 7 16:55 $
 **
 ** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
 **
@@ -38,35 +38,37 @@
 #include "qapplication.h"
 
 /*!
-  \class QGrid qgrid.h
-  \brief The QGrid widget provides simple geometry management of its children.
+    \class QGrid qgrid.h
+    \brief The QGrid widget provides simple geometry management of its children.
 
-  \ingroup geomanagement
-  \ingroup appearance
+    \ingroup geomanagement
+    \ingroup appearance
 
-    The grid places its widgets either in a single column or in a single
-    row. If you want a multi-column, multi-row grid use \l{QGridLayout}.
+    The grid places its widgets either in a single column or in a
+    single row. If you want a multi-column, multi-row grid use
+    \l{QGridLayout}.
 
-  The number of rows \e or columns is defined in the constructor. All the
-  grid's children will be placed and sized in accordance with their sizeHint()
-  and sizePolicy().
+    The number of rows \e or columns is defined in the constructor.
+    All the grid's children will be placed and sized in accordance
+    with their sizeHint() and sizePolicy().
 
-  Use setMargin() to add space around the grid itself, and setSpacing() to
-  add space between the widgets.
+    Use setMargin() to add space around the grid itself, and
+    setSpacing() to add space between the widgets.
 
-  <img src=qgrid-m.png>
+    <img src=qgrid-m.png>
 
-  \sa QVBox QHBox QGridLayout
+    \sa QVBox QHBox QGridLayout
 */
+
 /*! \enum QGrid::Direction
     \internal
 */
 
 /*!
-  Constructs a grid widget with parent \a parent and name \a name. If
-  \a orient is \c Horizontal, \a n specifies the number of columns.
-  If \a orient is \c Vertical, \a n specifies the number of rows.
-  The widget flags \a f are passed to the QFrame constructor.
+    Constructs a grid widget with parent \a parent, called \a name.
+    If \a orient is \c Horizontal, \a n specifies the number of
+    columns. If \a orient is \c Vertical, \a n specifies the number of
+    rows. The widget flags \a f are passed to the QFrame constructor.
 */
 QGrid::QGrid( int n, Orientation orient, QWidget *parent, const char *name,
 	      WFlags f )
@@ -87,9 +89,9 @@ QGrid::QGrid( int n, Orientation orient, QWidget *parent, const char *name,
 
 
 /*!
-  Constructs a grid widget with parent \a parent and name \a name.
-  \a n specifies the number of columns.
-  The widget flags \a f are passed to the QFrame constructor.
+    Constructs a grid widget with parent \a parent, called \a name.
+    \a n specifies the number of columns. The widget flags \a f are
+    passed to the QFrame constructor.
  */
 QGrid::QGrid( int n, QWidget *parent, const char *name, WFlags f )
     : QFrame( parent, name, f )
@@ -100,7 +102,7 @@ QGrid::QGrid( int n, QWidget *parent, const char *name, WFlags f )
 
 
 /*!
-  Sets the spacing between the child widgets to \a space.
+    Sets the spacing between the child widgets to \a space.
 */
 
 void QGrid::setSpacing( int space )
