@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qstatusbar.cpp   3.0.5   edited May 15 03:37 $
+** $Id: qt/qstatusbar.cpp   3.1.2   edited Feb 19 16:08 $
 **
 ** Implementation of QStatusBar class
 **
@@ -47,61 +47,61 @@
 #include "qsizegrip.h"
 
 /*!
-  \class QStatusBar qstatusbar.h
+    \class QStatusBar qstatusbar.h
+    \brief The QStatusBar class provides a horizontal bar suitable for
+    presenting status information.
 
-  \brief The QStatusBar class provides a horizontal bar suitable for
-  presenting status information.
+    \ingroup application
+    \ingroup helpsystem
+    \mainclass
 
-  \ingroup application
-  \ingroup helpsystem
-  \mainclass
+    Each status indicator falls into one of three categories:
 
-  Each status indicator falls into one of three categories:
+    \list
+    \i \e Temporary - briefly occupies most of the status bar. Used
+	to explain tool tip texts or menu entries, for example.
+    \i \e Normal - occupies part of the status bar and may be hidden
+	by temporary messages. Used to display the page and line
+	number in a word processor, for example.
+    \i \e Permanent - is never hidden. Used for important mode
+	indications, for example, some applications put a Caps Lock
+	indicator in the status bar.
+    \endlist
 
-  \list
-  \i \e Temporary - briefly occupies most of the status bar.  Used
-    to explain tool tip texts or menu entries, for example.
-  \i \e Normal - occupies part of the status bar and may be hidden
-    by temporary messages.  Used to display the page and line
-    number in a word processor, for example.
-  \i \e Permanent - is never hidden.  Used for important mode
-    indications, for example, some applications put a Caps Lock indicator in the
-    status bar.
-  \endlist
+    QStatusBar lets you display all three types of indicators.
 
-  QStatusBar lets you display all three types of indicators.
+    To display a \e temporary message, call message() (perhaps by
+    connecting a suitable signal to it). To remove a temporary
+    message, call clear(). There are two variants of message(): one
+    that displays the message until the next clear() or message() and
+    one that has a time limit:
 
-  To display a \e temporary message, call message() (perhaps by
-  connecting a suitable signal to it).  To remove a temporary message,
-  call clear().
-  There are two variants of message(): one that displays the message
-  until the next clear() or mesage() and one that has a time limit:
+    \code
+	connect( loader, SIGNAL(progressMessage(const QString&)),
+		 statusBar(), SLOT(message(const QString&)) );
 
-  \code
-     connect( loader, SIGNAL(progressMessage(const QString&)),
-	      statusBar(), SLOT(message(const QString&)) );
+	statusBar()->message("Loading...");  // Initial message
+	loader.loadStuff();                  // Emits progress messages
+	statusBar()->message("Done.", 2000); // Final message for 2 seconds
+    \endcode
 
-     statusBar()->message("Loading...");  // Initial message
-     loader.loadStuff();                  // Emits progress messages
-     statusBar()->message("Done.", 2000); // Final message for 2 seconds
-  \endcode
+    \e Normal and \e Permanent messages are displayed by creating a
+    small widget and then adding it to the status bar with
+    addWidget(). Widgets like QLabel, QProgressBar or even QToolButton
+    are useful for adding to status bars. removeWidget() is used to
+    remove widgets.
 
-  \e Normal and \e permanent messages are displayed by creating a small
-  widget and then adding it to the status bar with addWidget().  Widgets
-  like QLabel, QProgressBar or even QToolButton are useful for adding
-  to status bars.  removeWidget() is used to remove widgets.
+    \code
+	statusBar()->addWidget(new MyReadWriteIndication(statusBar()));
+    \endcode
 
-  \code
-     statusBar()->addWidget(new MyReadWriteIndication(statusBar()));
-  \endcode
+    By default QStatusBar provides a QSizeGrip in the lower-right
+    corner. You can disable it with setSizeGripEnabled(FALSE);
 
-  By default QStatusBar provides a QSizeGrip in the lower-right corner.
-  You can disable it with setSizeGripEnabled(FALSE);
+    <img src=qstatusbar-m.png> <img src=qstatusbar-w.png>
 
-  <img src=qstatusbar-m.png> <img src=qstatusbar-w.png>
-
-  \sa QToolBar QMainWindow QLabel
-  \link guibooks.html#fowler GUI Design Handbook: Status Bar.\endlink
+    \sa QToolBar QMainWindow QLabel
+    \link guibooks.html#fowler GUI Design Handbook: Status Bar.\endlink
 */
 
 
@@ -127,14 +127,16 @@ public:
 #ifndef QT_NO_SIZEGRIP
     QSizeGrip * resizer;
 #endif
+
+    int savedStrut;
 };
 
 
 /*!
-  Constructs a status bar with the parent \a parent and the name
-  \a name and with a size grip.
+    Constructs a status bar called \a name with parent \a parent and
+    with a size grip.
 
-  \sa setSizeGripEnabled()
+    \sa setSizeGripEnabled()
 */
 QStatusBar::QStatusBar( QWidget * parent, const char *name )
     : QWidget( parent, name )
@@ -154,8 +156,8 @@ QStatusBar::QStatusBar( QWidget * parent, const char *name )
 
 
 /*!
-  Destroys the status bar and frees any allocated resources and child
-  widgets.
+    Destroys the status bar and frees any allocated resources and
+    child widgets.
 */
 QStatusBar::~QStatusBar()
 {
@@ -165,22 +167,24 @@ QStatusBar::~QStatusBar()
 
 
 /*!
-  Adds \a widget to this status bar.
+    Adds \a widget to this status bar. \a widget is reparented if it
+    isn't already a child of the QStatusBar.
 
-  \a widget is permanently visible if \a permanent is TRUE and may be
-  obscured by temporary messages if \a permanent is FALSE.  The
-  default is FALSE.
+    \a widget is permanently visible if \a permanent is TRUE and may
+    be obscured by temporary messages if \a permanent is FALSE. The
+    default is FALSE.
 
-  If \a permanent is TRUE, \a widget is located at the far right of
-  the status bar.  If \a permanent is FALSE (the default), \a widget is
-  located just to the left of the first permanent widget.
+    If \a permanent is TRUE, \a widget is located at the far right of
+    the status bar. If \a permanent is FALSE (the default), \a widget
+    is located just to the left of the first permanent widget.
 
-  \a stretch is used to compute a suitable size for \a widget as the
-  status bar grows and shrinks. The default of 0 uses a minimum of space.
+    \a stretch is used to compute a suitable size for \a widget as the
+    status bar grows and shrinks. The default of 0 uses a minimum of
+    space.
 
-  This function may cause some flicker.
+    This function may cause some flicker.
 
-  \sa removeWidget()
+    \sa removeWidget()
 */
 
 void QStatusBar::addWidget( QWidget * widget, int stretch, bool permanent )
@@ -192,6 +196,9 @@ void QStatusBar::addWidget( QWidget * widget, int stretch, bool permanent )
 	return;
     }
 
+    if ( widget->parentWidget() != this )
+	widget->reparent( this, QPoint(0, 0), TRUE );
+
     QStatusBarPrivate::SBItem* item
 	= new QStatusBarPrivate::SBItem( widget, stretch, permanent );
 
@@ -209,13 +216,13 @@ void QStatusBar::addWidget( QWidget * widget, int stretch, bool permanent )
 
 
 /*!
-  Removes \a widget from the status bar.
+    Removes \a widget from the status bar.
 
-  This function may cause some flicker.
+    This function may cause some flicker.
 
-  Note that \a widget is not deleted.
+    Note that \a widget is not deleted.
 
-  \sa addWidget()
+    \sa addWidget()
 */
 
 void QStatusBar::removeWidget( QWidget* widget )
@@ -241,11 +248,11 @@ void QStatusBar::removeWidget( QWidget* widget )
 }
 
 /*!
-  \property QStatusBar::sizeGripEnabled
-  \brief whether the QSizeGrip in the bottom right of the status bar is enabled
+    \property QStatusBar::sizeGripEnabled
+    \brief whether the QSizeGrip in the bottom right of the status bar is enabled
 
-  Enables or disables the QSizeGrip in the bottom right of the status bar.
-  By default, the size grip is enabled.
+    Enables or disables the QSizeGrip in the bottom right of the
+    status bar. By default, the size grip is enabled.
 */
 
 bool QStatusBar::isSizeGripEnabled() const
@@ -275,9 +282,10 @@ void QStatusBar::setSizeGripEnabled(bool enabled)
 }
 
 
-/*!  Changes the status bar's appearance to account for item
-  changes. Special subclasses may need this, but geometry management
-  will usually take care of any necessary rearrangements.
+/*!
+    Changes the status bar's appearance to account for item changes.
+    Special subclasses may need this, but geometry management will
+    usually take care of any necessary rearrangements.
 */
 void QStatusBar::reformat()
 {
@@ -325,6 +333,7 @@ void QStatusBar::reformat()
     }
 #endif
     l->addStrut( maxH );
+    d->savedStrut = maxH;
     vbox->addSpacing( 2 );
     d->box->activate();
     repaint();
@@ -334,10 +343,10 @@ void QStatusBar::reformat()
 
 
 /*!
-  Hides the normal status indicators and displays \a message until
-  clear() or another message() is called.
+    Hides the normal status indicators and displays \a message until
+    clear() or another message() is called.
 
-  \sa clear()
+    \sa clear()
 */
 void QStatusBar::message( const QString &message )
 {
@@ -353,11 +362,11 @@ void QStatusBar::message( const QString &message )
 
 
 /*!
-  \overload
+    \overload
 
-  Hides the normal status indications and displays \a message for \a
-  ms milli-seconds or until clear() or another message() is called,
-  whichever occurs first.
+    Hides the normal status indications and displays \a message for \a
+    ms milli-seconds or until clear() or another message() is called,
+    whichever occurs first.
 */
 void QStatusBar::message( const QString &message, int ms )
 {
@@ -367,16 +376,21 @@ void QStatusBar::message( const QString &message, int ms )
 	d->timer = new QTimer( this );
 	connect( d->timer, SIGNAL(timeout()), this, SLOT(clear()) );
     }
-    if ( ms > 0 )
+    if ( ms > 0 ) {
 	d->timer->start( ms );
+    } else if ( d->timer ) {
+	delete d->timer;
+	d->timer = 0;
+    }
+
     hideOrShow();
 }
 
 
 /*!
-  Removes any temporary message being shown.
+    Removes any temporary message being shown.
 
-  \sa message()
+    \sa message()
 */
 
 void QStatusBar::clear()
@@ -391,10 +405,19 @@ void QStatusBar::clear()
     hideOrShow();
 }
 
+/*!
+    \fn QStatusBar::messageChanged( const QString &message )
+
+    This signal is emitted when the temporary status messages
+    changes. \a message is the new temporary message, and is a
+    null-string when the message has been removed.
+
+    \sa message(), clear()
+*/
 
 /*!
-  Ensures that the right widgets are visible.  Used by message()
-  and clear().
+    Ensures that the right widgets are visible. Used by message() and
+    clear().
 */
 void QStatusBar::hideOrShow()
 {
@@ -410,12 +433,13 @@ void QStatusBar::hideOrShow()
 	item = d->items.next();
     }
 
+    emit messageChanged( d->tempItem );
     repaint();
 }
 
 
 /*!
-  Shows the temporary message, if appropriate.
+    Shows the temporary message, if appropriate.
 */
 void QStatusBar::paintEvent( QPaintEvent * )
 {
@@ -425,9 +449,9 @@ void QStatusBar::paintEvent( QPaintEvent * )
     QStatusBarPrivate::SBItem* item = d->items.first();
 
 #ifndef QT_NO_SIZEGRIP
-    int psx = d->resizer ? d->resizer->x() : width()-12;
+    int psx = ( d->resizer && d->resizer->isVisible() ) ? d->resizer->x() : width()-12;
 #else
-    int psx = width()-12;
+    int psx = width() - 12;
 #endif
 
     while ( item ) {
@@ -438,7 +462,8 @@ void QStatusBar::paintEvent( QPaintEvent * )
 		style().drawPrimitive( QStyle::PE_StatusBarSection, &p,
 				       QRect(item->w->x() - 1, item->w->y() - 1,
 					     item->w->width()+2, item->w->height()+2),
-				       colorGroup() );
+				       colorGroup(), QStyle::Style_Default,
+				       QStyleOption(item->w) );
 	    }
 	item = d->items.next();
     }
@@ -448,7 +473,8 @@ void QStatusBar::paintEvent( QPaintEvent * )
     }
 }
 
-/*! \reimp
+/*!
+    \reimp
 */
 void QStatusBar::resizeEvent( QResizeEvent * e )
 {
@@ -456,13 +482,33 @@ void QStatusBar::resizeEvent( QResizeEvent * e )
 }
 
 /*!
-  \reimp
+    \reimp
 */
 
 bool QStatusBar::event( QEvent *e )
 {
-    if ( e->type() == QEvent::LayoutHint )
-	update();
+    if ( e->type() == QEvent::LayoutHint ) {
+	// Calculate new strut height and call reformat() if it has changed
+	int maxH = fontMetrics().height();
+
+	QStatusBarPrivate::SBItem* item = d->items.first();
+	while ( item ) {
+	    int itemH = QMIN(item->w->sizeHint().height(),
+			    item->w->maximumHeight());
+	    maxH = QMAX( maxH, itemH );
+	    item = d->items.next();
+	}
+
+#ifndef QT_NO_SIZEGRIP
+	if ( d->resizer )
+	    maxH = QMAX( maxH, d->resizer->sizeHint().height() );
+#endif
+
+	if ( maxH != d->savedStrut )
+	    reformat();
+	else
+	    update();
+    }
     if ( e->type() == QEvent::ChildRemoved ) {
 	QStatusBarPrivate::SBItem* item = d->items.first();
 	while ( item ) {
