@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qtoolbar.h   3.0.5   edited Oct 12 2001 $
+** $Id: qt/qtoolbar.h   3.1.2   edited Nov 7 16:55 $
 **
 ** Definition of QToolBar class
 **
@@ -90,9 +90,7 @@ protected:
     void styleChange( QStyle & );
 
 private slots:
-    void popupSelected( int id );
-    void emulateButtonClicked();
-    void setupArrowMenu();
+    void createPopup();
 
 private:
     void init();
