@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qwidgetresizehandler.cpp   3.0.5   edited Feb 12 12:06 $
+** $Id: qt/qwidgetresizehandler.cpp   3.1.2   edited Feb 3 10:42 $
 **
 ** Implementation of the QWidgetResizeHandler class
 **
@@ -77,10 +77,20 @@ bool QWidgetResizeHandler::eventFilter( QObject *o, QEvent *ee )
     if ( !active || !o->isWidgetType() )
 	return FALSE;
 
-    QWidget *w = childOf( widget, (QWidget*)o );
-    if ( !w || o->inherits( "QSizeGrip" ) )
+    if ( ee->type() != QEvent::MouseButtonPress &&
+	 ee->type() != QEvent::MouseButtonRelease &&
+	 ee->type() != QEvent::MouseMove &&
+	 ee->type() != QEvent::KeyPress &&
+	 ee->type() != QEvent::AccelOverride )
 	return FALSE;
 
+    QWidget *w = childOf( widget, (QWidget*)o );
+    if ( !w || o->inherits("QSizeGrip") || qApp->activePopupWidget() ) {
+	if ( buttonDown && ee->type() == QEvent::MouseButtonRelease )
+	    buttonDown = FALSE;
+	return FALSE;
+    }
+
     QMouseEvent *e = (QMouseEvent*)ee;
     switch ( e->type() ) {
     case QEvent::MouseButtonPress: {
@@ -259,22 +269,13 @@ void QWidgetResizeHandler::mouseMoveEvent( QMouseEvent *e )
 
 #if defined(Q_WS_WIN)
     MSG msg;
-#if defined(UNICODE)
-#  ifndef Q_OS_TEMP
-    if ( qWinVersion() & WV_NT_based ) {
-#  endif
+    QT_WA( {
 	while(PeekMessageW( &msg, widget->winId(), WM_MOUSEMOVE, WM_MOUSEMOVE, PM_REMOVE ))
 	    ;
-#  ifndef Q_OS_TEMP
-    } else
-#  endif
-#endif
-#ifndef Q_OS_TEMP
-    {
+    } , {
 	while(PeekMessageA( &msg, widget->winId(), WM_MOUSEMOVE, WM_MOUSEMOVE, PM_REMOVE ))
 	    ;
-    }
-#endif
+    } );
 #endif
 
     QApplication::syncX();
