@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id$
+** $Id: qt/qsvgdevice.cpp   3.1.2   edited Feb 18 15:41 $
 **
 ** Implementation of the QSvgDevice class
 **
@@ -59,25 +59,19 @@ const char systemId[] = "http://www.w3.org/TR/2000/CR-SVG-20001102/DTD/svg-20001
 struct QM_EXPORT_SVG ImgElement {
     QDomElement element;
     QImage image;
-#if defined(Q_FULL_TEMPLATE_INSTANTIATION)
-    bool operator==( const ImgElement& ) const { return FALSE; }
-#endif
+    Q_DUMMY_COMPARISON_OPERATOR( ImgElement )
 };
 
 struct QM_EXPORT_SVG PixElement {
     QDomElement element;
     QPixmap pixmap;
-#if defined(Q_FULL_TEMPLATE_INSTANTIATION)
-    bool operator==( const PixElement& ) const { return FALSE; }
-#endif
+    Q_DUMMY_COMPARISON_OPERATOR( PixElement )
 };
 
 struct QSvgDeviceState {
     int textx, texty;			// current text position
     int textalign;			// text alignment
-#if defined(Q_FULL_TEMPLATE_INSTANTIATION)
-    bool operator==( const QSvgDeviceState& ) const { return FALSE; }
-#endif
+    Q_DUMMY_COMPARISON_OPERATOR( QSvgDeviceState )
 };
 
 typedef QValueList<ImgElement> ImageList;
@@ -116,24 +110,24 @@ static QSvgTypeMap *qSvgTypeMap=0; // element types
 static QMap<QString,QString> *qSvgColMap=0; // recognized color keyword names
 
 /*!
-  \class QSvgDevice qsvgdevice.h
-  \brief The QSvgDevice class provides a paint device for SVG vector graphics.
+    \class QSvgDevice qsvgdevice.h
+    \brief The QSvgDevice class provides a paint device for SVG vector graphics.
 
-  \ingroup xml-tools
-  \module XML
-  \internal
+    \ingroup xml-tools
+    \module XML
+    \internal
 
-  SVG is an XML vector graphics format. This class supports the
-  loading and saving of SVG files with load() and save(), and the
-  rendering of an SVG onto a QPainter using play(). Use
-  toString() to put the SVG into a string.
+    SVG is an XML vector graphics format. This class supports the
+    loading and saving of SVG files with load() and save(), and the
+    rendering of an SVG onto a QPainter using play(). Use toString()
+    to put the SVG into a string.
 
-  \sa QPaintDevice QPainter
+    \sa QPaintDevice QPainter
 */
 
 /*!
-  Creates a QSvgDevice object.
- */
+    Creates a QSvgDevice object.
+*/
 
 QSvgDevice::QSvgDevice()
     : QPaintDevice( QInternal::ExternalDevice ),
@@ -143,7 +137,7 @@ QSvgDevice::QSvgDevice()
 }
 
 /*!
-  Destroys the QSvgDevice object and frees the resources it used.
+    Destroys the QSvgDevice object and frees the resources it used.
 */
 
 QSvgDevice::~QSvgDevice()
@@ -154,8 +148,9 @@ QSvgDevice::~QSvgDevice()
 }
 
 /*!
-  Loads and parses a graphic from \a dev into the device. Returns TRUE on
-  success, or FALSE if errors were encountered.
+    Loads and parses a SVG from \a dev into the device. Returns TRUE
+    on success (i.e. loaded and parsed without error); otherwise
+    returns FALSE.
 */
 
 bool QSvgDevice::load( QIODevice *dev )
@@ -164,8 +159,8 @@ bool QSvgDevice::load( QIODevice *dev )
 }
 
 /*!
-  Renders (replays) the graphic on the \a painter and returns TRUE if
-  successful, or FALSE if the document format is not valid.
+    Renders (replays) the SVG on the \a painter and returns TRUE if
+    successful (i.e. it is a valid SVG); otherwise returns FALSE.
 */
 
 bool QSvgDevice::play( QPainter *painter )
@@ -195,16 +190,20 @@ bool QSvgDevice::play( QPainter *painter )
     painter->setWorldXForm( TRUE );
 
     QDomNamedNodeMap attr = svg.attributes();
+    int x = lenToInt( attr, "x" );
+    int y = lenToInt( attr, "y" );
+    brect.setX( x );
+    brect.setY( y );
     QString wstr = attr.contains( "width" )
 		   ? attr.namedItem( "width" ).nodeValue() : QString( "100%" );
     QString hstr = attr.contains( "height" )
 		   ? attr.namedItem( "height" ).nodeValue() : QString( "100%" );
     double width = parseLen( wstr, 0, TRUE );
     double height = parseLen( hstr, 0, FALSE );
-    brect.setWidth( int(width) );
-    brect.setHeight( int(height) );
-    painter->setClipRect( 0, 0, int(width), int(height),
-			  QPainter::CoordPainter );
+    // SVG doesn't respect x and y. But we want a proper bounding rect.
+    brect.setWidth( int(width) - x );
+    brect.setHeight( int(height) - y );
+    painter->setClipRect( brect, QPainter::CoordPainter );
 
     if ( attr.contains( "viewBox" ) ) {
 	QRegExp re( "\\s*(\\S+)\\s*,?\\s*(\\S+)\\s*,?"
@@ -284,8 +283,8 @@ QString QSvgDevice::toString() const
 }
 
 /*!
-  Saves the SVG vector graphics to \a fileName.
- */
+    Saves the SVG to \a fileName.
+*/
 
 bool QSvgDevice::save( const QString &fileName )
 {
@@ -296,22 +295,25 @@ bool QSvgDevice::save( const QString &fileName )
     // now we have the info about name and dimensions available
     QDomElement root = doc.documentElement();
     root.setAttribute( "id", svgName );
+    // the standard doesn't take respect x and y. But we want a
+    // proper bounding rect. We make width and height bigger when
+    // writing out and subtract x and y when reading in.
     root.setAttribute( "x", brect.x() );
     root.setAttribute( "y", brect.y() );
-    root.setAttribute( "width", brect.width() );
-    root.setAttribute( "height", brect.height() );
+    root.setAttribute( "width", brect.width() + brect.x() );
+    root.setAttribute( "height", brect.height() + brect.y() );
 
     // ... and know how to name any image files to be written out
     int icount = 0;
     ImageList::Iterator iit = d->images.begin();
-    for ( ; iit != d->images.end(); iit++ ) {
+    for ( ; iit != d->images.end(); ++iit ) {
 	QString href = QString( "%1_%2.png" ).arg( svgName ).arg( icount );
 	(*iit).image.save( href, "PNG" );
 	(*iit).element.setAttribute( "xlink:href", href );
 	icount++;
     }
     PixmapList::Iterator pit = d->pixmaps.begin();
-    for ( ; pit != d->pixmaps.end(); pit++ ) {
+    for ( ; pit != d->pixmaps.end(); ++pit ) {
 	QString href = QString( "%1_%2.png" ).arg( svgName ).arg( icount );
 	(*pit).pixmap.save( href, "PNG" );
 	(*pit).element.setAttribute( "xlink:href", href );
@@ -329,9 +331,9 @@ bool QSvgDevice::save( const QString &fileName )
 }
 
 /*!
-  \overload
+    \overload
 
-  \a dev is the device to use for saving.
+    \a dev is the device to use for saving.
 */
 
 bool QSvgDevice::save( QIODevice *dev )
@@ -349,14 +351,14 @@ bool QSvgDevice::save( QIODevice *dev )
 }
 
 /*!
-  \fn QRect QSvgDevice::boundingRect() const
+    \fn QRect QSvgDevice::boundingRect() const
 
-  Returns the bounding rectangle of the vector graphic.
- */
+    Returns the bounding rectangle of the SVG.
+*/
 
 /*!
-  Sets the bounding rectangle of the vector graphic to rectangle \a r.
- */
+    Sets the bounding rectangle of the SVG to rectangle \a r.
+*/
 
 void QSvgDevice::setBoundingRect( const QRect &r )
 {
@@ -364,13 +366,13 @@ void QSvgDevice::setBoundingRect( const QRect &r )
 }
 
 /*!
-  Internal implementation of the virtual QPaintDevice::metric() function.
+    Internal implementation of the virtual QPaintDevice::metric()
+    function.
 
-  Use the QPaintDeviceMetrics class instead.
+    \warning Use the QPaintDeviceMetrics class instead.
 
-  A QSvgDevice has the following hard coded values:
-  dpi = 72, numcolors=16777216 and depth=24.
-  \a m is the metric to get.
+    A QSvgDevice has the following hard coded values: dpi=72,
+    numcolors=16777216 and depth=24. \a m is the metric to get.
 */
 
 int QSvgDevice::metric( int m ) const
@@ -411,9 +413,10 @@ int QSvgDevice::metric( int m ) const
 }
 
 /*!
-  \internal
-  Records painter commands and stores them in the QDomDocument doc.
- */
+    \internal
+
+    Records painter commands and stores them in the QDomDocument doc.
+*/
 
 bool QSvgDevice::cmd ( int c, QPainter *painter, QPDevCmdParam *p )
 {
@@ -501,19 +504,19 @@ bool QSvgDevice::cmd ( int c, QPainter *painter, QPDevCmdParam *p )
 	double ry = rect.height() / 2.0;
 	double x0 = (double)rect.x() + rx;
 	double y0 = (double)rect.y() + ry;
-	double x1 = x0 + rx*(1+cos(a));
-	double y1 = y0 + ry*(1-sin(a));
-	double x2 = x0 + rx*(1+cos(a+al));
-	double y2 = y0 + ry*(1-sin(a+al));
-	int large = al > 180.0 ? 1 : 0;
-	int sweep = al > 0.0 ? 1 : 0;
-	if ( c == PdcDrawPie ) {
-	    str = QString( "M %1 %2 " ).arg( x0 ).arg( y0 );
-	    str += QString( "L %1 %2 " ).arg( x1 ).arg( y1 );
-	} else
+	double x1 = x0 + rx*cos(a);
+	double y1 = y0 - ry*sin(a);
+	double x2 = x0 + rx*cos(a+al);
+	double y2 = y0 - ry*sin(a+al);
+	int large = QABS( al ) > ( 180.0 * deg2rad ) ? 1 : 0;
+	int sweep = al < 0.0 ? 1 : 0;
+	if ( c == PdcDrawPie )
+	    str = QString( "M %1 %2 L %3 %4 " ).arg( x0 ).arg( y0 )
+		  .arg( x1 ).arg( y1 );
+	else
 	    str = QString( "M %1 %2 " ).arg( x1 ).arg( y1 );
-	str += QString( "A %1 %2 0 %3 %4 %5 %6" )
-	       .arg( rx ).arg( ry ).arg( large ).arg( sweep )
+	str += QString( "A %1 %2 %3 %4 %5 %6 %7" )
+	       .arg( rx ).arg( ry ).arg( a/deg2rad ). arg( large ).arg( sweep )
 	       .arg( x2 ).arg( y2 );
 	if ( c != PdcDrawArc )
 	    str += "z";
@@ -563,14 +566,34 @@ bool QSvgDevice::cmd ( int c, QPainter *painter, QPDevCmdParam *p )
 	    e.setAttribute( "y", p[0].point->y() );
 	e.appendChild( doc.createTextNode( *p[1].str ) );
 	break;
-    case PdcDrawText2Formatted:
+    case PdcDrawText2Formatted: {
 	e = doc.createElement( "text" );
-	if ( p[0].rect->x() )
-	    e.setAttribute( "x", p[0].rect->x() );
-	if ( p[0].point->y() )
-	    e.setAttribute( "y", p[0].rect->y()+painter->fontMetrics().ascent() );
-	// ### int tf = p[1].ival;
+	const QRect *r = p[0].rect;
+	int tf = p[1].ival;
+	int x, y;
+	// horizontal text alignment
+	if ( ( tf & Qt::AlignHCenter ) != 0 ) {
+	    x = r->x() + r->width() / 2;
+	    e.setAttribute( "text-anchor", "middle" );
+	} else if ( ( tf & Qt::AlignRight ) != 0 ) {
+	    x = r->right();
+	    e.setAttribute( "text-anchor", "end" );
+	} else {
+	    x = r->x();
+	}
+	// vertical text alignment
+	if ( ( tf & Qt::AlignVCenter ) != 0 )
+	    y = r->y() + ( r->height() + painter->fontMetrics().ascent() ) / 2;
+	else if ( ( tf & Qt::AlignBottom ) != 0 )
+	    y = r->bottom();
+	else
+	    y = r->y() + painter->fontMetrics().ascent();
+	if ( x )
+	    e.setAttribute( "x", x );
+	if ( y )
+	    e.setAttribute( "y", y );
 	e.appendChild( doc.createTextNode( *p[2].str ) );
+    }
 	break;
     case PdcDrawPixmap:
     case PdcDrawImage:
@@ -652,9 +675,11 @@ bool QSvgDevice::cmd ( int c, QPainter *painter, QPDevCmdParam *p )
 }
 
 /*!
-  \internal
-  Push the current drawing attributes on a stack.
-  \sa restoreAttributes()
+    \internal
+
+    Push the current drawing attributes on a stack.
+
+    \sa restoreAttributes()
 */
 
 void QSvgDevice::saveAttributes()
@@ -667,9 +692,11 @@ void QSvgDevice::saveAttributes()
 }
 
 /*!
-  \internal
-  Pop the current drawing attributes off the stack.
-  \sa saveAttributes()
+    \internal
+
+    Pop the current drawing attributes off the stack.
+
+    \sa saveAttributes()
 */
 
 void QSvgDevice::restoreAttributes()
@@ -681,8 +708,9 @@ void QSvgDevice::restoreAttributes()
 }
 
 /*!
-  \internal
-  Evaluate \a node, drawing on \a p. Allows recursive calls.
+    \internal
+
+    Evaluate \a node, drawing on \a p. Allows recursive calls.
 */
 
 bool QSvgDevice::play( const QDomNode &node )
@@ -882,10 +910,11 @@ bool QSvgDevice::play( const QDomNode &node )
 }
 
 /*!
-  \internal
-  Parses a CSS2-compatible color specification. Either a keyword or a numerical
-  RGB specification like #ff00ff or rgb(255,0,50%).
- */
+    \internal
+
+    Parses a CSS2-compatible color specification. Either a keyword or
+    a numerical RGB specification like #ff00ff or rgb(255,0,50%).
+*/
 
 QColor QSvgDevice::parseColor( const QString &col )
 {
@@ -945,11 +974,12 @@ QColor QSvgDevice::parseColor( const QString &col )
 }
 
 /*!
-  \internal
-  Parse a <length> datatype consisting of a number followed by an optional
-  unit specifier. Can be used for type <coordinate> as well. For relative
-  units the value of \a horiz will determine whether the horizontal or
-  vertical dimension will be used.
+    \internal
+
+    Parse a <length> datatype consisting of a number followed by an
+    optional unit specifier. Can be used for type <coordinate> as
+    well. For relative units the value of \a horiz will determine
+    whether the horizontal or vertical dimension will be used.
 */
 
 double QSvgDevice::parseLen( const QString &str, bool *ok, bool horiz ) const
@@ -962,40 +992,42 @@ double QSvgDevice::parseLen( const QString &str, bool *ok, bool horiz ) const
 	return 0.0;
     }
 
-    double d = reg.cap( 1 ).toDouble();
+    double dbl = reg.cap( 1 ).toDouble();
     QString u = reg.cap( 2 );
     if ( !u.isEmpty() && u != "px" ) {
 	QPaintDeviceMetrics m( pt->device() );
 	if ( u == "em" ) {
 	    QFontInfo fi( pt->font() );
-	    d *= fi.pixelSize();
+	    dbl *= fi.pixelSize();
 	} else if ( u == "ex" ) {
 	    QFontInfo fi( pt->font() );
-	    d *= 0.5 * fi.pixelSize();
+	    dbl *= 0.5 * fi.pixelSize();
 	} else if ( u == "%" )
-	    d *= (horiz ? pt->window().width() : pt->window().height())/100.0;
+	    dbl *= (horiz ? pt->window().width() : pt->window().height())/100.0;
 	else if ( u == "cm" )
-	    d *= m.logicalDpiX() / 2.54;
+	    dbl *= m.logicalDpiX() / 2.54;
 	else if ( u == "mm" )
-	    d *= m.logicalDpiX() / 25.4;
+	    dbl *= m.logicalDpiX() / 25.4;
 	else if ( u == "in" )
-	    d *= m.logicalDpiX();
+	    dbl *= m.logicalDpiX();
 	else if ( u == "pt" )
-	    d *= m.logicalDpiX() / 72.0;
+	    dbl *= m.logicalDpiX() / 72.0;
 	else if ( u == "pc" )
-	    d *= m.logicalDpiX() / 6.0;
+	    dbl *= m.logicalDpiX() / 6.0;
 	else
 	    qWarning( "QSvgDevice::parseLen: Unknown unit " + u );
     }
     if ( ok )
 	*ok = TRUE;
-    return d;
+    return dbl;
 }
 
 /*!
-  \internal
-  Returns the length specified in attribute \a attr in \a map. If the
-  specified attribute doesn't exist or can't be parsed \a def is returned.
+    \internal
+
+    Returns the length specified in attribute \a attr in \a map. If
+    the specified attribute doesn't exist or can't be parsed \a def is
+    returned.
 */
 
 int QSvgDevice::lenToInt( const QDomNamedNodeMap &map, const QString &attr,
@@ -1003,9 +1035,9 @@ int QSvgDevice::lenToInt( const QDomNamedNodeMap &map, const QString &attr,
 {
     if ( map.contains( attr ) ) {
 	bool ok;
-	double d = parseLen( map.namedItem( attr ).nodeValue(), &ok );
+	double dbl = parseLen( map.namedItem( attr ).nodeValue(), &ok );
 	if ( ok )
-	    return qRound( d );
+	    return qRound( dbl );
     }
     return def;
 }
@@ -1086,7 +1118,7 @@ void QSvgDevice::setStyleProperty( const QString &prop, const QString &val,
 
 void QSvgDevice::setStyle( const QString &s )
 {
-    QStringList rules = QStringList::split( QRegExp( ";" ), s );
+    QStringList rules = QStringList::split( QChar(';'), s );
 
     QPen pen = pt->pen();
     QFont font = pt->font();
@@ -1238,10 +1270,10 @@ void QSvgDevice::drawPath( const QString &data )
 	case 8: {				// 'T' smooth shorthand
 	    quad.setPoint( 0, int(x), int(y) );
 	    // if possible, reflect last control point if smooth shorthand
-	    if ( mode == 'S' || mode == 'T' ) {
+	    if ( mode == 6 || mode == 8 ) { 	// smooth 'S' and 'T'
 		bool cont = mode == lastMode ||
-		     mode == 'S' && lastMode == 'C' ||
-		     mode == 'T' && lastMode == 'Q';
+		     mode == 6 && lastMode == 5 || 	// 'S' and 'C'
+		     mode == 8 && lastMode == 7;	// 'T' and 'Q'
 		x = cont ? 2*x-controlX : x;
 		y = cont ? 2*y-controlY : y;
 		quad.setPoint( 1, int(x), int(y) );
@@ -1256,7 +1288,7 @@ void QSvgDevice::drawPath( const QString &data )
 	    controlX = quad[ 2 ].x();
 	    controlY = quad[ 2 ].y();
 	    // transform quadratic into cubic Bezier
-	    if ( mode == 'Q' || mode == 'T' ) {
+	    if ( mode == 7 || mode == 8 ) {	// cubic 'Q' and 'T'
 		int x31 = quad[0].x()+int(2.0*(quad[2].x()-quad[0].x())/3.0);
 		int y31 = quad[0].y()+int(2.0*(quad[2].y()-quad[0].y())/3.0);
 		int x32 = quad[2].x()+int(2.0*(quad[3].x()-quad[2].x())/3.0);
@@ -1322,7 +1354,9 @@ void QSvgDevice::applyStyle( QDomElement *e, int c ) const
 	     .arg( pcol.red() ).arg( pcol.green() ).arg( pcol.blue() );
 	s += QString( "stroke-width:0;" );
 	QFont f = pt->font();
-	s += QString( "font-size:%1;" ).arg( f.pointSize() );
+	QFontInfo fi( f );
+	s += QString( "font-family:%1;" ).arg( fi.family() );
+	s += QString( "font-size:%1;" ).arg( fi.pixelSize() );
 	s += QString( "font-style:%1;" )
 	     .arg( f.italic() ? "italic" : "normal" );
 	// not a very scientific distribution
@@ -1349,8 +1383,9 @@ void QSvgDevice::applyStyle( QDomElement *e, int c ) const
 	if ( pw == 0 && pt->pen().style() != Qt::NoPen )
 	    pw = 0.9;
 	s += QString( "stroke-width:%1;" ).arg( pw );
-	if ( pt->brush().style() == Qt::NoBrush || c == PdcDrawPolyline )
-	    s += "fill:none;"; // Qt polylines use no brush
+	if ( pt->brush().style() == Qt::NoBrush || c == PdcDrawPolyline ||
+	     c == PdcDrawCubicBezier )
+	    s += "fill:none;"; // Qt polylines use no brush, neither do Beziers
 	else
 	    s += QString( "fill:rgb(%1,%2,%3);" )
 		 .arg( bcol.red() ).arg( bcol.green() ).arg( bcol.blue() );
